@@ -17,49 +17,50 @@ use Piwik\PluginDeactivatedException;
 use Piwik\SettingsServer;
 use Piwik\Url;
 use Piwik\UrlHelper;
+use Piwik\Log;
 
 /**
  * Dispatches API requests to the appropriate API method.
- * 
+ *
  * The Request class is used throughout Piwik to call API methods. The difference
  * between using Request and calling API methods directly is that Request
  * will do more after calling the API including: applying generic filters, applying queued filters,
  * and handling the **flat** and **label** query parameters.
- * 
+ *
  * Additionally, the Request class will **forward current query parameters** to the request
  * which is more convenient than calling {@link Piwik\Common::getRequestVar()} many times over.
- * 
+ *
  * In most cases, using a Request object to query the API is the correct approach.
  *
  * ### Post-processing
- * 
+ *
  * The return value of API methods undergo some extra processing before being returned by Request.
  * To learn more about what happens to API results, read [this](/guides/piwiks-web-api#extra-report-processing).
  *
  * ### Output Formats
- * 
+ *
  * The value returned by Request will be serialized to a certain format before being returned.
  * To see the list of supported output formats, read [this](/guides/piwiks-web-api#output-formats).
- * 
+ *
  * ### Examples
- * 
+ *
  * **Basic Usage**
- * 
- *     $request = new Request('method=UserSettings.getWideScreen&idSite=1&date=yesterday&period=week'
+ *
+ *     $request = new Request('method=UserSettings.getLanguage&idSite=1&date=yesterday&period=week'
  *                          . '&format=xml&filter_limit=5&filter_offset=0')
  *     $result = $request->process();
  *     echo $result;
- * 
+ *
  * **Getting a unrendered DataTable**
- * 
+ *
  *     // use the convenience method 'processRequest'
- *     $dataTable = Request::processRequest('UserSettings.getWideScreen', array(
+ *     $dataTable = Request::processRequest('UserSettings.getLanguage', array(
  *         'idSite' => 1,
  *         'date' => 'yesterday',
  *         'period' => 'week',
  *         'filter_limit' => 5,
  *         'filter_offset' => 0
- * 
+ *
  *         'format' => 'original', // this is the important bit
  *     ));
  *     echo "This DataTable has " . $dataTable->getRowsCount() . " rows.";
@@ -69,7 +70,7 @@ use Piwik\UrlHelper;
  */
 class Request
 {
-    protected $request = null;
+    private $request = null;
 
     /**
      * Converts the supplied request string into an array of query paramater name/value
@@ -77,33 +78,38 @@ class Request
      * forwarded to request array before it is returned.
      *
      * @param string|array $request The base request string or array, eg,
-     *                              `'module=UserSettings&action=getWidescreen'`.
+     *                              `'module=UserSettings&action=getLanguage'`.
+     * @param array $defaultRequest Default query parameters. If a query parameter is absent in `$request`, it will be loaded
+     *                              from this. Defaults to `$_GET + $_POST`.
      * @return array
      */
-    static public function getRequestArrayFromString($request)
+    public static function getRequestArrayFromString($request, $defaultRequest = null)
     {
-        $defaultRequest = $_GET + $_POST;
+        if ($defaultRequest === null) {
+            $defaultRequest = $_GET + $_POST;
 
-        $requestRaw = self::getRequestParametersGET();
-        if (!empty($requestRaw['segment'])) {
-            $defaultRequest['segment'] = $requestRaw['segment'];
+            $requestRaw = self::getRequestParametersGET();
+            if (!empty($requestRaw['segment'])) {
+                $defaultRequest['segment'] = $requestRaw['segment'];
+            }
+
+            if (empty($defaultRequest['format_metrics'])) {
+                $defaultRequest['format_metrics'] = 'bc';
+            }
         }
 
         $requestArray = $defaultRequest;
 
         if (!is_null($request)) {
             if (is_array($request)) {
-                $url = array();
-                foreach ($request as $key => $value) {
-                    $url[] = $key . "=" . $value;
-                }
-                $request = implode("&", $url);
+                $requestParsed = $request;
+            } else {
+                $request = trim($request);
+                $request = str_replace(array("\n", "\t"), '', $request);
+
+                $requestParsed = UrlHelper::getArrayFromQueryString($request);
             }
 
-            $request = trim($request);
-            $request = str_replace(array("\n", "\t"), '', $request);
-
-            $requestParsed = UrlHelper::getArrayFromQueryString($request);
             $requestArray = $requestParsed + $defaultRequest;
         }
 
@@ -119,14 +125,17 @@ class Request
      * Constructor.
      *
      * @param string|array $request Query string that defines the API call (must at least contain a **method** parameter),
-     *                              eg, `'method=UserSettings.getWideScreen&idSite=1&date=yesterday&period=week&format=xml'`
+     *                              eg, `'method=UserSettings.getLanguage&idSite=1&date=yesterday&period=week&format=xml'`
      *                              If a request is not provided, then we use the values in the `$_GET` and `$_POST`
      *                              superglobals.
+     * @param array $defaultRequest Default query parameters. If a query parameter is absent in `$request`, it will be loaded
+     *                              from this. Defaults to `$_GET + $_POST`.
      */
-    public function __construct($request = null)
+    public function __construct($request = null, $defaultRequest = null)
     {
-        $this->request = self::getRequestArrayFromString($request);
+        $this->request = self::getRequestArrayFromString($request, $defaultRequest);
         $this->sanitizeRequest();
+        $this->renameModuleAndActionInRequest();
     }
 
     /**
@@ -134,19 +143,23 @@ class Request
      * we rewrite to correct renamed plugin: Referrers
      *
      * @param $module
-     * @return string
+     * @param $action
+     * @return array( $module, $action )
      * @ignore
      */
-    public static function renameModule($module)
+    public static function getRenamedModuleAndAction($module, $action)
     {
-        $moduleToRedirect = array(
-            'Referers'   => 'Referrers',
-            'PDFReports' => 'ScheduledReports',
-        );
-        if (isset($moduleToRedirect[$module])) {
-            return $moduleToRedirect[$module];
-        }
-        return $module;
+        /**
+         * This event is posted in the Request dispatcher and can be used
+         * to overwrite the Module and Action to dispatch.
+         * This is useful when some Controller methods or API methods have been renamed or moved to another plugin.
+         *
+         * @param $module string
+         * @param $action string
+         */
+        Piwik::postEvent('Request.getRenamedModuleAndAction', array(&$module, &$action));
+
+        return array($module, $action);
     }
 
     /**
@@ -168,9 +181,9 @@ class Request
     /**
      * Dispatches the API request to the appropriate API method and returns the result
      * after post-processing.
-     * 
+     *
      * Post-processing includes:
-     * 
+     *
      * - flattening if **flat** is 0
      * - running generic filters unless **disable_generic_filters** is set to 1
      * - URL decoding label column values
@@ -178,10 +191,10 @@ class Request
      * - removing columns based on the values of the **hideColumns** and **showColumns** query parameters
      * - filtering rows if the **label** query parameter is set
      * - converting the result to the appropriate format (ie, XML, JSON, etc.)
-     * 
+     *
      * If `'original'` is supplied for the output format, the result is returned as a PHP
      * object.
-     * 
+     *
      * @throws PluginDeactivatedException if the module plugin is not activated.
      * @throws Exception if the requested API method cannot be called, if required parameters for the
      *                   API method are missing or if the API method throws an exception and the **format**
@@ -196,13 +209,16 @@ class Request
         // create the response
         $response = new ResponseBuilder($outputFormat, $this->request);
 
+        $corsHandler = new CORSHandler();
+        $corsHandler->handle();
+
         try {
             // read parameters
             $moduleMethod = Common::getRequestVar('method', null, 'string', $this->request);
 
             list($module, $method) = $this->extractModuleAndMethod($moduleMethod);
 
-            $module = $this->renameModule($module);
+            list($module, $method) = $this->getRenamedModuleAndAction($module, $method);
 
             if (!\Piwik\Plugin\Manager::getInstance()->isPluginActivated($module)) {
                 throw new PluginDeactivatedException($module);
@@ -216,6 +232,8 @@ class Request
 
             $toReturn = $response->getResponse($returnedValue, $module, $method);
         } catch (Exception $e) {
+            Log::debug($e);
+
             $toReturn = $response->getResponseException($e);
         }
         return $toReturn;
@@ -223,11 +241,11 @@ class Request
 
     /**
      * Returns the name of a plugin's API class by plugin name.
-     * 
+     *
      * @param string $plugin The plugin name, eg, `'Referrers'`.
      * @return string The fully qualified API class name, eg, `'\Piwik\Plugins\Referrers\API'`.
      */
-    static public function getClassNameAPI($plugin)
+    public static function getClassNameAPI($plugin)
     {
         return sprintf('\Piwik\Plugins\%s\API', $plugin);
     }
@@ -241,7 +259,7 @@ class Request
      * @return void
      * @ignore
      */
-    static public function reloadAuthUsingTokenAuth($request = null)
+    public static function reloadAuthUsingTokenAuth($request = null)
     {
         // if a token_auth is specified in the API request, we load the right permissions
         $token_auth = Common::getRequestVar('token_auth', '', 'string', $request);
@@ -250,11 +268,11 @@ class Request
             /**
              * Triggered when authenticating an API request, but only if the **token_auth**
              * query parameter is found in the request.
-             * 
+             *
              * Plugins that provide authentication capabilities should subscribe to this event
              * and make sure the global authentication object (the object returned by `Registry::get('auth')`)
              * is setup to use `$token_auth` when its `authenticate()` method is executed.
-             * 
+             *
              * @param string $token_auth The value of the **token_auth** query parameter.
              */
             Piwik::postEvent('API.Request.authenticate', array($token_auth));
@@ -286,9 +304,13 @@ class Request
      * @param string $method The API method to call, ie, `'Actions.getPageTitles'`.
      * @param array $paramOverride The parameter name-value pairs to use instead of what's
      *                             in `$_GET` & `$_POST`.
+     * @param array $defaultRequest Default query parameters. If a query parameter is absent in `$request`, it will be loaded
+     *                              from this. Defaults to `$_GET + $_POST`.
+     *
+     *                              To avoid using any parameters from $_GET or $_POST, set this to an empty `array()`.
      * @return mixed The result of the API request. See {@link process()}.
      */
-    public static function processRequest($method, $paramOverride = array())
+    public static function processRequest($method, $paramOverride = array(), $defaultRequest = null)
     {
         $params = array();
         $params['format'] = 'original';
@@ -297,7 +319,7 @@ class Request
         $params = $paramOverride + $params;
 
         // process request
-        $request = new Request($params);
+        $request = new Request($params, $defaultRequest);
         return $request->process();
     }
 
@@ -305,7 +327,7 @@ class Request
      * Returns the original request parameters in the current query string as an array mapping
      * query parameter names with values. The result of this function will not be affected
      * by any modifications to `$_GET` and will not include parameters in `$_POST`.
-     * 
+     *
      * @return array
      */
     public static function getRequestParametersGET()
@@ -379,10 +401,10 @@ class Request
 
     /**
      * Returns the segment query parameter from the original request, without modifications.
-     * 
+     *
      * @return array|bool
      */
-    static public function getRawSegmentFromRequest()
+    public static function getRawSegmentFromRequest()
     {
         // we need the URL encoded segment parameter, we fetch it from _SERVER['QUERY_STRING'] instead of default URL decoded _GET
         $segmentRaw = false;
@@ -395,4 +417,15 @@ class Request
         }
         return $segmentRaw;
     }
+
+    private function renameModuleAndActionInRequest()
+    {
+        if (empty($this->request['apiModule'])) {
+            return;
+        }
+        if (empty($this->request['apiAction'])) {
+            $this->request['apiAction'] = null;
+        }
+        list($this->request['apiModule'], $this->request['apiAction']) = $this->getRenamedModuleAndAction($this->request['apiModule'], $this->request['apiAction']);
+    }
 }
