@@ -10,55 +10,56 @@ namespace Piwik;
 
 use Piwik\Archive\Parameters;
 use Piwik\ArchiveProcessor\Rules;
+use Piwik\DataAccess\ArchiveInvalidator;
 use Piwik\DataAccess\ArchiveSelector;
-use Piwik\Period\Factory;
+use Piwik\Period\Factory as PeriodFactory;
 
 /**
  * The **Archive** class is used to query cached analytics statistics
  * (termed "archive data").
- * 
+ *
  * You can use **Archive** instances to get data that was archived for one or more sites,
  * for one or more periods and one optional segment.
- * 
+ *
  * If archive data is not found, this class will initiate the archiving process. [1](#footnote-1)
- * 
+ *
  * **Archive** instances must be created using the {@link build()} factory method;
  * they cannot be constructed.
- * 
+ *
  * You can search for metrics (such as `nb_visits`) using the {@link getNumeric()} and
  * {@link getDataTableFromNumeric()} methods. You can search for
  * reports using the {@link getBlob()}, {@link getDataTable()} and {@link getDataTableExpanded()} methods.
- * 
+ *
  * If you're creating an API that returns report data, you may want to use the
  * {@link getDataTableFromArchive()} helper function.
- * 
+ *
  * ### Learn more
- * 
+ *
  * Learn more about _archiving_ [here](/guides/all-about-analytics-data).
- * 
+ *
  * ### Limitations
- * 
+ *
  * - You cannot get data for multiple range periods in a single query.
  * - You cannot get data for periods of different types in a single query.
- * 
+ *
  * ### Examples
- * 
+ *
  * **_Querying metrics for an API method_**
- * 
+ *
  *     // one site and one period
  *     $archive = Archive::build($idSite = 1, $period = 'week', $date = '2013-03-08');
  *     return $archive->getDataTableFromNumeric(array('nb_visits', 'nb_actions'));
- *     
+ *
  *     // all sites and multiple dates
  *     $archive = Archive::build($idSite = 'all', $period = 'month', $date = '2013-01-02,2013-03-08');
  *     return $archive->getDataTableFromNumeric(array('nb_visits', 'nb_actions'));
- * 
+ *
  * **_Querying and using metrics immediately_**
- * 
+ *
  *     // one site and one period
  *     $archive = Archive::build($idSite = 1, $period = 'week', $date = '2013-03-08');
  *     $data = $archive->getNumeric(array('nb_visits', 'nb_actions'));
- *     
+ *
  *     $visits = $data['nb_visits'];
  *     $actions = $data['nb_actions'];
  *
@@ -67,37 +68,37 @@ use Piwik\Period\Factory;
  *     // multiple sites and multiple dates
  *     $archive = Archive::build($idSite = '1,2,3', $period = 'month', $date = '2013-01-02,2013-03-08');
  *     $data = $archive->getNumeric('nb_visits');
- *     
+ *
  *     $janSite1Visits = $data['1']['2013-01-01,2013-01-31']['nb_visits'];
  *     $febSite1Visits = $data['1']['2013-02-01,2013-02-28']['nb_visits'];
  *     // ... etc.
- *     
+ *
  * **_Querying for reports_**
- * 
+ *
  *     $archive = Archive::build($idSite = 1, $period = 'week', $date = '2013-03-08');
  *     $dataTable = $archive->getDataTable('MyPlugin_MyReport');
  *     // ... manipulate $dataTable ...
  *     return $dataTable;
- * 
+ *
  * **_Querying a report for an API method_**
- * 
+ *
  *     public function getMyReport($idSite, $period, $date, $segment = false, $expanded = false)
  *     {
  *         $dataTable = Archive::getDataTableFromArchive('MyPlugin_MyReport', $idSite, $period, $date, $segment, $expanded);
  *         $dataTable->queueFilter('ReplaceColumnNames');
  *         return $dataTable;
  *     }
- * 
+ *
  * **_Querying data for multiple range periods_**
- * 
+ *
  *     // get data for first range
  *     $archive = Archive::build($idSite = 1, $period = 'range', $date = '2013-03-08,2013-03-12');
  *     $dataTable = $archive->getDataTableFromNumeric(array('nb_visits', 'nb_actions'));
- *     
+ *
  *     // get data for second range
  *     $archive = Archive::build($idSite = 1, $period = 'range', $date = '2013-03-15,2013-03-20');
  *     $dataTable = $archive->getDataTableFromNumeric(array('nb_visits', 'nb_actions'));
- * 
+ *
  * <a name="footnote-1"></a>
  * [1]: The archiving process will not be launched if browser archiving is disabled
  *      and the current request came from a browser.
@@ -160,6 +161,11 @@ class Archive
      */
     private $params;
 
+    /**
+     * @var \Piwik\Cache\Cache
+     */
+    private static $cache;
+
     /**
      * @param Parameters $params
      * @param bool $forceIndexedBySite Whether to force index the result of a query by site ID.
@@ -197,31 +203,36 @@ class Archive
     {
         $websiteIds = Site::getIdSitesFromIdSitesString($idSites, $_restrictSitesToLogin);
 
-        $timezone = count($websiteIds) == 1 ? Site::getTimezoneFor($websiteIds[0]) : false;
+        $timezone = false;
+        if (count($websiteIds) == 1) {
+            $timezone = Site::getTimezoneFor($websiteIds[0]);
+        }
 
         if (Period::isMultiplePeriod($strDate, $period)) {
-            $oPeriod = Factory::build($period, $strDate, $timezone);
+            $oPeriod    = PeriodFactory::build($period, $strDate, $timezone);
             $allPeriods = $oPeriod->getSubperiods();
         } else {
-            $oPeriod = Factory::makePeriodFromQueryParams($timezone, $period, $strDate);
+            $oPeriod    = PeriodFactory::makePeriodFromQueryParams($timezone, $period, $strDate);
             $allPeriods = array($oPeriod);
         }
-        $segment = new Segment($segment, $websiteIds);
-        $idSiteIsAll = $idSites == self::REQUEST_ALL_WEBSITES_FLAG;
+
+        $segment        = new Segment($segment, $websiteIds);
+        $idSiteIsAll    = $idSites == self::REQUEST_ALL_WEBSITES_FLAG;
         $isMultipleDate = Period::isMultiplePeriod($strDate, $period);
+
         return Archive::factory($segment, $allPeriods, $websiteIds, $idSiteIsAll, $isMultipleDate, $skipAggregationOfSubTables);
     }
 
     /**
      * Returns a new Archive instance that will query archive data for the given set of
      * sites and periods, using an optional segment.
-     * 
+     *
      * This method uses an array of Period instances and a Segment instance, instead of strings
      * like {@link build()}.
-     * 
+     *
      * If you want to create an Archive instance using data found in query parameters,
      * use {@link build()}.
-     * 
+     *
      * @param Segment $segment The segment to use. For no segment, use `new Segment('', $idSites)`.
      * @param array $periods An array of Period instances.
      * @param array $idSites An array of site IDs (eg, `array(1, 2, 3)`).
@@ -239,9 +250,11 @@ class Archive
     {
         $forceIndexedBySite = false;
         $forceIndexedByDate = false;
+
         if ($idSiteIsAll || count($idSites) > 1) {
             $forceIndexedBySite = true;
         }
+
         if (count($periods) > 1 || $isMultipleDate) {
             $forceIndexedByDate = true;
         }
@@ -253,19 +266,19 @@ class Archive
 
     /**
      * Queries and returns metric data in an array.
-     * 
+     *
      * If multiple sites were requested in {@link build()} or {@link factory()} the result will
      * be indexed by site ID.
-     * 
+     *
      * If multiple periods were requested in {@link build()} or {@link factory()} the result will
      * be indexed by period.
-     * 
+     *
      * The site ID index is always first, so if multiple sites & periods were requested, the result
      * will be indexed by site ID first, then period.
-     * 
+     *
      * @param string|array $names One or more archive names, eg, `'nb_visits'`, `'Referrers_distinctKeywords'`,
      *                            etc.
-     * @return false|numeric|array `false` if there is no data to return, a single numeric value if we're not querying
+     * @return false|integer|array `false` if there is no data to return, a single numeric value if we're not querying
      *                             for multiple sites/periods, or an array if multiple sites, periods or names are
      *                             queried for.
      */
@@ -289,17 +302,17 @@ class Archive
 
     /**
      * Queries and returns blob data in an array.
-     * 
+     *
      * Reports are stored in blobs as serialized arrays of {@link DataTable\Row} instances, but this
      * data can technically be anything. In other words, you can store whatever you want
      * as archive data blobs.
      *
      * If multiple sites were requested in {@link build()} or {@link factory()} the result will
      * be indexed by site ID.
-     * 
+     *
      * If multiple periods were requested in {@link build()} or {@link factory()} the result will
      * be indexed by period.
-     * 
+     *
      * The site ID index is always first, so if multiple sites & periods were requested, the result
      * will be indexed by site ID first, then period.
      *
@@ -317,20 +330,20 @@ class Archive
 
     /**
      * Queries and returns metric data in a DataTable instance.
-     * 
+     *
      * If multiple sites were requested in {@link build()} or {@link factory()} the result will
      * be a DataTable\Map that is indexed by site ID.
-     * 
+     *
      * If multiple periods were requested in {@link build()} or {@link factory()} the result will
      * be a {@link DataTable\Map} that is indexed by period.
-     * 
+     *
      * The site ID index is always first, so if multiple sites & periods were requested, the result
      * will be a {@link DataTable\Map} indexed by site ID which contains {@link DataTable\Map} instances that are
      * indexed by period.
-     * 
+     *
      * _Note: Every DataTable instance returned will have at most one row in it. The contents of each
      *        row will be the requested metrics for the appropriate site and period._
-     * 
+     *
      * @param string|array $names One or more archive names, eg, 'nb_visits', 'Referrers_distinctKeywords',
      *                            etc.
      * @return DataTable|DataTable\Map A DataTable if multiple sites and periods were not requested.
@@ -344,20 +357,20 @@ class Archive
 
     /**
      * Queries and returns one or more reports as DataTable instances.
-     * 
+     *
      * This method will query blob data that is a serialized array of of {@link DataTable\Row}'s and
      * unserialize it.
-     * 
+     *
      * If multiple sites were requested in {@link build()} or {@link factory()} the result will
      * be a {@link DataTable\Map} that is indexed by site ID.
-     * 
+     *
      * If multiple periods were requested in {@link build()} or {@link factory()} the result will
      * be a DataTable\Map that is indexed by period.
-     * 
+     *
      * The site ID index is always first, so if multiple sites & periods were requested, the result
      * will be a {@link DataTable\Map} indexed by site ID which contains {@link DataTable\Map} instances that are
      * indexed by period.
-     * 
+     *
      * @param string $name The name of the record to get. This method can only query one record at a time.
      * @param int|string|null $idSubtable The ID of the subtable to get (if any).
      * @return DataTable|DataTable\Map A DataTable if multiple sites and periods were not requested.
@@ -371,13 +384,13 @@ class Archive
 
     /**
      * Queries and returns one report with all of its subtables loaded.
-     * 
+     *
      * If multiple sites were requested in {@link build()} or {@link factory()} the result will
      * be a DataTable\Map that is indexed by site ID.
-     * 
+     *
      * If multiple periods were requested in {@link build()} or {@link factory()} the result will
      * be a DataTable\Map that is indexed by period.
-     * 
+     *
      * The site ID index is always first, so if multiple sites & periods were requested, the result
      * will be a {@link DataTable\Map indexed} by site ID which contains {@link DataTable\Map} instances that are
      * indexed by period.
@@ -399,16 +412,18 @@ class Archive
 
     /**
      * Returns the list of plugins that archive the given reports.
-     * 
+     *
      * @param array $archiveNames
      * @return array
      */
     private function getRequestedPlugins($archiveNames)
     {
         $result = array();
+
         foreach ($archiveNames as $name) {
             $result[] = self::getPluginForReport($name);
         }
+
         return array_unique($result);
     }
 
@@ -426,7 +441,7 @@ class Archive
     /**
      * Helper function that creates an Archive instance and queries for report data using
      * query parameter data. API methods can use this method to reduce code redundancy.
-     * 
+     *
      * @param string $name The name of the report to return.
      * @param int|string|array $idSite @see {@link build()}
      * @param string $period @see {@link build()}
@@ -436,6 +451,7 @@ class Archive
      * @param int|null $idSubtable See {@link getDataTableExpanded()}
      * @param bool $skipAggregationOfSubTables Whether or not we should skip the aggregation of all sub-tables and only aggregate parent DataTable.
      * @param int|null $depth See {@link getDataTableExpanded()}
+     * @throws \Exception
      * @return DataTable|DataTable\Map See {@link getDataTable()} and
      *                                 {@link getDataTableExpanded()} for more
      *                                 information
@@ -445,9 +461,10 @@ class Archive
     {
         Piwik::checkUserHasViewAccess($idSite);
 
-        if($skipAggregationOfSubTables && ($expanded || $idSubtable)) {
+        if ($skipAggregationOfSubTables && ($expanded || $idSubtable)) {
             throw new \Exception("Not expected to skipAggregationOfSubTables when expanded=1 or idSubtable is set.");
         }
+
         $archive = Archive::build($idSite, $period, $date, $segment, $_restrictSitesToLogin = false, $skipAggregationOfSubTables);
         if ($idSubtable === false) {
             $idSubtable = null;
@@ -469,6 +486,69 @@ class Archive
         return $recordName . "_" . $id;
     }
 
+    private function getSiteIdsThatAreRequestedInThisArchiveButWereNotInvalidatedYet()
+    {
+        if (is_null(self::$cache)) {
+            self::$cache = Cache::getTransientCache();
+        }
+
+        $id = 'Archive.SiteIdsOfRememberedReportsInvalidated';
+
+        if (!self::$cache->contains($id)) {
+            self::$cache->save($id, array());
+        }
+
+        $siteIdsAlreadyHandled = self::$cache->fetch($id);
+        $siteIdsRequested      = $this->params->getIdSites();
+
+        foreach ($siteIdsRequested as $index => $siteIdRequested) {
+            $siteIdRequested = (int) $siteIdRequested;
+
+            if (in_array($siteIdRequested, $siteIdsAlreadyHandled)) {
+                unset($siteIdsRequested[$index]); // was already handled previously, do not do it again
+            } else {
+                $siteIdsAlreadyHandled[] = $siteIdRequested; // we will handle this id this time
+            }
+        }
+
+        self::$cache->save($id, $siteIdsAlreadyHandled);
+
+        return $siteIdsRequested;
+    }
+
+    private function invalidatedReportsIfNeeded()
+    {
+        $siteIdsRequested = $this->getSiteIdsThatAreRequestedInThisArchiveButWereNotInvalidatedYet();
+
+        if (empty($siteIdsRequested)) {
+            return; // all requested site ids were already handled
+        }
+
+        $invalidator  = new ArchiveInvalidator();
+        $sitesPerDays = $invalidator->getRememberedArchivedReportsThatShouldBeInvalidated();
+
+        foreach ($sitesPerDays as $date => $siteIds) {
+            if (empty($siteIds)) {
+                continue;
+            }
+
+            $siteIdsToActuallyInvalidate = array_intersect($siteIds, $siteIdsRequested);
+
+            if (empty($siteIdsToActuallyInvalidate)) {
+                continue; // all site ids that should be handled are already handled
+            }
+
+            try {
+                $invalidator->markArchivesAsInvalidated($siteIdsToActuallyInvalidate, $date, false);
+            } catch (\Exception $e) {
+                Site::clearCache();
+                throw $e;
+            }
+        }
+
+        Site::clearCache();
+    }
+
     /**
      * Queries archive tables for data and returns the result.
      * @param array|string $archiveNames
@@ -478,6 +558,8 @@ class Archive
      */
     private function get($archiveNames, $archiveDataType, $idSubtable = null)
     {
+        $this->invalidatedReportsIfNeeded();
+
         if (!is_array($archiveNames)) {
             $archiveNames = array($archiveNames);
         }
@@ -495,6 +577,7 @@ class Archive
             $archiveNames, $archiveDataType, $this->params->getIdSites(), $this->params->getPeriods(), $defaultRow = null);
 
         $archiveIds = $this->getArchiveIds($archiveNames);
+
         if (empty($archiveIds)) {
             return $result;
         }
@@ -532,7 +615,7 @@ class Archive
 
         // figure out which archives haven't been processed (if an archive has been processed,
         // then we have the archive IDs in $this->idarchives)
-        $doneFlags = array();
+        $doneFlags     = array();
         $archiveGroups = array();
         foreach ($plugins as $plugin) {
             $doneFlag = $this->getDoneStringForPlugin($plugin);
@@ -541,7 +624,7 @@ class Archive
             if (!isset($this->idarchives[$doneFlag])) {
                 $archiveGroup = $this->getArchiveGroupOfPlugin($plugin);
 
-                if($archiveGroup == self::ARCHIVE_ALL_PLUGINS_FLAG) {
+                if ($archiveGroup == self::ARCHIVE_ALL_PLUGINS_FLAG) {
                     $archiveGroup = reset($plugins);
                 }
                 $archiveGroups[] = $archiveGroup;
@@ -559,19 +642,7 @@ class Archive
             }
         }
 
-        // order idarchives by the table month they belong to
-        $idArchivesByMonth = array();
-        foreach (array_keys($doneFlags) as $doneFlag) {
-            if (empty($this->idarchives[$doneFlag])) {
-                continue;
-            }
-
-            foreach ($this->idarchives[$doneFlag] as $dateRange => $idarchives) {
-                foreach ($idarchives as $id) {
-                    $idArchivesByMonth[$dateRange][] = $id;
-                }
-            }
-        }
+        $idArchivesByMonth = $this->getIdArchivesByMonth($doneFlags);
 
         return $idArchivesByMonth;
     }
@@ -599,14 +670,14 @@ class Archive
                 // we already know there are no stats for this period
                 // we add one day to make sure we don't miss the day of the website creation
                 if ($twoDaysAfterPeriod->isEarlier($site->getCreationDate())) {
-                    Log::verbose("Archive site %s, %s (%s) skipped, archive is before the website was created.",
+                    Log::debug("Archive site %s, %s (%s) skipped, archive is before the website was created.",
                         $idSite, $period->getLabel(), $period->getPrettyString());
                     continue;
                 }
 
                 // if the starting date is in the future we know there is no visiidsite = ?t
                 if ($twoDaysBeforePeriod->isLater($today)) {
-                    Log::verbose("Archive site %s, %s (%s) skipped, archive is after today.",
+                    Log::debug("Archive site %s, %s (%s) skipped, archive is after today.",
                         $idSite, $period->getLabel(), $period->getPrettyString());
                     continue;
                 }
@@ -744,7 +815,10 @@ class Archive
      */
     private function getArchiveGroupOfPlugin($plugin)
     {
-        if ($this->getPeriodLabel() != 'range') {
+        $periods = $this->params->getPeriods();
+        $periodLabel = reset($periods)->getLabel();
+        
+        if (Rules::shouldProcessReportsAllPlugins($this->params->getIdSites(), $this->params->getSegment(), $periodLabel)) {
             return self::ARCHIVE_ALL_PLUGINS_FLAG;
         }
 
@@ -800,9 +874,29 @@ class Archive
 
             $idArchive = $archiveLoader->prepareArchive($plugin);
 
-            if($idArchive) {
+            if ($idArchive) {
                 $this->idarchives[$doneFlag][$periodString][] = $idArchive;
             }
         }
     }
+
+    private function getIdArchivesByMonth($doneFlags)
+    {
+        // order idarchives by the table month they belong to
+        $idArchivesByMonth = array();
+
+        foreach (array_keys($doneFlags) as $doneFlag) {
+            if (empty($this->idarchives[$doneFlag])) {
+                continue;
+            }
+
+            foreach ($this->idarchives[$doneFlag] as $dateRange => $idarchives) {
+                foreach ($idarchives as $id) {
+                    $idArchivesByMonth[$dateRange][] = $id;
+                }
+            }
+        }
+
+        return $idArchivesByMonth;
+    }
 }
