@@ -1,17 +1,14 @@
 <?php
 /**
- * Piwik - Open source web analytics
+ * Piwik - free/libre analytics platform
  *
  * @link http://piwik.org
  * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
  *
- * @category Piwik
- * @package Piwik
  */
 
 namespace Piwik\Archive;
 
-use Exception;
 use Piwik\Period;
 use Piwik\Segment;
 
@@ -38,16 +35,22 @@ class Parameters
      */
     private $segment;
 
+    /**
+     * @var bool
+     */
+    private $skipAggregationOfSubTables;
+
     public function getSegment()
     {
         return $this->segment;
     }
 
-    public function __construct($idSites, $periods, Segment $segment)
+    public function __construct($idSites, $periods, Segment $segment, $skipAggregationOfSubTables)
     {
         $this->idSites = $idSites;
         $this->periods = $periods;
         $this->segment = $segment;
+        $this->skipAggregationOfSubTables = $skipAggregationOfSubTables;
     }
 
     public function getPeriods()
@@ -60,5 +63,10 @@ class Parameters
         return $this->idSites;
     }
 
+    public function isSkipAggregationOfSubTables()
+    {
+        return $this->skipAggregationOfSubTables;
+    }
+
 }
 
