@@ -10,6 +10,7 @@ namespace Piwik;
 
 use Exception;
 use Piwik\ArchiveProcessor\Parameters;
+use Piwik\ArchiveProcessor\Rules;
 use Piwik\DataAccess\ArchiveWriter;
 use Piwik\DataAccess\LogAggregator;
 use Piwik\DataTable\Manager;
@@ -20,70 +21,70 @@ use Piwik\Period;
 
 /**
  * Used by {@link Piwik\Plugin\Archiver} instances to insert and aggregate archive data.
- * 
+ *
  * ### See also
- * 
+ *
  * - **{@link Piwik\Plugin\Archiver}** - to learn how plugins should implement their own analytics
  *                                       aggregation logic.
  * - **{@link Piwik\DataAccess\LogAggregator}** - to learn how plugins can perform data aggregation
  *                                                across Piwik's log tables.
- * 
+ *
  * ### Examples
- * 
+ *
  * **Inserting numeric data**
- * 
+ *
  *     // function in an Archiver descendant
  *     public function aggregateDayReport()
  *     {
  *         $archiveProcessor = $this->getProcessor();
- * 
+ *
  *         $myFancyMetric = // ... calculate the metric value ...
  *         $archiveProcessor->insertNumericRecord('MyPlugin_myFancyMetric', $myFancyMetric);
  *     }
- * 
+ *
  * **Inserting serialized DataTables**
- * 
+ *
  *     // function in an Archiver descendant
  *     public function aggregateDayReport()
  *     {
  *         $archiveProcessor = $this->getProcessor();
- * 
+ *
  *         $maxRowsInTable = Config::getInstance()->General['datatable_archiving_maximum_rows_standard'];j
- * 
+ *
  *         $dataTable = // ... build by aggregating visits ...
  *         $serializedData = $dataTable->getSerialized($maxRowsInTable, $maxRowsInSubtable = $maxRowsInTable,
  *                                                     $columnToSortBy = Metrics::INDEX_NB_VISITS);
- *         
+ *
  *         $archiveProcessor->insertBlobRecords('MyPlugin_myFancyReport', $serializedData);
  *     }
- * 
+ *
  * **Aggregating archive data**
- * 
+ *
  *     // function in Archiver descendant
  *     public function aggregateMultipleReports()
  *     {
  *         $archiveProcessor = $this->getProcessor();
- * 
+ *
  *         // aggregate a metric
  *         $archiveProcessor->aggregateNumericMetrics('MyPlugin_myFancyMetric');
  *         $archiveProcessor->aggregateNumericMetrics('MyPlugin_mySuperFancyMetric', 'max');
- * 
- *         // aggregate a report        
+ *
+ *         // aggregate a report
  *         $archiveProcessor->aggregateDataTableRecords('MyPlugin_myFancyReport');
  *     }
- * 
+ *
  */
 class ArchiveProcessor
 {
     /**
      * @var \Piwik\DataAccess\ArchiveWriter
      */
-    protected $archiveWriter;
+    private $archiveWriter;
 
     /**
      * @var \Piwik\DataAccess\LogAggregator
      */
-    protected $logAggregator;
+    private $logAggregator;
 
     /**
      * @var Archive
@@ -93,28 +94,43 @@ class ArchiveProcessor
     /**
      * @var Parameters
      */
-    protected $params;
+    private $params;
 
     /**
      * @var int
      */
-    protected $numberOfVisits = false;
-    protected $numberOfVisitsConverted = false;
+    private $numberOfVisits = false;
+
+    private $numberOfVisitsConverted = false;
+
+    /**
+     * If true, unique visitors are not calculated when we are aggregating data for multiple sites.
+     * The `[General] enable_processing_unique_visitors_multiple_sites` INI config option controls
+     * the value of this variable.
+     *
+     * @var bool
+     */
+    private $skipUniqueVisitorsCalculationForMultipleSites = true;
+
+    const SKIP_UNIQUE_VISITORS_FOR_MULTIPLE_SITES = 'enable_processing_unique_visitors_multiple_sites';
 
     public function __construct(Parameters $params, ArchiveWriter $archiveWriter)
     {
         $this->params = $params;
         $this->logAggregator = new LogAggregator($params);
         $this->archiveWriter = $archiveWriter;
+
+        $this->skipUniqueVisitorsCalculationForMultipleSites = Rules::shouldSkipUniqueVisitorsCalculationForMultipleSites();
     }
 
     protected function getArchive()
     {
-        if(empty($this->archive)) {
+        if (empty($this->archive)) {
             $subPeriods = $this->params->getSubPeriods();
-            $idSites = $this->params->getIdSites();
+            $idSites    = $this->params->getIdSites();
             $this->archive = Archive::factory($this->params->getSegment(), $subPeriods, $idSites);
         }
+
         return $this->archive;
     }
 
@@ -154,7 +170,8 @@ class ArchiveProcessor
      * @var array
      */
     protected static $columnsToRenameAfterAggregation = array(
-        Metrics::INDEX_NB_UNIQ_VISITORS => Metrics::INDEX_SUM_DAILY_NB_UNIQ_VISITORS
+        Metrics::INDEX_NB_UNIQ_VISITORS => Metrics::INDEX_SUM_DAILY_NB_UNIQ_VISITORS,
+        Metrics::INDEX_NB_USERS         => Metrics::INDEX_SUM_DAILY_NB_USERS,
     );
 
     /**
@@ -172,7 +189,7 @@ class ArchiveProcessor
      *                                               when summed because they cannot be summed, eg,
      *                                               `array('nb_uniq_visitors' => 'sum_daily_nb_uniq_visitors')`.
      * @return array Returns the row counts of each aggregated report before truncation, eg,
-     *               
+     *
      *                   array(
      *                       'report1' => array('level0' => $report1->getRowsCount,
      *                                          'recursive' => $report1->getRowsCountRecursive()),
@@ -192,6 +209,7 @@ class ArchiveProcessor
         if (!is_array($recordNames)) {
             $recordNames = array($recordNames);
         }
+
         $nameToCount = array();
         foreach ($recordNames as $recordName) {
             $latestUsedTableId = Manager::getInstance()->getMostRecentTableId();
@@ -202,7 +220,7 @@ class ArchiveProcessor
             $nameToCount[$recordName]['level0'] = $rowsCount;
 
             $rowsCountRecursive = $rowsCount;
-            if($this->isAggregateSubTables()) {
+            if ($this->isAggregateSubTables()) {
                 $rowsCountRecursive = $table->getRowsCountRecursive();
             }
             $nameToCount[$recordName]['recursive'] = $rowsCountRecursive;
@@ -227,12 +245,12 @@ class ArchiveProcessor
      * @return array|int Returns the array of aggregate values. If only one metric was aggregated,
      *                   the aggregate value will be returned as is, not in an array.
      *                   For example, if `array('nb_visits', 'nb_hits')` is supplied for `$columns`,
-     *                   
+     *
      *                       array(
      *                           'nb_visits' => 3040,
      *                           'nb_hits' => 405
      *                       )
-     *                   
+     *
      *                   could be returned. If `array('nb_visits')` or `'nb_visits'` is used for `$columns`,
      *                   then `3040` would be returned.
      * @api
@@ -242,6 +260,7 @@ class ArchiveProcessor
         $metrics = $this->getAggregatedNumericMetrics($columns, $operationToApply);
 
         foreach($metrics as $column => $value) {
+            $value = Common::forceDotAsSeparatorForDecimalPoint($value);
             $this->archiveWriter->insertRecord($column, $value);
         }
         // if asked for only one field to sum
@@ -255,7 +274,7 @@ class ArchiveProcessor
 
     public function getNumberOfVisits()
     {
-        if($this->numberOfVisits === false) {
+        if ($this->numberOfVisits === false) {
             throw new Exception("visits should have been set here");
         }
         return $this->numberOfVisits;
@@ -272,7 +291,7 @@ class ArchiveProcessor
      *
      * @param array $numericRecords A name-value mapping of numeric values that should be
      *                              archived, eg,
-     *                              
+     *
      *                                  array('Referrers_distinctKeywords' => 23, 'Referrers_distinctCampaigns' => 234)
      * @api
      */
@@ -296,6 +315,8 @@ class ArchiveProcessor
     public function insertNumericRecord($name, $value)
     {
         $value = round($value, 2);
+        $value = Common::forceDotAsSeparatorForDecimalPoint($value);
+
         $this->archiveWriter->insertRecord($name, $value);
     }
 
@@ -327,7 +348,7 @@ class ArchiveProcessor
      */
     protected function aggregateDataTableRecord($name, $columnsAggregationOperation = null, $columnsToRenameAfterAggregation = null)
     {
-        if($this->isAggregateSubTables()) {
+        if ($this->isAggregateSubTables()) {
             // By default we shall aggregate all sub-tables.
             $dataTable = $this->getArchive()->getDataTableExpanded($name, $idSubTable = null, $depth = null, $addMetadataSubtableId = false);
         } else {
@@ -364,16 +385,23 @@ class ArchiveProcessor
 
     protected function enrichWithUniqueVisitorsMetric(Row $row)
     {
-        if(!$this->getParams()->isSingleSite() ) {
-            // we only compute unique visitors for a single site
+        // skip unique visitors metrics calculation if calculating for multiple sites is disabled
+        if (!$this->getParams()->isSingleSite()
+            && $this->skipUniqueVisitorsCalculationForMultipleSites
+        ) {
             return;
         }
-        if ( $row->getColumn('nb_uniq_visitors') !== false) {
+        if ($row->getColumn('nb_uniq_visitors') !== false
+            || $row->getColumn('nb_users') !== false
+        ) {
             if (SettingsPiwik::isUniqueVisitorsEnabled($this->getParams()->getPeriod()->getLabel())) {
-                $uniqueVisitors = (float)$this->computeNbUniqVisitors();
-                $row->setColumn('nb_uniq_visitors', $uniqueVisitors);
+                $metrics = array(Metrics::INDEX_NB_UNIQ_VISITORS, Metrics::INDEX_NB_USERS);
+                $uniques = $this->computeNbUniques( $metrics );
+                $row->setColumn('nb_uniq_visitors', $uniques[Metrics::INDEX_NB_UNIQ_VISITORS]);
+                $row->setColumn('nb_users', $uniques[Metrics::INDEX_NB_USERS]);
             } else {
                 $row->deleteColumn('nb_uniq_visitors');
+                $row->deleteColumn('nb_users');
             }
         }
     }
@@ -395,14 +423,15 @@ class ArchiveProcessor
      * This is the only Period metric (ie. week/month/year/range) that we process from the logs directly,
      * since unique visitors cannot be summed like other metrics.
      *
+     * @param array Metrics Ids for which to aggregates count of values
      * @return int
      */
-    protected function computeNbUniqVisitors()
+    protected function computeNbUniques($metrics)
     {
         $logAggregator = $this->getLogAggregator();
-        $query = $logAggregator->queryVisitsByDimension(array(), false, array(), array(Metrics::INDEX_NB_UNIQ_VISITORS));
+        $query = $logAggregator->queryVisitsByDimension(array(), false, array(), $metrics);
         $data = $query->fetch();
-        return $data[Metrics::INDEX_NB_UNIQ_VISITORS];
+        return $data;
     }
 
     /**
@@ -416,15 +445,18 @@ class ArchiveProcessor
     protected function getAggregatedDataTableMap($data, $columnsAggregationOperation)
     {
         $table = new DataTable();
+
         if (!empty($columnsAggregationOperation)) {
             $table->setMetadata(DataTable::COLUMN_AGGREGATION_OPS_METADATA_NAME, $columnsAggregationOperation);
         }
+
         if ($data instanceof DataTable\Map) {
             // as $date => $tableToSum
             $this->aggregatedDataTableMapsAsOne($data, $table);
         } else {
             $table->addDataTable($data, $this->isAggregateSubTables());
         }
+
         return $table;
     }
 
@@ -436,7 +468,7 @@ class ArchiveProcessor
     protected function aggregatedDataTableMapsAsOne(Map $map, DataTable $aggregated)
     {
         foreach ($map->getDataTables() as $tableToAggregate) {
-            if($tableToAggregate instanceof Map) {
+            if ($tableToAggregate instanceof Map) {
                 $this->aggregatedDataTableMapsAsOne($tableToAggregate, $aggregated);
             } else {
                 $aggregated->addDataTable($tableToAggregate, $this->isAggregateSubTables());
@@ -453,6 +485,7 @@ class ArchiveProcessor
         if (is_null($columnsToRenameAfterAggregation)) {
             $columnsToRenameAfterAggregation = self::$columnsToRenameAfterAggregation;
         }
+
         foreach ($columnsToRenameAfterAggregation as $oldName => $newName) {
             $table->renameColumn($oldName, $newName, $this->isAggregateSubTables());
         }
@@ -463,6 +496,7 @@ class ArchiveProcessor
         if (!is_array($columns)) {
             $columns = array($columns);
         }
+
         $operationForColumn = $this->getOperationForColumns($columns, $operationToApply);
 
         $dataTable = $this->getArchive()->getDataTableFromNumeric($columns);
@@ -473,7 +507,7 @@ class ArchiveProcessor
         }
 
         $rowMetrics = $results->getFirstRow();
-        if($rowMetrics === false) {
+        if ($rowMetrics === false) {
             $rowMetrics = new Row;
         }
         $this->enrichWithUniqueVisitorsMetric($rowMetrics);
@@ -486,6 +520,7 @@ class ArchiveProcessor
                 $metrics[$name] = 0;
             }
         }
+
         return $metrics;
     }
 
