@@ -45,7 +45,7 @@ class Parameters
 
     /**
      * Constructor.
-     * 
+     *
      * @ignore
      */
     public function __construct(Site $site, Period $period, Segment $segment, $skipAggregationOfSubTables = false)
@@ -91,7 +91,7 @@ class Parameters
      */
     public function getSubPeriods()
     {
-        if($this->getPeriod()->getLabel() == 'day') {
+        if ($this->getPeriod()->getLabel() == 'day') {
             return array( $this->getPeriod() );
         }
         return $this->getPeriod()->getSubperiods();
@@ -180,7 +180,7 @@ class Parameters
         if ($isTemporary) {
             $temporary = 'temporary archive';
         }
-        Log::verbose(
+        Log::debug(
             "%s archive, idSite = %d (%s), segment '%s', report = '%s', UTC datetime [%s -> %s]",
             $this->getPeriod()->getLabel(),
             $this->getSite()->getId(),
