@@ -48,7 +48,7 @@ abstract class UIAssetMerger
 
     public function generateFile()
     {
-        if(!$this->shouldGenerate())
+        if (!$this->shouldGenerate())
             return;
 
         $this->mergedContent = $this->getMergedAssets();
@@ -138,8 +138,9 @@ abstract class UIAssetMerger
      */
     private function shouldGenerate()
     {
-        if(!$this->mergedAsset->exists())
+        if (!$this->mergedAsset->exists()) {
             return true;
+        }
 
         return !$this->isFileUpToDate();
     }
@@ -162,19 +163,11 @@ abstract class UIAssetMerger
         return false;
     }
 
-    /**
-     * @return boolean
-     */
-    private function isMergedAssetsDisabled()
-    {
-        return AssetManager::getInstance()->isMergedAssetsDisabled();
-    }
-
     private function adjustPaths()
     {
         $theme = $this->assetFetcher->getTheme();
         // During installation theme is not yet ready
-        if($theme) {
+        if ($theme) {
             $this->mergedContent = $this->assetFetcher->getTheme()->rewriteAssetsPathToTheme($this->mergedContent);
         }
     }
@@ -189,7 +182,7 @@ abstract class UIAssetMerger
      */
     protected function getCacheBusterValue()
     {
-        if(empty($this->cacheBusterValue))
+        if (empty($this->cacheBusterValue))
             $this->cacheBusterValue = $this->generateCacheBuster();
 
         return $this->cacheBusterValue;
@@ -199,12 +192,4 @@ abstract class UIAssetMerger
     {
         $this->mergedContent = $this->getPreamble() . $this->mergedContent;
     }
-
-    /**
-     * @return boolean
-     */
-    private function shouldCompareExistingVersion()
-    {
-        return $this->isMergedAssetsDisabled();
-    }
 }
