@@ -34,6 +34,7 @@ class Common
     /*
      * Database
      */
+    const LANGUAGE_CODE_INVALID = 'xx';
 
     /**
      * Hashes a string into an integer which should be very low collision risks
@@ -48,7 +49,7 @@ class Common
 
     /**
      * Returns a prefixed table name.
-     * 
+     *
      * The table prefix is determined by the `[database] tables_prefix` INI config
      * option.
      *
@@ -82,7 +83,7 @@ class Common
      *
      * The table prefix is determined by the `[database] tables_prefix` INI config
      * option.
-     * 
+     *
      * @param string $table The prefixed table name, eg "piwik-production_log_visit".
      * @return string The unprefixed table name, eg "log_visit".
      * @api
@@ -168,7 +169,7 @@ class Common
 
     /**
      * Multi-byte substr() - works with UTF-8.
-     * 
+     *
      * Calls `mb_substr` if available and falls back to `substr` if it's not.
      *
      * @param string $string
@@ -192,7 +193,7 @@ class Common
 
     /**
      * Multi-byte strlen() - works with UTF-8
-     * 
+     *
      * Calls `mb_substr` if available and falls back to `substr` if not.
      *
      * @param string $string
@@ -210,9 +211,9 @@ class Common
 
     /**
      * Multi-byte strtolower() - works with UTF-8.
-     * 
+     *
      * Calls `mb_strtolower` if available and falls back to `strtolower` if not.
-     * 
+     *
      * @param string $string
      * @return string
      * @api
@@ -232,18 +233,18 @@ class Common
 
     /**
      * Sanitizes a string to help avoid XSS vulnerabilities.
-     * 
+     *
      * This function is automatically called when {@link getRequestVar()} is called,
      * so you should not normally have to use it.
-     * 
+     *
      * This function should be used when outputting data that isn't escaped and was
      * obtained from the user (for example when using the `|raw` twig filter on goal names).
-     * 
+     *
      * _NOTE: Sanitized input should not be used directly in an SQL query; SQL placeholders
      * should still be used._
-     * 
+     *
      * **Implementation Details**
-     * 
+     *
      * - [htmlspecialchars](http://php.net/manual/en/function.htmlspecialchars.php) is used to escape text.
      * - Single quotes are not escaped so **Piwik's amazing community** will still be
      *   **Piwik's amazing community**.
@@ -263,7 +264,7 @@ class Common
         if (is_numeric($value)) {
             return $value;
         } elseif (is_string($value)) {
-            $value = self::sanitizeInputValue($value);
+            $value = self::sanitizeString($value);
 
             if (!$alreadyStripslashed) // a JSON array was already stripslashed, don't do it again for each value
             {
@@ -289,20 +290,32 @@ class Common
     }
 
     /**
-     * Sanitize a single input value
+     * Sanitize a single input value and removes line breaks, tabs and null characters.
      *
      * @param string $value
      * @return string  sanitized input
      */
     public static function sanitizeInputValue($value)
+    {
+        $value = self::sanitizeLineBreaks($value);
+        $value = self::sanitizeString($value);
+        return $value;
+    }
+
+    /**
+     * Sanitize a single input value
+     *
+     * @param $value
+     * @return string
+     */
+    private static function sanitizeString($value)
     {
         // $_GET and $_REQUEST already urldecode()'d
         // decode
         // note: before php 5.2.7, htmlspecialchars() double encodes &#x hex items
         $value = html_entity_decode($value, self::HTML_ENCODING_QUOTE_STYLE, 'UTF-8');
 
-        // filter
-        $value = self::sanitizeLineBreaks($value);
+        $value = self::sanitizeNullBytes($value);
 
         // escape
         $tmp = @htmlspecialchars($value, self::HTML_ENCODING_QUOTE_STYLE, 'UTF-8');
@@ -312,13 +325,14 @@ class Common
             // convert and escape
             $value = utf8_encode($value);
             $tmp = htmlspecialchars($value, self::HTML_ENCODING_QUOTE_STYLE, 'UTF-8');
+            return $tmp;
         }
         return $tmp;
     }
 
     /**
      * Unsanitizes a single input value and returns the result.
-     * 
+     *
      * @param string $value
      * @return string  unsanitized input
      */
@@ -332,10 +346,10 @@ class Common
      *
      * This method should be used when you need to unescape data that was obtained from
      * the user.
-     * 
+     *
      * Some data in Piwik is stored sanitized (such as site name). In this case you may
      * have to use this method to unsanitize it in order to, for example, output it in JSON.
-     * 
+     *
      * @param string|array $value The data to unsanitize. If an array is passed, the
      *                            array is sanitized recursively. Key values are not unsanitized.
      * @return string|array The unsanitized data.
@@ -362,28 +376,38 @@ class Common
      */
     private static function undoMagicQuotes($value)
     {
-        return version_compare(PHP_VERSION, '5.4', '<')
-            && get_magic_quotes_gpc()
-            ? stripslashes($value)
-            : $value;
-    }
+        if (version_compare(PHP_VERSION, '5.4', '<') &&
+            get_magic_quotes_gpc()) {
+
+            $value = stripslashes($value);
+        }
 
-    /**
-     *
-     * @param string
-     * @return string Line breaks and line carriage removed
-     */
-    public static function sanitizeLineBreaks($value)
-    {
-        $value = str_replace(array("\n", "\r", "\0"), '', $value);
         return $value;
     }
 
+    /**
+     * @param string $value
+     * @return string Line breaks and line carriage removed
+     */
+    public static function sanitizeLineBreaks($value)
+    {
+        return str_replace(array("\n", "\r"), '', $value);
+    }
+
+    /**
+     * @param string $value
+     * @return string Null bytes removed
+     */
+    public static function sanitizeNullBytes($value)
+    {
+        return str_replace(array("\0"), '', $value);
+    }
+
     /**
      * Gets a sanitized request parameter by name from the `$_GET` and `$_POST` superglobals.
-     * 
+     *
      * Use this function to get request parameter values. **_NEVER use `$_GET` and `$_POST` directly._**
-     * 
+     *
      * If the variable cannot be found, and a default value was not provided, an exception is raised.
      *
      * _See {@link sanitizeInputValues()} to learn more about sanitization._
@@ -393,7 +417,7 @@ class Common
      * @param string|null $varDefault The value to return if the request parameter cannot be found or has an empty value.
      * @param string|null $varType Expected type of the request variable. This parameters value must be one of the following:
      *                             `'array'`, `'int'`, `'integer'`, `'string'`, `'json'`.
-     *                             
+     *
      *                             If `'json'`, the string value will be `json_decode`-d and then sanitized.
      * @param array|null $requestArrayToUse The array to use instead of `$_GET` and `$_POST`.
      * @throws Exception If the request parameter doesn't exist and there is no default value, or if the request parameter
@@ -406,6 +430,7 @@ class Common
         if (is_null($requestArrayToUse)) {
             $requestArrayToUse = $_GET + $_POST;
         }
+
         $varDefault = self::sanitizeInputValues($varDefault);
         if ($varType === 'int') {
             // settype accepts only integer
@@ -437,7 +462,7 @@ class Common
         // we deal w/ json differently
         if ($varType == 'json') {
             $value = self::undoMagicQuotes($requestArrayToUse[$varName]);
-            $value = self::json_decode($value, $assoc = true);
+            $value = json_decode($value, $assoc = true);
             return self::sanitizeInputValues($value, $alreadyStripslashed = true);
         }
 
@@ -446,11 +471,20 @@ class Common
             $ok = false;
 
             if ($varType === 'string') {
-                if (is_string($value)) $ok = true;
+                if (is_string($value) || is_int($value)) {
+                    $ok = true;
+                } else if (is_float($value)) {
+                    $value = Common::forceDotAsSeparatorForDecimalPoint($value);
+                    $ok    = true;
+                }
+
             } elseif ($varType === 'integer') {
                 if ($value == (string)(int)$value) $ok = true;
             } elseif ($varType === 'float') {
-                if ($value == (string)(float)$value) $ok = true;
+                $valueToCompare = (string)(float)$value;
+                $valueToCompare = Common::forceDotAsSeparatorForDecimalPoint($valueToCompare);
+
+                if ($value == $valueToCompare) $ok = true;
             } elseif ($varType === 'array') {
                 if (is_array($value)) $ok = true;
             } else {
@@ -469,6 +503,7 @@ class Common
             }
             settype($value, $varType);
         }
+
         return $value;
     }
 
@@ -483,7 +518,13 @@ class Common
      */
     public static function generateUniqId()
     {
-        return md5(uniqid(rand(), true));
+        if (function_exists('mt_rand')) {
+            $rand = mt_rand();
+        } else {
+            $rand = rand();
+        }
+
+        return md5(uniqid($rand, true));
     }
 
     /**
@@ -496,14 +537,17 @@ class Common
     public static function hash($str, $raw_output = false)
     {
         static $hashAlgorithm = null;
+
         if (is_null($hashAlgorithm)) {
             $hashAlgorithm = @Config::getInstance()->General['hash_algorithm'];
         }
 
         if ($hashAlgorithm) {
             $hash = @hash($hashAlgorithm, $str, $raw_output);
-            if ($hash !== false)
+            if ($hash !== false) {
+
                 return $hash;
+            }
         }
 
         return md5($str, $raw_output);
@@ -520,7 +564,7 @@ class Common
     public static function getRandomString($length = 16, $alphabet = "abcdefghijklmnoprstuvwxyz0123456789")
     {
         $chars = $alphabet;
-        $str = '';
+        $str   = '';
 
         list($usec, $sec) = explode(" ", microtime());
         $seed = ((float)$sec + (float)$usec) * 100000;
@@ -530,6 +574,7 @@ class Common
             $rand_key = mt_rand(0, strlen($chars) - 1);
             $str .= substr($chars, $rand_key, 1);
         }
+
         return str_shuffle($str);
     }
 
@@ -571,9 +616,24 @@ class Common
         ) {
             throw new Exception("visitorId is expected to be a " . Tracker::LENGTH_HEX_ID_STRING . " hex char string");
         }
+
         return self::hex2bin($id);
     }
 
+    /**
+     * Converts a User ID string to the Visitor ID Binary representation.
+     *
+     * @param $userId
+     * @return string
+     */
+    public static function convertUserIdToVisitorIdBin($userId)
+    {
+        require_once PIWIK_INCLUDE_PATH . '/libs/PiwikTracker/PiwikTracker.php';
+        $userIdHashed = \PiwikTracker::getUserIdHashed($userId);
+
+        return self::convertVisitorIdToBin($userIdHashed);
+    }
+
     /**
      * Convert IP address (in network address format) to presentation format.
      * This is a backward compatibility function for code that only expects
@@ -717,6 +777,7 @@ class Common
         if ($includeInternalCodes) {
             return array_merge($countriesList, $extras);
         }
+
         return $countriesList;
     }
 
@@ -857,7 +918,7 @@ class Common
             }
         }
 
-        if (is_null($browserLang)) {
+        if (empty($browserLang)) {
             // a fallback might be to infer the language in HTTP_USER_AGENT (i.e., localized build)
             $browserLang = "";
         } else {
@@ -889,8 +950,8 @@ class Common
      */
     public static function getCountry($lang, $enableLanguageToCountryGuess, $ip)
     {
-        if (empty($lang) || strlen($lang) < 2 || $lang == 'xx') {
-            return 'xx';
+        if (empty($lang) || strlen($lang) < 2 || $lang == self::LANGUAGE_CODE_INVALID) {
+            return self::LANGUAGE_CODE_INVALID;
         }
 
         $validCountries = self::getCountriesList();
@@ -926,35 +987,73 @@ class Common
                 }
             }
         }
-        return 'xx';
+        return self::LANGUAGE_CODE_INVALID;
     }
 
     /**
-     * Returns the visitor language based only on the Browser 'accepted language' information
+     * Returns the language and region string, based only on the Browser 'accepted language' information.
+     * * The language tag is defined by ISO 639-1
      *
      * @param string $browserLanguage Browser's accepted langauge header
      * @param array $validLanguages array of valid language codes
-     * @return string  2 letter ISO 639 code
+     * @return string  2 letter ISO 639 code  'es' (Spanish)
      */
-    public static function extractLanguageCodeFromBrowserLanguage($browserLanguage, $validLanguages)
+    public static function extractLanguageCodeFromBrowserLanguage($browserLanguage, $validLanguages = array())
     {
-        // assumes language preference is sorted;
-        // does not handle language-script-region tags or language range (*)
-        if (!empty($validLanguages) && preg_match_all('/(?:^|,)([a-z]{2,3})([-][a-z]{2})?/', $browserLanguage, $matches, PREG_SET_ORDER)) {
-            foreach ($matches as $parts) {
-                if (count($parts) == 3) {
-                    // match locale (language and location)
-                    if (in_array($parts[1] . $parts[2], $validLanguages)) {
-                        return $parts[1] . $parts[2];
-                    }
+        $validLanguages = self::checkValidLanguagesIsSet($validLanguages);
+        $languageRegionCode = self::extractLanguageAndRegionCodeFromBrowserLanguage($browserLanguage, $validLanguages);
+
+        if(strlen($languageRegionCode) == 2) {
+            $languageCode = $languageRegionCode;
+        } else {
+            $languageCode = substr($languageRegionCode, 0, 2);
+        }
+        if(in_array($languageCode, $validLanguages)) {
+            return $languageCode;
+        }
+        return self::LANGUAGE_CODE_INVALID;
+    }
+
+    /**
+     * Returns the language and region string, based only on the Browser 'accepted language' information.
+     * * The language tag is defined by ISO 639-1
+     * * The region tag is defined by ISO 3166-1
+     *
+     * @param string $browserLanguage Browser's accepted langauge header
+     * @param array $validLanguages array of valid language codes. Note that if the array includes "fr" then it will consider all regional variants of this language valid, such as "fr-ca" etc.
+     * @return string 2 letter ISO 639 code 'es' (Spanish) or if found, includes the region as well: 'es-ar'
+     */
+    public static function extractLanguageAndRegionCodeFromBrowserLanguage($browserLanguage, $validLanguages = array() )
+    {
+        $validLanguages = self::checkValidLanguagesIsSet($validLanguages);
+
+        if(!preg_match_all('/(?:^|,)([a-z]{2,3})([-][a-z]{2})?/', $browserLanguage, $matches, PREG_SET_ORDER)) {
+            return self::LANGUAGE_CODE_INVALID;
+        }
+        foreach ($matches as $parts) {
+            $langIso639 = $parts[1];
+            if(empty($langIso639)) {
+                continue;
+            }
+
+            // If a region tag is found eg. "fr-ca"
+            if (count($parts) == 3) {
+                $regionIso3166 = $parts[2]; // eg. "-ca"
+
+                if (in_array($langIso639 . $regionIso3166, $validLanguages)) {
+                    return $langIso639 . $regionIso3166;
                 }
-                // match language only (where no region provided)
-                if (in_array($parts[1], $validLanguages)) {
-                    return $parts[1];
+
+                if (in_array($langIso639, $validLanguages)) {
+                    return $langIso639 . $regionIso3166;
                 }
             }
+            // eg. "fr" or "es"
+            if (in_array($langIso639, $validLanguages)) {
+                return $langIso639;
+            }
         }
-        return 'xx';
+        return self::LANGUAGE_CODE_INVALID;
     }
 
     /**
@@ -1012,10 +1111,10 @@ class Common
     /**
      * Returns a string with a comma separated list of placeholders for use in an SQL query. Used mainly
      * to fill the `IN (...)` part of a query.
-     * 
+     *
      * @param array|string $fields The names of the mysql table fields to bind, e.g.
      *                             `array(fieldName1, fieldName2, fieldName3)`.
-     * 
+     *
      *                             _Note: The content of the array isn't important, just its length._
      * @return string The placeholder string, e.g. `"?, ?, ?"`.
      * @api
@@ -1032,6 +1131,22 @@ class Common
         return '?' . str_repeat(',?', $count - 1);
     }
 
+    /**
+     * Force the separator for decimal point to be a dot. See https://github.com/piwik/piwik/issues/6435
+     * If for instance a German locale is used it would be a comma otherwise.
+     *
+     * @param  float|string $value
+     * @return string
+     */
+    public static function forceDotAsSeparatorForDecimalPoint($value)
+    {
+        if (null === $value || false === $value) {
+            return $value;
+        }
+
+        return str_replace(',', '.', $value);
+    }
+
     /**
      * Sets outgoing header.
      *
@@ -1041,16 +1156,55 @@ class Common
     public static function sendHeader($header, $replace = true)
     {
         // don't send header in CLI mode
-        if(Common::isPhpCliMode()) {
-            return;
-        }
-        if (isset($GLOBALS['PIWIK_TRACKER_LOCAL_TRACKING']) && $GLOBALS['PIWIK_TRACKER_LOCAL_TRACKING']) {
-            @header($header, $replace);
-        } else {
+        if (!Common::isPhpCliMode() and !headers_sent()) {
             header($header, $replace);
         }
     }
 
+    /**
+     * Sends the given response code if supported.
+     *
+     * @param int $code  Eg 204
+     *
+     * @throws Exception
+     */
+    public static function sendResponseCode($code)
+    {
+        $messages = array(
+            200 => 'Ok',
+            204 => 'No Response',
+            301 => 'Moved Permanently',
+            302 => 'Found',
+            304 => 'Not Modified',
+            400 => 'Bad Request',
+            401 => 'Unauthorized',
+            403 => 'Forbidden',
+            404 => 'Not Found',
+            500 => 'Internal Server Error'
+        );
+
+        if (!array_key_exists($code, $messages)) {
+            throw new Exception('Response code not supported: ' . $code);
+        }
+
+        if (strpos(PHP_SAPI, '-fcgi') === false) {
+            $key = 'HTTP/1.1';
+
+            if (array_key_exists('SERVER_PROTOCOL', $_SERVER)
+                && strlen($_SERVER['SERVER_PROTOCOL']) < 15
+                && strlen($_SERVER['SERVER_PROTOCOL']) > 1) {
+                $key = $_SERVER['SERVER_PROTOCOL'];
+            }
+
+        } else {
+            // FastCGI
+            $key = 'Status:';
+        }
+
+        $message = $messages[$code];
+        Common::sendHeader($key . ' ' . $code . ' ' . $message);
+    }
+
     /**
      * Returns the ID of the current LocationProvider (see UserCountry plugin code) from
      * the Tracker cache.
@@ -1070,7 +1224,7 @@ class Common
      * @param $var The object to destroy.
      * @api
      */
-    static public function destroy(&$var)
+    public static function destroy(&$var)
     {
         if (is_object($var) && method_exists($var, '__destruct')) {
             $var->__destruct();
@@ -1079,7 +1233,11 @@ class Common
         $var = null;
     }
 
-    static public function printDebug($info = '')
+    /**
+     * @todo This method is weird, it's debugging statements but seem to only work for the tracker, maybe it
+     * should be moved elsewhere
+     */
+    public static function  printDebug($info = '')
     {
         if (isset($GLOBALS['PIWIK_TRACKER_DEBUG']) && $GLOBALS['PIWIK_TRACKER_DEBUG']) {
 
@@ -1087,8 +1245,6 @@ class Common
                 $info = var_export($info, true);
             }
 
-            Log::getInstance()->setLogLevel(Log::DEBUG);
-
             if (is_array($info) || is_object($info)) {
                 $info = Common::sanitizeInputValues($info);
                 $out = var_export($info, true);
@@ -1102,4 +1258,17 @@ class Common
             }
         }
     }
+
+    /**
+     * @param $validLanguages
+     * @return array
+     */
+    protected static function checkValidLanguagesIsSet($validLanguages)
+    {
+        if (empty($validLanguages)) {
+            $validLanguages = array_keys(Common::getLanguagesList());
+            return $validLanguages;
+        }
+        return $validLanguages;
+    }
 }
