@@ -8,18 +8,20 @@
  */
 namespace Piwik;
 
+use Piwik\Container\StaticContainer;
 use Piwik\Plugin\Manager as PluginManager;
+use Symfony\Bridge\Monolog\Handler\ConsoleHandler;
 use Symfony\Component\Console\Application;
-use Symfony\Component\Console\Input\ArgvInput;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Input\InputOption;
-use Symfony\Component\Console\Input\StringInput;
 use Symfony\Component\Console\Output\OutputInterface;
 
 class Console extends Application
 {
     public function __construct()
     {
+        $this->checkCompatibility();
+
         parent::__construct();
 
         $option = new InputOption('piwik-domain',
@@ -29,17 +31,16 @@ class Console extends Application
         );
 
         $this->getDefinition()->addOption($option);
-    }
 
-    public function init()
-    {
-        $this->checkCompatibility();
+        StaticContainer::setEnvironment('cli');
     }
 
     public function doRun(InputInterface $input, OutputInterface $output)
     {
         $this->initPiwikHost($input);
         $this->initConfig($output);
+        $this->initLoggerOutput($output);
+
         try {
             self::initPlugins();
         } catch(\Exception $e) {
@@ -51,17 +52,24 @@ class Console extends Application
         $commands = $this->getAvailableCommands();
 
         foreach ($commands as $command) {
-
-            if (!class_exists($command)) {
-                Log::warning(sprintf('Cannot add command %s, class does not exist', $command));
-            } elseif (!is_subclass_of($command, 'Piwik\Plugin\ConsoleCommand')) {
-                Log::warning(sprintf('Cannot add command %s, class does not extend Piwik\Plugin\ConsoleCommand', $command));
-            } else {
-                $this->add(new $command);
-            }
+            $this->addCommandIfExists($command);
         }
 
-        return parent::doRun($input, $output);
+        $self = $this;
+        return Access::doAsSuperUser(function () use ($input, $output, $self) {
+            return call_user_func(array($self, 'Symfony\Component\Console\Application::doRun'), $input, $output);
+        });
+    }
+
+    private function addCommandIfExists($command)
+    {
+        if (!class_exists($command)) {
+            Log::warning(sprintf('Cannot add command %s, class does not exist', $command));
+        } elseif (!is_subclass_of($command, 'Piwik\Plugin\ConsoleCommand')) {
+            Log::warning(sprintf('Cannot add command %s, class does not extend Piwik\Plugin\ConsoleCommand', $command));
+        } else {
+            $this->add(new $command);
+        }
     }
 
     /**
@@ -72,11 +80,9 @@ class Console extends Application
     private function getAvailableCommands()
     {
         $commands = $this->getDefaultPiwikCommands();
+        $detected = PluginManager::getInstance()->findMultipleComponents('Commands', 'Piwik\\Plugin\\ConsoleCommand');
 
-        $pluginNames = PluginManager::getInstance()->getLoadedPluginsName();
-        foreach ($pluginNames as $pluginName) {
-            $commands = array_merge($commands, $this->findCommandsInPlugin($pluginName));
-        }
+        $commands = array_merge($commands, $detected);
 
         /**
          * Triggered to filter / restrict console commands. Plugins that want to restrict commands
@@ -101,43 +107,29 @@ class Console extends Application
         return $commands;
     }
 
-    private function findCommandsInPlugin($pluginName)
-    {
-        $commands = array();
-
-        $files = Filesystem::globr(PIWIK_INCLUDE_PATH . '/plugins/' . $pluginName .'/Commands', '*.php');
-
-        foreach ($files as $file) {
-            $klassName = sprintf('Piwik\\Plugins\\%s\\Commands\\%s', $pluginName, basename($file, '.php'));
-
-            if (!class_exists($klassName) || !is_subclass_of($klassName, 'Piwik\\Plugin\\ConsoleCommand')) {
-                continue;
-            }
-
-            $klass = new \ReflectionClass($klassName);
-
-            if ($klass->isAbstract()) {
-                continue;
-            }
-
-            $commands[] = $klassName;
-        }
-
-        return $commands;
-    }
-
     private function checkCompatibility()
     {
         if (Common::isPhpCgiType()) {
-            echo 'Piwik Console is known to be not compatible with PHP-CGI. Please execute console using PHP-CLI. For instance "/usr/bin/php-cli console ..."';
+            echo 'Piwik Console is known to be not compatible with PHP-CGI (you are using '.php_sapi_name().'). ' .
+                 'Please execute console using PHP-CLI. For instance "/usr/bin/php-cli console ..."';
             echo "\n";
             exit(1);
         }
     }
 
+    public static function isSupported()
+    {
+        return Common::isPhpCliMode() && !Common::isPhpCgiType();
+    }
+
     protected function initPiwikHost(InputInterface $input)
     {
         $piwikHostname = $input->getParameterOption('--piwik-domain');
+
+        if (empty($piwikHostname)) {
+            $piwikHostname = $input->getParameterOption('--url');
+        }
+
         $piwikHostname = UrlHelper::getHostFromUrl($piwikHostname);
         Url::setHost($piwikHostname);
     }
@@ -145,14 +137,31 @@ class Console extends Application
     protected function initConfig(OutputInterface $output)
     {
         $config = Config::getInstance();
+
         try {
             $config->checkLocalConfigFound();
             return $config;
+
         } catch (\Exception $e) {
             $output->writeln($e->getMessage() . "\n");
         }
     }
 
+    /**
+     * Register the console output into the logger.
+     *
+     * Ideally, this should be done automatically with events:
+     * @see http://symfony.com/fr/doc/current/components/console/events.html
+     * @see Symfony\Bridge\Monolog\Handler\ConsoleHandler::onCommand()
+     * But it would require to install Symfony's Event Dispatcher.
+     */
+    private function initLoggerOutput(OutputInterface $output)
+    {
+        /** @var ConsoleHandler $consoleLogHandler */
+        $consoleLogHandler = StaticContainer::getContainer()->get('Symfony\Bridge\Monolog\Handler\ConsoleHandler');
+        $consoleLogHandler->setOutput($output);
+    }
+
     public static function initPlugins()
     {
         Plugin\Manager::getInstance()->loadActivatedPlugins();
@@ -163,11 +172,13 @@ class Console extends Application
         $commands = array(
             'Piwik\CliMulti\RequestCommand'
         );
-        
+
         if (class_exists('Piwik\Plugins\EnterpriseAdmin\EnterpriseAdmin')) {
             $extra = new \Piwik\Plugins\EnterpriseAdmin\EnterpriseAdmin();
             $extra->addConsoleCommands($commands);
         }
+
         return $commands;
     }
+
 }
