@@ -57,11 +57,12 @@ class DataArray
      *
      * @return array
      */
-    static public function makeEmptyRow()
+    public static function makeEmptyRow()
     {
         return array(Metrics::INDEX_NB_UNIQ_VISITORS    => 0,
                      Metrics::INDEX_NB_VISITS           => 0,
                      Metrics::INDEX_NB_ACTIONS          => 0,
+                     Metrics::INDEX_NB_USERS            => 0,
                      Metrics::INDEX_MAX_ACTIONS         => 0,
                      Metrics::INDEX_SUM_VISIT_LENGTH    => 0,
                      Metrics::INDEX_BOUNCE_COUNT        => 0,
@@ -90,6 +91,7 @@ class DataArray
             if ($onlyMetricsAvailableInActionsTable) {
                 return;
             }
+            $oldRowToUpdate[Metrics::INDEX_NB_USERS] += $newRowToAdd['nb_users'];
             $oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS] = (float)max($newRowToAdd['max_actions'], $oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS]);
             $oldRowToUpdate[Metrics::INDEX_SUM_VISIT_LENGTH] += $newRowToAdd['sum_visit_length'];
             $oldRowToUpdate[Metrics::INDEX_BOUNCE_COUNT] += $newRowToAdd['bounce_count'];
@@ -97,6 +99,11 @@ class DataArray
             return;
         }
 
+        // Edge case fail safe
+        if(!isset($oldRowToUpdate[Metrics::INDEX_NB_VISITS])) {
+            return;
+        }
+
         $oldRowToUpdate[Metrics::INDEX_NB_VISITS] += $newRowToAdd[Metrics::INDEX_NB_VISITS];
         $oldRowToUpdate[Metrics::INDEX_NB_ACTIONS] += $newRowToAdd[Metrics::INDEX_NB_ACTIONS];
         $oldRowToUpdate[Metrics::INDEX_NB_UNIQ_VISITORS] += $newRowToAdd[Metrics::INDEX_NB_UNIQ_VISITORS];
@@ -106,16 +113,20 @@ class DataArray
 
         // In case the existing Row had no action metrics (eg. Custom Variable XYZ with "visit" scope)
         // but the new Row has action metrics (eg. same Custom Variable XYZ this time with a "page" scope)
-        if(!isset($oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS])) {
-            $toZero = array(Metrics::INDEX_MAX_ACTIONS,
+        if (!isset($oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS])) {
+            $toZero = array(
+                            Metrics::INDEX_NB_USERS,
+                            Metrics::INDEX_MAX_ACTIONS,
                             Metrics::INDEX_SUM_VISIT_LENGTH,
                             Metrics::INDEX_BOUNCE_COUNT,
-                            Metrics::INDEX_NB_VISITS_CONVERTED);
+                            Metrics::INDEX_NB_VISITS_CONVERTED
+            );
             foreach($toZero as $metric) {
                 $oldRowToUpdate[$metric] = 0;
             }
         }
 
+        $oldRowToUpdate[Metrics::INDEX_NB_USERS] += $newRowToAdd[Metrics::INDEX_NB_USERS];
         $oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS] = (float)max($newRowToAdd[Metrics::INDEX_MAX_ACTIONS], $oldRowToUpdate[Metrics::INDEX_MAX_ACTIONS]);
         $oldRowToUpdate[Metrics::INDEX_SUM_VISIT_LENGTH] += $newRowToAdd[Metrics::INDEX_SUM_VISIT_LENGTH];
         $oldRowToUpdate[Metrics::INDEX_BOUNCE_COUNT] += $newRowToAdd[Metrics::INDEX_BOUNCE_COUNT];
@@ -195,7 +206,7 @@ class DataArray
         $this->doSumVisitsMetrics($row, $this->data[$label], $onlyMetricsAvailableInActionsTable = true);
     }
 
-    static protected function makeEmptyActionRow()
+    protected static function makeEmptyActionRow()
     {
         return array(
             Metrics::INDEX_NB_UNIQ_VISITORS => 0,
@@ -212,7 +223,7 @@ class DataArray
         $this->doSumEventsMetrics($row, $this->data[$label], $onlyMetricsAvailableInActionsTable = true);
     }
 
-    static protected function makeEmptyEventRow()
+    protected static function makeEmptyEventRow()
     {
         return array(
             Metrics::INDEX_NB_UNIQ_VISITORS         => 0,
@@ -244,8 +255,8 @@ class DataArray
         $oldRowToUpdate[Metrics::INDEX_EVENT_MAX_EVENT_VALUE] = round(max($newRowToAdd[Metrics::INDEX_EVENT_MAX_EVENT_VALUE], $oldRowToUpdate[Metrics::INDEX_EVENT_MAX_EVENT_VALUE]), self::EVENT_VALUE_PRECISION);
 
         // Update minimum only if it is set
-        if($newRowToAdd[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] !== false) {
-            if($oldRowToUpdate[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] === false) {
+        if ($newRowToAdd[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] !== false) {
+            if ($oldRowToUpdate[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] === false) {
                 $oldRowToUpdate[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] = round($newRowToAdd[Metrics::INDEX_EVENT_MIN_EVENT_VALUE], self::EVENT_VALUE_PRECISION);
             } else {
                 $oldRowToUpdate[Metrics::INDEX_EVENT_MIN_EVENT_VALUE] = round(min($newRowToAdd[Metrics::INDEX_EVENT_MIN_EVENT_VALUE], $oldRowToUpdate[Metrics::INDEX_EVENT_MIN_EVENT_VALUE]), self::EVENT_VALUE_PRECISION);
@@ -332,7 +343,7 @@ class DataArray
      */
     protected function enrichWithConversions(&$data)
     {
-        foreach ($data as $label => &$values) {
+        foreach ($data as &$values) {
             if (!isset($values[Metrics::INDEX_GOALS])) {
                 continue;
             }
@@ -357,7 +368,7 @@ class DataArray
 
             // if there are no "visit" column, we force one to prevent future complications
             // eg. This helps the setDefaultColumnsToDisplay() call
-            if(!isset($values[Metrics::INDEX_NB_VISITS])) {
+            if (!isset($values[Metrics::INDEX_NB_VISITS])) {
                 $values[Metrics::INDEX_NB_VISITS] = 0;
             }
         }
@@ -369,7 +380,7 @@ class DataArray
      * @param $row
      * @return bool
      */
-    static public function isRowActions($row)
+    public static function isRowActions($row)
     {
         return (count($row) == count(self::makeEmptyActionRow())) && isset($row[Metrics::INDEX_NB_ACTIONS]);
     }
