@@ -180,6 +180,7 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'www.qbyrd.com'                  => array('Ask'),
         '{}.qbyrd.com'                   => array('Ask'),
         'www.search-results.com'         => array('Ask'),
+        'www1.search-results.com'        => array('Ask'),
         'int.search-results.com'         => array('Ask'),
         '{}.search-results.com'          => array('Ask'),
         'search.ask.com'                 => array('Ask'),
@@ -187,12 +188,14 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'avira-int.ask.com'              => array('Ask'),
         'searchqu.com'                   => array('Ask'),
         'search.tb.ask.com'              => array('Ask'),
+        'nortonsafe.search.ask.com'      => array('Ask'),
 
         // Atlas
         'searchatlas.centrum.cz'         => array('Atlas', 'q', '?q={k}'),
 
         // auone
         'search.auone.jp'                => array('auone', 'q', '?q={k}'),
+        'sp-image.search.auone.jp'       => array('auone Images', 'q', '?q={k}'),
 
         // Austronaut
         'www2.austronaut.at'             => array('Austronaut', 'q'),
@@ -349,6 +352,9 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         // Francite
         'recherche.francite.com'         => array('Francite', 'name'),
 
+        // Findhurtig
+        'www.findhurtig.dk'              => array('Findhurtig', 'q', 'web?q={k}'),
+
         // Fireball
         'www.fireball.de'                => array('Fireball', 'q', 'ajax.asp?q={k}'),
 
@@ -383,6 +389,9 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         // GAIS
         'gais.cs.ccu.edu.tw'             => array('GAIS', 'q', 'search.php?q={k}'),
 
+        // Genieo
+        'search.genieo.com'              => array('Genieo', 'q', '&q={k}'),
+
         // Geona
         'geona.net'                      => array('Geona', 'q', 'search?q={k}'),
 
@@ -451,8 +460,8 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'www.optuszoo.com.au'            => array('Google'),
         'search.dolphin-browser.jp'      => array('Google'),
         'search.smt.docomo.ne.jp'        => array('Google', 'MT'),
-        'image.search.smt.docomo.ne.jp'     => array('Google', 'MT'),
-
+        'image.search.smt.docomo.ne.jp'  => array('Google', 'MT'),
+        'gfsoso.com'                     => array('Google', 'q'),
 
         // Google Earth
         // - 2010-09-13: are these redirects now?
@@ -573,6 +582,8 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'searchya.com'                   => array('InfoSpace', 'q'),
         'search.handycafe.com'           => array('InfoSpace', 'q'),
         'search.v9.com'                  => array('InfoSpace', 'q'),
+        'search.iminent.com'             => array('InfoSpace', 'q'),
+        'utorrent.inspsearch.com'        => array('InfoSpace', 'q'),
 
         /*
          * Other InfoSpace powered metasearches are handled in Common::extractSearchEngineInformationFromUrl()
@@ -614,6 +625,9 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'junglekey.com'                  => array('Jungle Key', 'query', 'search.php?query={k}&type=web&lang=en'),
         'junglekey.fr'                   => array('Jungle Key'),
 
+        // K9 Safe Search
+        'k9safesearch.com'               => array('K9 Safe Search', 'q', 'search.jsp?q={k}'),
+
         // Kataweb
         'www.kataweb.it'                 => array('Kataweb', 'q'),
 
@@ -695,7 +709,6 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'search.myway.com'               => array('MyWebSearch'),
         'search.mywebsearch.com'         => array('MyWebSearch'),
 
-
         // Najdi
         'www.najdi.si'                   => array('Najdi.si', 'q', 'search.jsp?q={k}'),
 
@@ -713,6 +726,7 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
 
         // Nifty
         'search.nifty.com'               => array('Nifty', array('q', 'Text'), 'websearch/search?q={k}'),
+        'search.azby.fmworld.net'        => array('Nifty'),
         'videosearch.nifty.com'          => array('Nifty Videos', 'kw', 'search?kw={k}'),
 
         // Nigma
@@ -804,6 +818,9 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
 
         // Seznam
         'search.seznam.cz'               => array('Seznam', 'q', '?q={k}'),
+        
+        // Seznam Videa (Video)
+        'videa.seznam.cz'               => array('Seznam Videa', 'q', '?q={k}'),
 
         // Sharelook
         'www.sharelook.fr'               => array('Sharelook', 'keyword'),
@@ -833,6 +850,9 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         // soso.com
         'www.soso.com'                   => array('Soso', 'w', 'q?w={k}', 'gb2312'),
 
+        // sputnik.ru
+        'www.sputnik.ru'                 => array('Sputnik', 'q', 'search?q={k}'),
+
         // Startpagina
         'startgoogle.startpagina.nl'     => array('Startpagina (Google)', 'q', '?q={k}'),
 
@@ -968,12 +988,11 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
         'www.x-recherche.com'            => array('X-Recherche', 'MOTS', 'cgi-bin/websearch?MOTS={k}'),
 
         // Yahoo! Japan
-        'search.yahoo.co.jp'             => array('Yahoo! Japan', 'p', 'search?p={k}'),
+        'search.yahoo.co.jp'             => array('Yahoo! Japan', array('p', 'vp'), 'search?p={k}'),
         'jp.hao123.com'                  => array('Yahoo! Japan', 'query'),
         'video.search.yahoo.co.jp'       => array('Yahoo! Japan Videos', 'p', 'search?p={k}'),
         'image.search.yahoo.co.jp'       => array('Yahoo! Japan Images', 'p', 'search?p={k}'),
 
-
         // Yahoo
         'search.yahoo.com'               => array('Yahoo!', array('p', 'q'), 'search?p={k}'),
 //		'*.search.yahoo.com'		=> array('Yahoo!'), // see built-in helper in Common.php
@@ -1040,9 +1059,14 @@ if (!isset($GLOBALS['Piwik_SearchEngines'])) {
 
         // Zapmeta
         'www.zapmeta.com'                => array('Zapmeta', array('q', 'query'), '?q={k}'),
-        'www.zapmeta.nl'                 => array('Zapmeta'),
-        'www.zapmeta.de'                 => array('Zapmeta'),
+        'zapmeta.{}'                     => array('Zapmeta'),
         'uk.zapmeta.com'                 => array('Zapmeta'),
+        'ar.zapmeta.com'                 => array('Zapmeta'),
+        'au.zapmeta.com'                 => array('Zapmeta'),
+        'ca.zapmeta.com'                 => array('Zapmeta'),
+        'fi.zapmeta.com'                 => array('Zapmeta'),
+        'no.zapmeta.com'                 => array('Zapmeta'),
+        'tr.zapmeta.com'                 => array('Zapmeta'),
 
         // Zoek
         'www3.zoek.nl'                   => array('Zoek', 'q'),
