@@ -27,83 +27,83 @@ require_once PIWIK_INCLUDE_PATH . '/core/Common.php';
 
 /**
  * The primary data structure used to store analytics data in Piwik.
- * 
+ *
  * <a name="class-desc-the-basics"></a>
  * ### The Basics
- * 
+ *
  * DataTables consist of rows and each row consists of columns. A column value can be
  * a numeric, a string or an array.
- * 
+ *
  * Every row has an ID. The ID is either the index of the row or {@link ID_SUMMARY_ROW}.
- * 
+ *
  * DataTables are hierarchical data structures. Each row can also contain an additional
  * nested sub-DataTable (commonly referred to as a 'subtable').
- * 
+ *
  * Both DataTables and DataTable rows can hold **metadata**. _DataTable metadata_ is information
  * regarding all the data, such as the site or period that the data is for. _Row metadata_
  * is information regarding that row, such as a browser logo or website URL.
- * 
+ *
  * Finally, all DataTables contain a special _summary_ row. This row, if it exists, is
  * always at the end of the DataTable.
- * 
+ *
  * ### Populating DataTables
- * 
+ *
  * Data can be added to DataTables in three different ways. You can either:
- * 
+ *
  * 1. create rows one by one and add them through {@link addRow()} then truncate if desired,
  * 2. create an array of DataTable\Row instances or an array of arrays and add them using
  *    {@link addRowsFromArray()} or {@link addRowsFromSimpleArray()}
  *    then truncate if desired,
  * 3. or set the maximum number of allowed rows (with {@link setMaximumAllowedRows()})
  *    and add rows one by one.
- * 
+ *
  * If you want to eventually truncate your data (standard practice for all Piwik plugins),
  * the third method is the most memory efficient. It is, unfortunately, not always possible
  * to use since it requires that the data be sorted before adding.
- * 
+ *
  * ### Manipulating DataTables
- * 
+ *
  * There are two ways to manipulate a DataTable. You can either:
- * 
+ *
  * 1. manually iterate through each row and manipulate the data,
  * 2. or you can use predefined filters.
- * 
+ *
  * A filter is a class that has a 'filter' method which will manipulate a DataTable in
  * some way. There are several predefined Filters that allow you to do common things,
  * such as,
- * 
+ *
  * - add a new column to each row,
  * - add new metadata to each row,
  * - modify an existing column value for each row,
  * - sort an entire DataTable,
  * - and more.
- * 
+ *
  * Using these filters instead of writing your own code will increase code clarity and
  * reduce code redundancy. Additionally, filters have the advantage that they can be
  * applied to DataTable\Map instances. So you can visit every DataTable in a {@link DataTable\Map}
  * without having to write a recursive visiting function.
- * 
+ *
  * All predefined filters exist in the **Piwik\DataTable\BaseFilter** namespace.
- * 
+ *
  * _Note: For convenience, [anonymous functions](http://www.php.net/manual/en/functions.anonymous.php)
  * can be used as DataTable filters._
- * 
+ *
  * ### Applying Filters
- * 
+ *
  * Filters can be applied now (via {@link filter()}), or they can be applied later (via
  * {@link queueFilter()}).
- * 
+ *
  * Filters that sort rows or manipulate the number of rows should be applied right away.
  * Non-essential, presentation filters should be queued.
- * 
+ *
  * ### Learn more
- * 
+ *
  * - See **{@link ArchiveProcessor}** to learn how DataTables are persisted.
- * 
+ *
  * ### Examples
- * 
+ *
  * **Populating a DataTable**
- * 
+ *
  *     // adding one row at a time
  *     $dataTable = new DataTable();
  *     $dataTable->addRow(new Row(array(
@@ -114,7 +114,7 @@ require_once PIWIK_INCLUDE_PATH . '/core/Common.php';
  *         Row::COLUMNS => array('label' => 'thing2', 'nb_visits' => 2, 'nb_actions' => 2),
  *         Row::METADATA => array('url' => 'http://thing2.com')
  *     )));
- *     
+ *
  *     // using an array of rows
  *     $dataTable = new DataTable();
  *     $dataTable->addRowsFromArray(array(
@@ -127,32 +127,32 @@ require_once PIWIK_INCLUDE_PATH . '/core/Common.php';
  *             Row::METADATA => array('url' => 'http://thing2.com')
  *         )
  *     ));
- * 
+ *
  *     // using a "simple" array
  *     $dataTable->addRowsFromSimpleArray(array(
  *         array('label' => 'thing1', 'nb_visits' => 1, 'nb_actions' => 1),
  *         array('label' => 'thing2', 'nb_visits' => 2, 'nb_actions' => 2)
  *     ));
- * 
+ *
  * **Getting & setting metadata**
- * 
+ *
  *     $dataTable = \Piwik\Plugins\Referrers\API::getInstance()->getSearchEngines($idSite = 1, $period = 'day', $date = '2007-07-24');
  *     $oldPeriod = $dataTable->metadata['period'];
  *     $dataTable->metadata['period'] = Period\Factory::build('week', Date::factory('2013-10-18'));
- * 
+ *
  * **Serializing & unserializing**
- * 
+ *
  *     $maxRowsInTable = Config::getInstance()->General['datatable_archiving_maximum_rows_standard'];j
- *     
+ *
  *     $dataTable = // ... build by aggregating visits ...
  *     $serializedData = $dataTable->getSerialized($maxRowsInTable, $maxRowsInSubtable = $maxRowsInTable,
  *                                                 $columnToSortBy = Metrics::INDEX_NB_VISITS);
- *     
+ *
  *     $serializedDataTable = $serializedData[0];
  *     $serailizedSubTable = $serializedData[$idSubtable];
- * 
+ *
  * **Filtering for an API method**
- * 
+ *
  *     public function getMyReport($idSite, $period, $date, $segment = false, $expanded = false)
  *     {
  *         $dataTable = Archive::getDataTableFromArchive('MyPlugin_MyReport', $idSite, $period, $date, $segment, $expanded);
@@ -161,11 +161,11 @@ require_once PIWIK_INCLUDE_PATH . '/core/Common.php';
  *         $dataTable->queueFilter('ColumnCallbackAddMetadata', array('label', 'url', __NAMESPACE__ . '\getUrlFromLabelForMyReport'));
  *         return $dataTable;
  *     }
- * 
+ *
  *
  * @api
  */
-class DataTable implements DataTableInterface
+class DataTable implements DataTableInterface, \IteratorAggregate, \ArrayAccess
 {
     const MAX_DEPTH_DEFAULT = 15;
 
@@ -181,14 +181,14 @@ class DataTable implements DataTableInterface
     /**
      * Name for metadata that describes how individual columns should be aggregated when {@link addDataTable()}
      * or {@link Piwik\DataTable\Row::sumRow()} is called.
-     * 
+     *
      * This metadata value must be an array that maps column names with valid operations. Valid aggregation operations are:
-     * 
+     *
      * - `'skip'`: do nothing
      * - `'max'`: does `max($column1, $column2)`
      * - `'min'`: does `min($column1, $column2)`
      * - `'sum'`: does `$column1 + $column2`
-     * 
+     *
      * See {@link addDataTable()} and {@link DataTable\Row::sumRow()} for more information.
      */
     const COLUMN_AGGREGATION_OPS_METADATA_NAME = 'column_aggregation_ops';
@@ -199,6 +199,13 @@ class DataTable implements DataTableInterface
     /** The original label of the Summary Row. */
     const LABEL_SUMMARY_ROW = -1;
 
+    /**
+     * Name for metadata that contains extra {@link Piwik\Plugin\ProcessedMetric}s for a DataTable.
+     * These metrics will be added in addition to the ones specified in the table's associated
+     * {@link Piwik\Plugin\Report} class.
+     */
+    const EXTRA_PROCESSED_METRICS_METADATA_NAME = 'extra_processed_metrics';
+
     /**
      * Maximum nesting level.
      */
@@ -290,7 +297,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Table metadata. Read [this](#class-desc-the-basics) to learn more.
-     * 
+     *
      * Any data that describes the data held in the table's rows should go here.
      *
      * @var array
@@ -349,10 +356,11 @@ class DataTable implements DataTableInterface
 
         if ($this->enableRecursiveSort === true) {
             foreach ($this->getRows() as $row) {
-                if (($idSubtable = $row->getIdSubDataTable()) !== null) {
-                    $table = Manager::getInstance()->getTable($idSubtable);
-                    $table->enableRecursiveSort();
-                    $table->sort($functionCallback, $columnSortedBy);
+
+                $subTable = $row->getSubtable();
+                if ($subTable) {
+                    $subTable->enableRecursiveSort();
+                    $subTable->sort($functionCallback, $columnSortedBy);
                 }
             }
         }
@@ -390,7 +398,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Applies a filter to this datatable.
-     * 
+     *
      * If {@link enableRecursiveFilters()} was called, the filter will be applied
      * to all subtables as well.
      *
@@ -428,7 +436,7 @@ class DataTable implements DataTableInterface
     /**
      * Adds a filter and a list of parameters to the list of queued filters. These filters will be
      * executed when {@link applyQueuedFilters()} is called.
-     * 
+     *
      * Filters that prettify the column values or don't need the full set of rows should be queued. This
      * way they will be run after the table is truncated which will result in better performance.
      *
@@ -452,28 +460,30 @@ class DataTable implements DataTableInterface
         foreach ($this->queuedFilters as $filter) {
             $this->filter($filter['className'], $filter['parameters']);
         }
-        $this->queuedFilters = array();
+        $this->clearQueuedFilters();
     }
 
     /**
      * Sums a DataTable to this one.
-     * 
+     *
      * This method will sum rows that have the same label. If a row is found in `$tableToSum` whose
      * label is not found in `$this`, the row will be added to `$this`.
-     * 
+     *
      * If the subtables for this table are loaded, they will be summed as well.
-     * 
+     *
      * Rows are summed together by summing individual columns. By default columns are summed by
      * adding one column value to another. Some columns cannot be aggregated this way. In these
      * cases, the {@link COLUMN_AGGREGATION_OPS_METADATA_NAME}
      * metadata can be used to specify a different type of operation.
-     * 
+     *
      * @param \Piwik\DataTable $tableToSum
+     * @param bool $doAggregateSubTables
+     * @throws Exception
      */
     public function addDataTable(DataTable $tableToSum, $doAggregateSubTables = true)
     {
-        if($tableToSum instanceof Simple) {
-            if($tableToSum->getRowsCount() > 1) {
+        if ($tableToSum instanceof Simple) {
+            if ($tableToSum->getRowsCount() > 1) {
                 throw new Exception("Did not expect a Simple table with more than one row in addDataTable()");
             }
             $row = $tableToSum->getFirstRow();
@@ -487,10 +497,10 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns the Row whose `'label'` column is equal to `$label`.
-     * 
+     *
      * This method executes in constant time except for the first call which caches row
      * label => row ID mappings.
-     * 
+     *
      * @param string $label `'label'` column value to look for.
      * @return Row|false The row if found, `false` if otherwise.
      */
@@ -516,7 +526,7 @@ class DataTable implements DataTableInterface
      *
      * This method executes in constant time except for the first call which caches row
      * label => row ID mappings.
-     * 
+     *
      * @param string $label `'label'` column value to look for.
      * @return int The row ID.
      */
@@ -591,7 +601,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns the row that has a subtable with ID matching `$idSubtable`.
-     * 
+     *
      * @param int $idSubTable The subtable ID.
      * @return Row|false The row or false if not found
      */
@@ -608,7 +618,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Adds a row to this table.
-     * 
+     *
      * If {@link setMaximumAllowedRows()} was called and the current row count is
      * at the maximum, the new row will be summed to the summary row. If there is no summary row,
      * this row is set as the summary row.
@@ -648,7 +658,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Sets the summary row.
-     * 
+     *
      * _Note: A DataTable can have only one summary row._
      *
      * @param Row $row
@@ -683,7 +693,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Adds a new row from an array.
-     * 
+     *
      * You can add row metadata with this method.
      *
      * @param array $row eg. `array(Row::COLUMNS => array('visits' => 13, 'test' => 'toto'),
@@ -696,7 +706,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Adds a new row a from an array of column values.
-     * 
+     *
      * Row metadata cannot be added with this method.
      *
      * @param array $row eg. `array('name' => 'google analytics', 'license' => 'commercial')`
@@ -758,10 +768,10 @@ class DataTable implements DataTableInterface
     /**
      * Returns the names of every column this DataTable contains. This method will return the
      * columns of the first row with data and will assume they occur in every other row as well.
-     * 
+     *
      *_ Note: If column names still use their in-database INDEX values (@see Metrics), they
      *        will be converted to their string name in the array result._
-     * 
+     *
      * @return array Array of string column names.
      */
     public function getColumns()
@@ -787,7 +797,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns an array containing the requested metadata value of each row.
-     * 
+     *
      * @param string $name The metadata column to return.
      * @return array
      */
@@ -802,7 +812,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns the number of rows in the table including the summary row.
-     * 
+     *
      * @return int
      */
     public function getRowsCount()
@@ -859,8 +869,8 @@ class DataTable implements DataTableInterface
     {
         $totalCount = 0;
         foreach ($this->rows as $row) {
-            if (($idSubTable = $row->getIdSubDataTable()) !== null) {
-                $subTable = Manager::getInstance()->getTable($idSubTable);
+            $subTable = $row->getSubtable();
+            if ($subTable) {
                 $count = $subTable->getRowsCountRecursive();
                 $totalCount += $count;
             }
@@ -897,9 +907,10 @@ class DataTable implements DataTableInterface
         foreach ($this->getRows() as $row) {
             $row->renameColumn($oldName, $newName);
 
-            if($doRenameColumnsOfSubTables) {
-                if (($idSubDataTable = $row->getIdSubDataTable()) !== null) {
-                    Manager::getInstance()->getTable($idSubDataTable)->renameColumn($oldName, $newName);
+            if ($doRenameColumnsOfSubTables) {
+                $subTable = $row->getSubtable();
+                if ($subTable) {
+                    $subTable->renameColumn($oldName, $newName);
                 }
             }
         }
@@ -920,8 +931,9 @@ class DataTable implements DataTableInterface
             foreach ($names as $name) {
                 $row->deleteColumn($name);
             }
-            if (($idSubDataTable = $row->getIdSubDataTable()) !== null) {
-                Manager::getInstance()->getTable($idSubDataTable)->deleteColumns($names, $deleteRecursiveInSubtables);
+            $subTable = $row->getSubtable();
+            if ($subTable) {
+                $subTable->deleteColumns($names, $deleteRecursiveInSubtables);
             }
         }
         if (!is_null($this->summaryRow)) {
@@ -999,7 +1011,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns a string representation of this DataTable for convenient viewing.
-     * 
+     *
      * _Note: This uses the **html** DataTable renderer._
      *
      * @return string
@@ -1018,7 +1030,7 @@ class DataTable implements DataTableInterface
      * each row has a label that exists in the other table, and if each row
      * is equal to the row in the other table with the same label. The order
      * of rows is not important.
-     * 
+     *
      * @param \Piwik\DataTable $table1
      * @param \Piwik\DataTable $table2
      * @return bool
@@ -1026,7 +1038,6 @@ class DataTable implements DataTableInterface
     public static function isEqual(DataTable $table1, DataTable $table2)
     {
         $rows1 = $table1->getRows();
-        $rows2 = $table2->getRows();
 
         $table1->rebuildIndex();
         $table2->rebuildIndex();
@@ -1049,10 +1060,10 @@ class DataTable implements DataTableInterface
 
     /**
      * Serializes an entire DataTable hierarchy and returns the array of serialized DataTables.
-     * 
+     *
      * The first element in the returned array will be the serialized representation of this DataTable.
      * Every subsequent element will be a serialized subtable.
-     * 
+     *
      * This DataTable and subtables can optionally be truncated before being serialized. In most
      * cases where DataTables can become quite large, they should be truncated before being persisted
      * in an archive.
@@ -1064,7 +1075,7 @@ class DataTable implements DataTableInterface
      * @param int $maximumRowsInSubDataTable If not null, defines the maximum number of rows allowed in serialized subtables.
      * @param string $columnToSortByBeforeTruncation The column to sort by before truncating, eg, `Metrics::INDEX_NB_VISITS`.
      * @return array The array of serialized DataTables:
-     * 
+     *
      *                   array(
      *                       // this DataTable (the root)
      *                       0 => 'eghuighahgaueytae78yaet7yaetae',
@@ -1074,7 +1085,7 @@ class DataTable implements DataTableInterface
      *
      *                       // another subtable
      *                       2 => 'gqegJHUIGHEQjkgneqjgnqeugUGEQHGUHQE',
-     *                    
+     *
      *                       // etc.
      *                   );
      */
@@ -1102,20 +1113,13 @@ class DataTable implements DataTableInterface
         // but returns all serialized tables and subtable in an array of 1 dimension
         $aSerializedDataTable = array();
         foreach ($this->rows as $row) {
-            if (($idSubTable = $row->getIdSubDataTable()) !== null) {
-                $subTable = null;
-                try {
-                    $subTable = Manager::getInstance()->getTable($idSubTable);
-                } catch(TableNotFoundException $e) {
-                    // This occurs is an unknown & random data issue. Catch Exception and remove subtable from the row.
-                    $row->removeSubtable();
-                    // Go to next row
-                    continue;
-                }
-
+            $subTable = $row->getSubtable();
+            if ($subTable) {
                 $depth++;
                 $aSerializedDataTable = $aSerializedDataTable + $subTable->getSerialized($maximumRowsInSubDataTable, $maximumRowsInSubDataTable, $columnToSortByBeforeTruncation);
                 $depth--;
+            } else {
+                $row->removeSubtable();
             }
         }
         // we load the current Id of the DataTable
@@ -1141,9 +1145,9 @@ class DataTable implements DataTableInterface
      * Adds a set of rows from a serialized DataTable string.
      *
      * See {@link serialize()}.
-     * 
+     *
      * _Note: This function will successfully load DataTables serialized by Piwik 1.X._
-     * 
+     *
      * @param string $stringSerialized A string with the format of a string in the array returned by
      *                                 {@link serialize()}.
      * @throws Exception if `$stringSerialized` is invalid.
@@ -1151,7 +1155,7 @@ class DataTable implements DataTableInterface
     public function addRowsFromSerializedArray($stringSerialized)
     {
         require_once PIWIK_INCLUDE_PATH . "/core/DataTable/Bridges.php";
-        
+
         $serialized = unserialize($stringSerialized);
         if ($serialized === false) {
             throw new Exception("The unserialization has failed!");
@@ -1161,11 +1165,11 @@ class DataTable implements DataTableInterface
 
     /**
      * Adds multiple rows from an array.
-     * 
+     *
      * You can add row metadata with this method.
      *
      * @param array $array Array with the following structure
-     * 
+     *
      *                         array(
      *                             // row1
      *                             array(
@@ -1192,11 +1196,11 @@ class DataTable implements DataTableInterface
 
     /**
      * Adds multiple rows from an array containing arrays of column values.
-     * 
+     *
      * Row metadata cannot be added with this method.
      *
      * @param array $array Array with the following structure:
-     * 
+     *
      *                       array(
      *                             array( col1_name => valueA, col2_name => valueC, ...),
      *                             array( col1_name => valueB, col2_name => valueD, ...),
@@ -1273,28 +1277,28 @@ class DataTable implements DataTableInterface
 
     /**
      * Rewrites the input `$array`
-     * 
+     *
      *     array (
      *         LABEL => array(col1 => X, col2 => Y),
      *         LABEL2 => array(col1 => X, col2 => Y),
      *     )
-     * 
+     *
      * to a DataTable with rows that look like:
-     * 
+     *
      *     array (
      *         array( Row::COLUMNS => array('label' => LABEL, col1 => X, col2 => Y)),
      *         array( Row::COLUMNS => array('label' => LABEL2, col1 => X, col2 => Y)),
      *     )
      *
-     * Will also convert arrays like: 
-     * 
+     * Will also convert arrays like:
+     *
      *     array (
      *         LABEL => X,
      *         LABEL2 => Y,
      *     )
-     * 
+     *
      * to:
-     * 
+     *
      *     array (
      *         array( Row::COLUMNS => array('label' => LABEL, 'value' => X)),
      *         array( Row::COLUMNS => array('label' => LABEL2, 'value' => Y)),
@@ -1328,11 +1332,11 @@ class DataTable implements DataTableInterface
     /**
      * Sets the maximum depth level to at least a certain value. If the current value is
      * greater than `$atLeastLevel`, the maximum nesting level is not changed.
-     * 
+     *
      * The maximum depth level determines the maximum number of subtable levels in the
      * DataTable tree. For example, if it is set to `2`, this DataTable is allowed to
      * have subtables, but the subtables are not.
-     * 
+     *
      * @param int $atLeastLevel
      */
     public static function setMaximumDepthLevelAllowedAtLeast($atLeastLevel)
@@ -1380,7 +1384,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Sets several metadata values by name.
-     * 
+     *
      * @param array $values Array mapping metadata names with metadata values.
      */
     public function setMetadataValues($values)
@@ -1392,7 +1396,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Sets metadata, erasing existing values.
-     * 
+     *
      * @param array $values Array mapping metadata names with metadata values.
      */
     public function setAllTableMetadata($metadata)
@@ -1416,14 +1420,14 @@ class DataTable implements DataTableInterface
      * Traverses a DataTable tree using an array of labels and returns the row
      * it finds or `false` if it cannot find one. The number of path segments that
      * were successfully walked is also returned.
-     * 
+     *
      * If `$missingRowColumns` is supplied, the specified path is created. When
      * a subtable is encountered w/o the required label, a new row is created
      * with the label, and a new subtable is added to the row.
      *
      * Read [http://en.wikipedia.org/wiki/Tree_(data_structure)#Traversal_methods](http://en.wikipedia.org/wiki/Tree_(data_structure)#Traversal_methods)
      * for more information about tree walking.
-     * 
+     *
      * @param array $path The path to walk. An array of label values. The first element
      *                    refers to a row in this DataTable, the second in a subtable of
      *                    the first row, the third a subtable of the second row, etc.
@@ -1494,7 +1498,7 @@ class DataTable implements DataTableInterface
      *
      * @param string|bool $labelColumn If supplied the label of the parent row will be added to
      *                                 a new column in each subtable row.
-     * 
+     *
      *                                 If set to, `'label'` each subtable row's label will be prepended
      *                                 w/ the parent row's label. So `'child_label'` becomes
      *                                 `'parent_label - child_label'`.
@@ -1550,9 +1554,9 @@ class DataTable implements DataTableInterface
 
     /**
      * Returns a new DataTable created with data from a 'simple' array.
-     * 
+     *
      * See {@link addRowsFromSimpleArray()}.
-     * 
+     *
      * @param array $array
      * @return \Piwik\DataTable
      */
@@ -1565,7 +1569,7 @@ class DataTable implements DataTableInterface
 
     /**
      * Creates a new DataTable instance from a serialized DataTable string.
-     * 
+     *
      * See {@link getSerialized()} and {@link addRowsFromSerializedArray()}
      * for more information on DataTable serialization.
      *
@@ -1603,13 +1607,13 @@ class DataTable implements DataTableInterface
         } else {
             $rowFound->sumRow($row, $copyMeta = true, $this->getMetadata(self::COLUMN_AGGREGATION_OPS_METADATA_NAME));
 
-            if($doAggregateSubTables) {
+            if ($doAggregateSubTables) {
                 // if the row to add has a subtable whereas the current row doesn't
                 // we simply add it (cloning the subtable)
                 // if the row has the subtable already
                 // then we have to recursively sum the subtables
-                if (($idSubTable = $row->getIdSubDataTable()) !== null) {
-                    $subTable = Manager::getInstance()->getTable($idSubTable);
+                $subTable = $row->getSubtable();
+                if ($subTable) {
                     $subTable->metadata[self::COLUMN_AGGREGATION_OPS_METADATA_NAME]
                         = $this->getMetadata(self::COLUMN_AGGREGATION_OPS_METADATA_NAME);
                     $rowFound->sumSubtable($subTable);
@@ -1634,4 +1638,41 @@ class DataTable implements DataTableInterface
         }
         $thisRow->sumRow($row, $copyMeta = true, $this->getMetadata(self::COLUMN_AGGREGATION_OPS_METADATA_NAME));
     }
+
+    /**
+     * Unsets all queued filters.
+     */
+    public function clearQueuedFilters()
+    {
+        $this->queuedFilters = array();
+    }
+
+    /**
+     * @return \ArrayIterator|Row[]
+     */
+    public function getIterator() {
+        return new \ArrayIterator($this->getRows());
+    }
+
+    public function offsetExists($offset)
+    {
+        $row = $this->getRowFromId($offset);
+
+        return false !== $row;
+    }
+
+    public function offsetGet($offset)
+    {
+        return $this->getRowFromId($offset);
+    }
+
+    public function offsetSet($offset, $value)
+    {
+        $this->rows[$offset] = $value;
+    }
+
+    public function offsetUnset($offset)
+    {
+        $this->deleteRow($offset);
+    }
 }
\ No newline at end of file
