@@ -15,14 +15,14 @@ use Piwik\DataTable\Row;
 /**
  * Calculates the quotient of two columns and adds the result as a new column
  * for each row of a DataTable.
- * 
+ *
  * This filter is used to calculate rate values (eg, `'bounce_rate'`), averages
  * (eg, `'avg_time_on_page'`) and other types of values.
  *
  * **Basic usage example**
- * 
+ *
  *     $dataTable->queueFilter('ColumnCallbackAddColumnQuotient', array('bounce_rate', 'bounce_count', 'nb_visits', $precision = 2));
- * 
+ *
  * @api
  */
 class ColumnCallbackAddColumnQuotient extends BaseFilter
@@ -38,7 +38,7 @@ class ColumnCallbackAddColumnQuotient extends BaseFilter
 
     /**
      * Constructor.
-     * 
+     *
      * @param DataTable $table The DataTable that will eventually be filtered.
      * @param string $columnNameToAdd The name of the column to add the quotient value to.
      * @param string $columnValueToRead The name of the column that holds the dividend.
@@ -75,7 +75,7 @@ class ColumnCallbackAddColumnQuotient extends BaseFilter
      */
     public function filter($table)
     {
-        foreach ($table->getRows() as $key => $row) {
+        foreach ($table->getRows() as $row) {
             $value = $this->getDividend($row);
             if ($value === false && $this->shouldSkipRows) {
                 continue;
@@ -109,6 +109,7 @@ class ColumnCallbackAddColumnQuotient extends BaseFilter
         if ($divisor > 0 && $value > 0) {
             $quotient = round($value / $divisor, $this->quotientPrecision);
         }
+
         return $quotient;
     }
 
