@@ -16,15 +16,15 @@ use Piwik\DataTable\BaseFilter;
  * whitelist or both.
  *
  * This filter is used to handle the **hideColumn** and **showColumn** query parameters.
- * 
+ *
  * **Basic usage example**
- * 
+ *
  *     $columnsToRemove = array('nb_hits', 'nb_pageviews');
  *     $dataTable->filter('ColumnDelete', array($columnsToRemove));
- * 
+ *
  *     $columnsToKeep = array('nb_visits');
  *     $dataTable->filter('ColumnDelete', array(array(), $columnsToKeep));
- * 
+ *
  * @api
  */
 class ColumnDelete extends BaseFilter
@@ -91,6 +91,7 @@ class ColumnDelete extends BaseFilter
      * See {@link ColumnDelete}.
      *
      * @param DataTable $table
+     * @return DataTable
      */
     public function filter($table)
     {
@@ -100,15 +101,16 @@ class ColumnDelete extends BaseFilter
 
         // remove columns specified in $this->columnsToRemove
         if (!empty($this->columnsToRemove)) {
-            foreach ($table->getRows() as $row) {
+            foreach ($table as $index => $row) {
                 foreach ($this->columnsToRemove as $column) {
                     if ($this->deleteIfZeroOnly) {
-                        $value = $row->getColumn($column);
+                        $value = $row[$column];
                         if ($value === false || !empty($value)) {
                             continue;
                         }
                     }
-                    $row->deleteColumn($column);
+
+                    unset($table[$index][$column]);
                 }
             }
 
@@ -117,8 +119,8 @@ class ColumnDelete extends BaseFilter
 
         // remove columns not specified in $columnsToKeep
         if (!empty($this->columnsToKeep)) {
-            foreach ($table->getRows() as $row) {
-                foreach ($row->getColumns() as $name => $value) {
+            foreach ($table as $index => $row) {
+                foreach ($row as $name => $value) {
 
                     $keep = false;
                     // @see self::APPEND_TO_COLUMN_NAME_TO_KEEP
@@ -132,7 +134,7 @@ class ColumnDelete extends BaseFilter
                         && $name != 'label' // label cannot be removed via whitelisting
                         && !isset($this->columnsToKeep[$name])
                     ) {
-                        $row->deleteColumn($name);
+                        unset($table[$index][$name]);
                     }
                 }
             }
@@ -141,10 +143,12 @@ class ColumnDelete extends BaseFilter
         }
 
         // recurse
-        if ($recurse) {
-            foreach ($table->getRows() as $row) {
+        if ($recurse && !is_array($table)) {
+            foreach ($table as $row) {
                 $this->filterSubTable($row);
             }
         }
+
+        return $table;
     }
 }
