@@ -16,7 +16,7 @@ use Piwik\Metrics;
 
 /**
  * Sorts a {@link DataTable} based on the value of a specific column.
- * 
+ *
  * It is possible to specify a natural sorting (see [php.net/natsort](http://php.net/natsort) for details).
  *
  * @api
@@ -28,7 +28,7 @@ class Sort extends BaseFilter
 
     /**
      * Constructor.
-     * 
+     *
      * @param DataTable $table The table to eventually filter.
      * @param string $columnToSort The name of the column to sort by.
      * @param string $order order `'asc'` or `'desc'`.
@@ -38,11 +38,13 @@ class Sort extends BaseFilter
     public function __construct($table, $columnToSort, $order = 'desc', $naturalSort = true, $recursiveSort = false)
     {
         parent::__construct($table);
+
         if ($recursiveSort) {
             $table->enableRecursiveSort();
         }
+
         $this->columnToSort = $columnToSort;
-        $this->naturalSort = $naturalSort;
+        $this->naturalSort  = $naturalSort;
         $this->setOrder($order);
     }
 
@@ -55,10 +57,10 @@ class Sort extends BaseFilter
     {
         if ($order == 'asc') {
             $this->order = 'asc';
-            $this->sign = 1;
+            $this->sign  = 1;
         } else {
             $this->order = 'desc';
-            $this->sign = -1;
+            $this->sign  = -1;
         }
     }
 
@@ -71,16 +73,8 @@ class Sort extends BaseFilter
      */
     public function numberSort($a, $b)
     {
-        $valA = $a->getColumn($this->columnToSort);
-        $valB = $b->getColumn($this->columnToSort);
-
-        if ($valA === false) {
-            $valA = null;
-        }
-
-        if ($valB === false) {
-            $valB = null;
-        }
+        $valA = $this->getColumnValue($a);
+        $valB = $this->getColumnValue($b);
 
         return !isset($valA)
         && !isset($valB)
@@ -116,16 +110,8 @@ class Sort extends BaseFilter
      */
     function naturalSort($a, $b)
     {
-        $valA = $a->getColumn($this->columnToSort);
-        $valB = $b->getColumn($this->columnToSort);
-
-        if ($valA === false) {
-            $valA = null;
-        }
-
-        if ($valB === false) {
-            $valB = null;
-        }
+        $valA = $this->getColumnValue($a);
+        $valB = $this->getColumnValue($b);
 
         return !isset($valA)
         && !isset($valB)
@@ -151,16 +137,8 @@ class Sort extends BaseFilter
      */
     function sortString($a, $b)
     {
-        $valA = $a->getColumn($this->columnToSort);
-        $valB = $b->getColumn($this->columnToSort);
-
-        if ($valA === false) {
-            $valA = null;
-        }
-
-        if ($valB === false) {
-            $valB = null;
-        }
+        $valA = $this->getColumnValue($a);
+        $valB = $this->getColumnValue($b);
 
         return !isset($valA)
         && !isset($valB)
@@ -177,6 +155,18 @@ class Sort extends BaseFilter
             );
     }
 
+    protected function getColumnValue(Row $table )
+    {
+        $value = $table->getColumn($this->columnToSort);
+
+        if ($value === false
+            || is_array($value)
+        ) {
+            return null;
+        }
+        return $value;
+    }
+
     /**
      * Sets the column to be used for sorting
      *
@@ -190,7 +180,7 @@ class Sort extends BaseFilter
             return $this->columnToSort;
         }
 
-        $columnIdToName = Metrics::getMappingFromIdToName();
+        $columnIdToName = Metrics::getMappingFromNameToId();
         // sorting by "nb_visits" but the index is Metrics::INDEX_NB_VISITS in the table
         if (isset($columnIdToName[$this->columnToSort])) {
             $column = $columnIdToName[$this->columnToSort];
@@ -225,17 +215,21 @@ class Sort extends BaseFilter
         if ($table instanceof Simple) {
             return;
         }
+
         if (empty($this->columnToSort)) {
             return;
         }
+
         $rows = $table->getRows();
         if (count($rows) == 0) {
             return;
         }
+
         $row = current($rows);
         if ($row === false) {
             return;
         }
+
         $this->columnToSort = $this->selectColumnToSort($row);
 
         $value = $row->getColumn($this->columnToSort);
