@@ -10,18 +10,19 @@ namespace Piwik\DataTable;
 
 use Exception;
 use Piwik\DataTable;
+use Piwik\Log;
 use Piwik\Metrics;
 
 /**
  * This is what a {@link Piwik\DataTable} is composed of.
- * 
+ *
  * DataTable rows contain columns, metadata and a subtable ID. Columns and metadata
  * are stored as an array of name => value mappings.
  *
  *
  * @api
  */
-class Row
+class Row implements \ArrayAccess, \IteratorAggregate
 {
     /**
      * List of columns that cannot be summed. An associative array for speed.
@@ -59,7 +60,7 @@ class Row
      * Constructor.
      *
      * @param array $row An array with the following structure:
-     *                   
+     *
      *                       array(
      *                           Row::COLUMNS => array('label' => 'Piwik',
      *                                                 'column1' => 42,
@@ -101,7 +102,7 @@ class Row
         if (!empty($this->c[self::DATATABLE_ASSOCIATED])
             && $this->c[self::DATATABLE_ASSOCIATED] < 0
         ) {
-            $this->c[self::DATATABLE_ASSOCIATED] = -1 * $this->c[self::DATATABLE_ASSOCIATED];
+            $this->switchFlagSubtableIsLoaded();
             $this->subtableIdWasNegativeBeforeSerialize = true;
         }
         return array('c');
@@ -114,11 +115,20 @@ class Row
     public function cleanPostSerialize()
     {
         if ($this->subtableIdWasNegativeBeforeSerialize) {
-            $this->c[self::DATATABLE_ASSOCIATED] = -1 * $this->c[self::DATATABLE_ASSOCIATED];
+            $this->switchFlagSubtableIsLoaded();
             $this->subtableIdWasNegativeBeforeSerialize = false;
         }
     }
 
+    /**
+     * note: also used by unit tests
+     * @ignore
+     */
+    public function switchFlagSubtableIsLoaded()
+    {
+        $this->c[self::DATATABLE_ASSOCIATED] = -1 * $this->c[self::DATATABLE_ASSOCIATED];
+    }
+
     /**
      * When destroyed, a row destroys its associated subtable if there is one.
      * @ignore
@@ -183,7 +193,7 @@ class Row
         if (isset($this->c[self::COLUMNS][$oldName])) {
             $this->c[self::COLUMNS][$newName] = $this->c[self::COLUMNS][$oldName];
         }
-        // outside the if() since we want to delete nulled columns
+        // outside the if () since we want to delete nulled columns
         unset($this->c[self::COLUMNS][$oldName]);
     }
 
@@ -264,7 +274,7 @@ class Row
      * Returns the array containing all the columns.
      *
      * @return array  Example:
-     *                
+     *
      *                    array(
      *                        'column1'   => VALUE,
      *                        'label'     => 'www.php.net'
@@ -307,7 +317,11 @@ class Row
     public function getSubtable()
     {
         if ($this->isSubtableLoaded()) {
-            return Manager::getInstance()->getTable($this->getIdSubDataTable());
+            try {
+                return Manager::getInstance()->getTable($this->getIdSubDataTable());
+            } catch(TableNotFoundException $e) {
+                // edge case
+            }
         }
         return false;
     }
@@ -315,9 +329,9 @@ class Row
     /**
      * Sums a DataTable to this row's subtable. If this row has no subtable a new
      * one is created.
-     * 
+     *
      * See {@link Piwik\DataTable::addDataTable()} to learn how DataTables are summed.
-     * 
+     *
      * @param DataTable $subTable Table to sum to this row's subtable.
      */
     public function sumSubtable(DataTable $subTable)
@@ -325,29 +339,16 @@ class Row
         if ($this->isSubtableLoaded()) {
             $thisSubTable = $this->getSubtable();
         } else {
+            $this->warnIfSubtableAlreadyExists();
+
             $thisSubTable = new DataTable();
-            $this->addSubtable($thisSubTable);
+            $this->setSubtable($thisSubTable);
         }
         $columnOps = $subTable->getMetadata(DataTable::COLUMN_AGGREGATION_OPS_METADATA_NAME);
         $thisSubTable->setMetadata(DataTable::COLUMN_AGGREGATION_OPS_METADATA_NAME, $columnOps);
         $thisSubTable->addDataTable($subTable);
     }
 
-    /**
-     * Attaches a subtable to this row.
-     *
-     * @param DataTable $subTable DataTable to associate to this row.
-     * @return DataTable Returns `$subTable`.
-     * @throws Exception if a subtable already exists for this row.
-     */
-    public function addSubtable(DataTable $subTable)
-    {
-        if (!is_null($this->c[self::DATATABLE_ASSOCIATED])) {
-            throw new Exception("Adding a subtable to the row, but it already has a subtable associated.");
-        }
-        return $this->setSubtable($subTable);
-    }
-
     /**
      * Attaches a subtable to this row, overwriting the existing subtable,
      * if any.
@@ -495,14 +496,15 @@ class Row
      * Sums the given `$rowToSum` columns values to the existing row column values.
      * Only the int or float values will be summed. Label columns will be ignored
      * even if they have a numeric value.
-     * 
+     *
      * Columns in `$rowToSum` that don't exist in `$this` are added to `$this`.
      *
      * @param \Piwik\DataTable\Row $rowToSum The row to sum to this row.
      * @param bool $enableCopyMetadata Whether metadata should be copied or not.
-     * @param array $aggregationOperations for columns that should not be summed, determine which
+     * @param array|bool $aggregationOperations for columns that should not be summed, determine which
      *                                     aggregation should be used (min, max). format:
      *                                     `array('column name' => 'function name')`
+     * @throws Exception
      */
     public function sumRow(Row $rowToSum, $enableCopyMetadata = true, $aggregationOperations = false)
     {
@@ -528,7 +530,7 @@ class Row
             if ($columnToSumName == Metrics::INDEX_MAX_ACTIONS) {
                 $operation = 'max';
             }
-            if(empty($operation)) {
+            if (empty($operation)) {
                 throw new Exception("Unknown aggregation operation for column $columnToSumName.");
             }
 
@@ -573,7 +575,7 @@ class Row
 
     /**
      * Sums the metadata in `$rowToSum` with the metadata in `$this` row.
-     * 
+     *
      * @param Row $rowToSum
      */
     public function sumRowMetadata($rowToSum)
@@ -597,7 +599,7 @@ class Row
     /**
      * Returns `true` if this row is the summary row, `false` if otherwise. This function
      * depends on the label of the row, and so, is not 100% accurate.
-     * 
+     *
      * @return bool
      */
     public function isSummaryRow()
@@ -642,10 +644,7 @@ class Row
             return $newValue;
         }
 
-        if (is_string($columnToSumValue)) {
-            throw new Exception("Trying to add two strings in DataTable\Row::sumRowArray: "
-                              . "'$thisColumnValue' + '$columnToSumValue'" . " for row " . $this->__toString());
-        }
+        $this->warnWhenSummingTwoStrings($thisColumnValue, $columnToSumValue);
 
         return 0;
     }
@@ -658,7 +657,7 @@ class Row
      * @return bool
      * @ignore
      */
-    static public function compareElements($elem1, $elem2)
+    public static function compareElements($elem1, $elem2)
     {
         if (is_array($elem1)) {
             if (is_array($elem2)) {
@@ -679,17 +678,17 @@ class Row
      * Helper function that tests if two rows are equal.
      *
      * Two rows are equal if:
-     * 
+     *
      * - they have exactly the same columns / metadata
      * - they have a subDataTable associated, then we check that both of them are the same.
-     * 
+     *
      * Column order is not important.
      *
      * @param \Piwik\DataTable\Row $row1 first to compare
      * @param \Piwik\DataTable\Row $row2 second to compare
      * @return bool
      */
-    static public function isEqual(Row $row1, Row $row2)
+    public static function isEqual(Row $row1, Row $row2)
     {
         //same columns
         $cols1 = $row1->getColumns();
@@ -726,4 +725,53 @@ class Row
         }
         return true;
     }
+
+    public function offsetExists($offset)
+    {
+        return $this->hasColumn($offset);
+    }
+
+    public function offsetGet($offset)
+    {
+        return $this->getColumn($offset);
+    }
+
+    public function offsetSet($offset, $value)
+    {
+        $this->setColumn($offset, $value);
+    }
+
+    public function offsetUnset($offset)
+    {
+        $this->deleteColumn($offset);
+    }
+
+    public function getIterator() {
+        return new \ArrayIterator($this->c[self::COLUMNS]);
+    }
+
+    private function warnIfSubtableAlreadyExists()
+    {
+        if (!is_null($this->c[self::DATATABLE_ASSOCIATED])) {
+            Log::warning(
+                "Row with label '%s' (columns = %s) has already a subtable id=%s but it was not loaded - overwriting the existing sub-table.",
+                $this->getColumn('label'),
+                implode(", ", $this->getColumns()),
+                $this->getIdSubDataTable()
+            );
+        }
+    }
+
+    protected function warnWhenSummingTwoStrings($thisColumnValue, $columnToSumValue)
+    {
+        if (is_string($columnToSumValue)) {
+            Log::warning(
+                "Trying to add two strings in DataTable\Row::sumRowArray: %s + %s for row %s",
+                $thisColumnValue,
+                $columnToSumValue,
+                $this->__toString()
+            );
+        }
+    }
+
 }
