@@ -14,9 +14,9 @@ use Piwik\DataTable\Row;
 
 /**
  * A special row whose column values are the aggregate of the row's subtable.
- * 
+ *
  * This class creates sets its own columns to the sum of each row in the row's subtable.
- * 
+ *
  * Non-numeric columns are bypassed during summation and do not appear in this
  * rows columns.
  *
@@ -27,7 +27,7 @@ class DataTableSummaryRow extends Row
 {
     /**
      * Constructor.
-     * 
+     *
      * @param DataTable|null $subTable The subtable of this row. This parameter is mostly for
      *                                 convenience. If set, its rows will be summed to this one,
      *                                 but it will not be set as this row's subtable (so
@@ -47,9 +47,8 @@ class DataTableSummaryRow extends Row
      */
     public function recalculate()
     {
-        $id = $this->getIdSubDataTable();
-        if ($id !== null) {
-            $subTable = Manager::getInstance()->getTable($id);
+        $subTable = $this->getSubtable();
+        if ($subTable) {
             $this->sumTable($subTable);
         }
     }
