@@ -8,8 +8,6 @@
  */
 namespace Piwik\Db;
 
-
-use Piwik\Loader;
 use Zend_Db_Table;
 
 /**
@@ -40,9 +38,7 @@ class Adapter
         }
 
         $className = self::getAdapterClassName($adapterName);
-        Loader::loadClass($className);
-
-        $adapter = new $className($dbInfos);
+        $adapter   = new $className($dbInfos);
 
         if ($connect) {
             $adapter->getConnection();
