@@ -29,7 +29,6 @@ class Schema extends Singleton
      */
     private $schema = null;
 
-
     /**
      * Get schema class name
      *
@@ -39,7 +38,7 @@ class Schema extends Singleton
     private static function getSchemaClassName($schemaName)
     {
         // Upgrade from pre 2.0.4
-        if(strtolower($schemaName) == 'myisam'
+        if (strtolower($schemaName) == 'myisam'
             || empty($schemaName)) {
             $schemaName = self::DEFAULT_SCHEMA;
         }
@@ -117,11 +116,11 @@ class Schema extends Singleton
      */
     private function loadSchema()
     {
-        $config = Config::getInstance();
-        $dbInfos = $config->database;
+        $config     = Config::getInstance();
+        $dbInfos    = $config->database;
         $schemaName = trim($dbInfos['schema']);
 
-        $className = self::getSchemaClassName($schemaName);
+        $className    = self::getSchemaClassName($schemaName);
         $this->schema = new $className();
     }
 
@@ -135,6 +134,7 @@ class Schema extends Singleton
         if ($this->schema === null) {
             $this->loadSchema();
         }
+
         return $this->schema;
     }
 
@@ -234,6 +234,18 @@ class Schema extends Singleton
         return $this->getSchema()->getTablesInstalled($forceReload);
     }
 
+    /**
+     * Get list of installed columns in a table
+     *
+     * @param  string $tableName The name of a table.
+     *
+     * @return array  Installed columns indexed by the column name.
+     */
+    public function getTableColumns($tableName)
+    {
+        return $this->getSchema()->getTableColumns($tableName);
+    }
+
     /**
      * Returns true if Piwik tables exist
      *
