@@ -11,6 +11,7 @@ namespace Piwik;
 use Exception;
 use Piwik\Db\Adapter;
 use Piwik\Db\Schema;
+use Piwik\DataAccess\ArchiveTableCreator;
 
 /**
  * Contains database related helper functions.
@@ -28,6 +29,18 @@ class DbHelper
         return Schema::getInstance()->getTablesInstalled($forceReload);
     }
 
+    /**
+     * Get list of installed columns in a table
+     *
+     * @param  string $tableName The name of a table.
+     *
+     * @return array  Installed columns indexed by the column name.
+     */
+    public static function getTableColumns($tableName)
+    {
+        return Schema::getInstance()->getTableColumns($tableName);
+    }
+
     /**
      * Creates a new table in the database.
      *
@@ -161,4 +174,17 @@ class DbHelper
         return Schema::getInstance()->getTableCreateSql($tableName);
     }
 
-}
+    /**
+     * Deletes archive tables. For use in tests.
+     */
+    public static function deleteArchiveTables()
+    {
+        foreach (ArchiveTableCreator::getTablesArchivesInstalled() as $table) {
+            Log::debug("Dropping table $table");
+
+            Db::query("DROP TABLE IF EXISTS `$table`");
+        }
+
+        ArchiveTableCreator::refreshTableList($forceReload = true);
+    }
+}
\ No newline at end of file
