@@ -45,6 +45,21 @@ class EventDispatcher extends Singleton
      */
     private $pendingEvents = array();
 
+    /**
+     * Plugin\Manager instance used to get list of loaded plugins.
+     *
+     * @var \Piwik\Plugin\Manager
+     */
+    private $pluginManager;
+
+    /**
+     * Constructor.
+     */
+    public function __construct($pluginManager = null)
+    {
+        $this->pluginManager = $pluginManager;
+    }
+
     /**
      * Triggers an event, executing all callbacks associated with it.
      *
@@ -64,7 +79,7 @@ class EventDispatcher extends Singleton
         }
 
         if (empty($plugins)) {
-            $plugins = \Piwik\Plugin\Manager::getInstance()->getPluginsLoadedAndActivated();
+            $plugins = $this->getPluginManager()->getPluginsLoadedAndActivated();
         }
 
         $callbacks = array();
@@ -72,9 +87,14 @@ class EventDispatcher extends Singleton
         // collect all callbacks to execute
         foreach ($plugins as $plugin) {
             if (is_string($plugin)) {
-                $plugin = \Piwik\Plugin\Manager::getInstance()->getLoadedPlugin($plugin);
+                $plugin = $this->getPluginManager()->getLoadedPlugin($plugin);
             }
 
+            if (empty($plugin)) {
+                return; // may happen in unit tests
+            }
+
+
             $hooks = $plugin->getListHooksRegistered();
 
             if (isset($hooks[$eventName])) {
@@ -92,6 +112,9 @@ class EventDispatcher extends Singleton
             }
         }
 
+        // sort callbacks by their importance
+        ksort($callbacks);
+
         // execute callbacks in order
         foreach ($callbacks as $callbackGroup) {
             foreach ($callbackGroup as $callback) {
@@ -171,10 +194,9 @@ class EventDispatcher extends Singleton
             if (!empty($hookInfo['before'])) {
                 $callbackGroup = self::EVENT_CALLBACK_GROUP_FIRST;
             } else if (!empty($hookInfo['after'])) {
-                $callbackGroup = self::EVENT_CALLBACK_GROUP_SECOND;
-            } else {
                 $callbackGroup = self::EVENT_CALLBACK_GROUP_THIRD;
-            }
+            } else {
+                $callbackGroup = self::EVENT_CALLBACK_GROUP_SECOND;            }
         } else {
             $pluginFunction = $hookInfo;
             $callbackGroup = self::EVENT_CALLBACK_GROUP_SECOND;
@@ -182,5 +204,17 @@ class EventDispatcher extends Singleton
 
         return array($pluginFunction, $callbackGroup);
     }
-}
 
+    /**
+     * Returns the Plugin\Manager instance used by the event dispatcher.
+     *
+     * @return Plugin\Manager
+     */
+    private function getPluginManager()
+    {
+        if ($this->pluginManager === null) {
+            $this->pluginManager = Plugin\Manager::getInstance();
+        }
+        return $this->pluginManager;
+    }
+}
\ No newline at end of file
