@@ -9,18 +9,17 @@
 namespace Piwik\Menu;
 
 use Piwik\Common;
-use Piwik\Log;
 use Piwik\Plugins\SitesManager\API;
 use Piwik\Singleton;
 use Piwik\Plugin\Manager as PluginManager;
 
 /**
  * Base class for classes that manage one of Piwik's menus.
- * 
+ *
  * There are three menus in Piwik, the main menu, the top menu and the admin menu.
  * Each menu has a class that manages the menu's content. Each class invokes
  * a different event to allow plugins to add new menu items.
- * 
+ *
  * @static \Piwik\Menu\MenuAbstract getInstance()
  */
 abstract class MenuAbstract extends Singleton
@@ -55,7 +54,7 @@ abstract class MenuAbstract extends Singleton
      *
      * @return \Piwik\Plugin\Menu[]
      */
-    protected function getAvailableMenus()
+    protected function getAllMenus()
     {
         if (!empty(self::$menus)) {
             return self::$menus;
@@ -77,7 +76,8 @@ abstract class MenuAbstract extends Singleton
      *                                         current user. If false, the entry will not be added.
      * @param int $order The order hint.
      * @param bool|string $tooltip An optional tooltip to display or false to display the tooltip.
-     * @api
+     *
+     * @deprecated since 2.7.0 Use {@link addItem() instead}. Method will be removed in Piwik 3.0
      */
     public function add($menuName, $subMenuName, $url, $displayedForCurrentUser = true, $order = 50, $tooltip = false)
     {
@@ -85,8 +85,25 @@ abstract class MenuAbstract extends Singleton
             return;
         }
 
+        $this->addItem($menuName, $subMenuName, $url, $order, $tooltip);
+    }
+
+    /**
+     * Adds a new entry to the menu.
+     *
+     * @param string $menuName The menu's category name. Can be a translation token.
+     * @param string $subMenuName The menu item's name. Can be a translation token.
+     * @param string|array $url The URL the admin menu entry should link to, or an array of query parameters
+     *                          that can be used to build the URL.
+     * @param int $order The order hint.
+     * @param bool|string $tooltip An optional tooltip to display or false to display the tooltip.
+     * @since 2.7.0
+     * @api
+     */
+    public function addItem($menuName, $subMenuName, $url, $order = 50, $tooltip = false)
+    {
         // make sure the idSite value used is numeric (hack-y fix for #3426)
-        if (!is_numeric(Common::getRequestVar('idSite', false))) {
+        if (isset($url['idSite']) && !is_numeric($url['idSite'])) {
             $idSites = API::getInstance()->getSitesIdWithAtLeastViewAccess();
             $url['idSite'] = reset($idSites);
         }
@@ -126,13 +143,17 @@ abstract class MenuAbstract extends Singleton
      */
     private function buildMenuItem($menuName, $subMenuName, $url, $order = 50, $tooltip = false)
     {
-        if (!isset($this->menu[$menuName]) || empty($subMenuName)) {
-            $this->menu[$menuName]['_url'] = $url;
-            if (empty($subMenuName)) {
-                $this->menu[$menuName]['_order'] = $order;
-            }
-            $this->menu[$menuName]['_name'] = $menuName;
-            $this->menu[$menuName]['_hasSubmenu'] = false;
+        if (!isset($this->menu[$menuName])) {
+            $this->menu[$menuName] = array(
+                '_hasSubmenu' => false,
+                '_order' => $order
+            );
+        }
+
+        if (empty($subMenuName)) {
+            $this->menu[$menuName]['_url']   = $url;
+            $this->menu[$menuName]['_order'] = $order;
+            $this->menu[$menuName]['_name']  = $menuName;
             $this->menu[$menuName]['_tooltip'] = $tooltip;
         }
         if (!empty($subMenuName)) {
@@ -141,7 +162,10 @@ abstract class MenuAbstract extends Singleton
             $this->menu[$menuName][$subMenuName]['_name'] = $subMenuName;
             $this->menu[$menuName][$subMenuName]['_tooltip'] = $tooltip;
             $this->menu[$menuName]['_hasSubmenu'] = true;
-            $this->menu[$menuName]['_tooltip'] = $tooltip;
+
+            if (!array_key_exists('_tooltip', $this->menu[$menuName])) {
+                $this->menu[$menuName]['_tooltip'] = $tooltip;
+            }
         }
     }
 
@@ -190,8 +214,8 @@ abstract class MenuAbstract extends Singleton
     {
         foreach ($this->edits as $edit) {
             $mainMenuToEdit = $edit[0];
-            $subMenuToEdit = $edit[1];
-            $newUrl = $edit[2];
+            $subMenuToEdit  = $edit[1];
+            $newUrl         = $edit[2];
 
             if ($subMenuToEdit === null) {
                 $menuDataToEdit = @$this->menu[$mainMenuToEdit];
@@ -211,14 +235,14 @@ abstract class MenuAbstract extends Singleton
     {
         foreach($this->menuEntriesToRemove as $menuToDelete) {
 
-            if(empty($menuToDelete[1])) {
+            if (empty($menuToDelete[1])) {
                 // Delete Main Menu
-                if(isset($this->menu[$menuToDelete[0]])) {
+                if (isset($this->menu[$menuToDelete[0]])) {
                     unset($this->menu[$menuToDelete[0]]);
                 }
             } else {
                 // Delete Sub Menu
-                if(isset($this->menu[$menuToDelete[0]][$menuToDelete[1]])) {
+                if (isset($this->menu[$menuToDelete[0]][$menuToDelete[1]])) {
                     unset($this->menu[$menuToDelete[0]][$menuToDelete[1]]);
                 }
             }
@@ -231,9 +255,10 @@ abstract class MenuAbstract extends Singleton
     {
         foreach ($this->renames as $rename) {
             $mainMenuOriginal = $rename[0];
-            $subMenuOriginal = $rename[1];
-            $mainMenuRenamed = $rename[2];
-            $subMenuRenamed = $rename[3];
+            $subMenuOriginal  = $rename[1];
+            $mainMenuRenamed  = $rename[2];
+            $subMenuRenamed   = $rename[3];
+
             // Are we changing a submenu?
             if (!empty($subMenuOriginal)) {
                 if (isset($this->menu[$mainMenuOriginal][$subMenuOriginal])) {
@@ -284,15 +309,36 @@ abstract class MenuAbstract extends Singleton
      */
     protected function menuCompare($itemOne, $itemTwo)
     {
-        if (!is_array($itemOne) || !is_array($itemTwo)
-            || !isset($itemOne['_order']) || !isset($itemTwo['_order'])
-        ) {
+        if (!is_array($itemOne) && !is_array($itemTwo)) {
             return 0;
         }
 
-        if ($itemOne['_order'] == $itemTwo['_order']) {
-            return strcmp($itemOne['_name'], $itemTwo['_name']);
+        if (!is_array($itemOne) && is_array($itemTwo)) {
+            return -1;
         }
+
+        if (is_array($itemOne) && !is_array($itemTwo)) {
+            return 1;
+        }
+
+        if (!isset($itemOne['_order']) && !isset($itemTwo['_order'])) {
+            return 0;
+        }
+
+        if (!isset($itemOne['_order']) && isset($itemTwo['_order'])) {
+            return -1;
+        }
+
+        if (isset($itemOne['_order']) && !isset($itemTwo['_order'])) {
+            return 1;
+        }
+
+        if ($itemOne['_order'] == $itemTwo['_order']) {
+            return strcmp(
+                @$itemOne['_name'],
+                @$itemTwo['_name']);
+        }
+
         return ($itemOne['_order'] < $itemTwo['_order']) ? -1 : 1;
     }
 }
