@@ -8,17 +8,17 @@
  */
 namespace Piwik;
 
-use Piwik\Period\Factory;
+use Piwik\Period\Factory as PeriodFactory;
 use Piwik\Period\Range;
 
 /**
  * Date range representation.
- * 
+ *
  * Piwik allows users to view aggregated statistics for single days and for date
  * ranges consisting of several days. When requesting data, a **date** string and
  * a **period** string must be used to specify the date range that the data regards.
  * This is the class Piwik uses to represent and manipulate those date ranges.
- * 
+ *
  * There are five types of periods in Piwik: day, week, month, year and range,
  * where **range** is any date range. The reason the other periods exist instead
  * of just **range** is that Piwik will pre-archive reports for days, weeks, months
@@ -47,7 +47,7 @@ abstract class Period
 
     /**
      * Constructor.
-     * 
+     *
      * @param Date $date
      * @ignore
      */
@@ -64,21 +64,21 @@ abstract class Period
      */
     public static function factory($period, $date)
     {
-        return Factory::build($period, $date);
+        return PeriodFactory::build($period, $date);
     }
 
     /**
      * Returns true if `$dateString` and `$period` represent multiple periods.
-     * 
+     *
      * Will return true for date/period combinations where date references multiple
      * dates and period is not `'range'`. For example, will return true for:
-     * 
+     *
      * - **date** = `2012-01-01,2012-02-01` and **period** = `'day'`
      * - **date** = `2012-01-01,2012-02-01` and **period** = `'week'`
      * - **date** = `last7` and **period** = `'month'`
-     * 
+     *
      * etc.
-     * 
+     *
      * @static
      * @param  $dateString string The **date** query parameter value.
      * @param  $period string The **period** query parameter value.
@@ -92,7 +92,6 @@ abstract class Period
             && $period != 'range';
     }
 
-
     /**
      * Returns the first day of the period.
      *
@@ -101,16 +100,20 @@ abstract class Period
     public function getDateStart()
     {
         $this->generate();
+
         if (count($this->subperiods) == 0) {
             return $this->getDate();
         }
+
         $periods = $this->getSubperiods();
+
         /** @var $currentPeriod Period */
         $currentPeriod = $periods[0];
         while ($currentPeriod->getNumberOfSubperiods() > 0) {
-            $periods = $currentPeriod->getSubperiods();
+            $periods       = $currentPeriod->getSubperiods();
             $currentPeriod = $periods[0];
         }
+
         return $currentPeriod->getDate();
     }
 
@@ -122,22 +125,26 @@ abstract class Period
     public function getDateEnd()
     {
         $this->generate();
+
         if (count($this->subperiods) == 0) {
             return $this->getDate();
         }
+
         $periods = $this->getSubperiods();
+
         /** @var $currentPeriod Period */
         $currentPeriod = $periods[count($periods) - 1];
         while ($currentPeriod->getNumberOfSubperiods() > 0) {
-            $periods = $currentPeriod->getSubperiods();
+            $periods       = $currentPeriod->getSubperiods();
             $currentPeriod = $periods[count($periods) - 1];
         }
+
         return $currentPeriod->getDate();
     }
 
     /**
      * Returns the period ID.
-     * 
+     *
      * @return int A unique integer for this type of period.
      */
     public function getId()
@@ -147,7 +154,7 @@ abstract class Period
 
     /**
      * Returns the label for the current period.
-     * 
+     *
      * @return string `"day"`, `"week"`, `"month"`, `"year"`, `"range"`
      */
     public function getLabel()
@@ -170,7 +177,7 @@ abstract class Period
 
     /**
      * Returns the number of available subperiods.
-     * 
+     *
      * @return int
      */
     public function getNumberOfSubperiods()
@@ -182,7 +189,7 @@ abstract class Period
     /**
      * Returns the set of Period instances that together make up this period. For a year,
      * this would be 12 months. For a month this would be 28-31 days. Etc.
-     * 
+     *
      * @return Period[]
      */
     public function getSubperiods()
@@ -213,16 +220,18 @@ abstract class Period
     public function toString($format = "Y-m-d")
     {
         $this->generate();
+
         $dateString = array();
         foreach ($this->subperiods as $period) {
             $dateString[] = $period->toString($format);
         }
+
         return $dateString;
     }
 
     /**
      * See {@link toString()}.
-     * 
+     *
      * @return string
      */
     public function __toString()
@@ -232,7 +241,7 @@ abstract class Period
 
     /**
      * Returns a pretty string describing this period.
-     * 
+     *
      * @return string
      */
     abstract public function getPrettyString();
@@ -240,7 +249,7 @@ abstract class Period
     /**
      * Returns a short string description of this period that is localized with the currently used
      * language.
-     * 
+     *
      * @return string
      */
     abstract public function getLocalizedShortString();
@@ -248,18 +257,21 @@ abstract class Period
     /**
      * Returns a long string description of this period that is localized with the currently used
      * language.
-     * 
+     *
      * @return string
      */
     abstract public function getLocalizedLongString();
 
     /**
-     * Returns a succinct string describing this period.
-     * 
+     * Returns the date range string comprising two dates
+     *
      * @return string eg, `'2012-01-01,2012-01-31'`.
      */
     public function getRangeString()
     {
-        return $this->getDateStart()->toString("Y-m-d") . "," . $this->getDateEnd()->toString("Y-m-d");
+        $dateStart = $this->getDateStart();
+        $dateEnd   = $this->getDateEnd();
+
+        return $dateStart->toString("Y-m-d") . "," . $dateEnd->toString("Y-m-d");
     }
 }
