@@ -8,7 +8,6 @@
  */
 namespace Piwik\Period;
 
-
 use Piwik\Period;
 use Piwik\Piwik;
 
@@ -27,7 +26,7 @@ class Week extends Period
     {
         //"30 Dec - 6 Jan 09"
         $dateStart = $this->getDateStart();
-        $dateEnd = $this->getDateEnd();
+        $dateEnd   = $this->getDateEnd();
 
         $string = Piwik::translate('CoreHome_ShortWeekFormat');
         $string = self::getTranslatedRange($string, $dateStart, $dateEnd);
@@ -43,6 +42,7 @@ class Week extends Period
     {
         $format = Piwik::translate('CoreHome_LongWeekFormat');
         $string = self::getTranslatedRange($format, $this->getDateStart(), $this->getDateEnd());
+
         return Piwik::translate('CoreHome_PeriodWeek') . " " . $string;
     }
 
@@ -53,12 +53,13 @@ class Week extends Period
      *
      * @return mixed
      */
-    static protected function getTranslatedRange($format, $dateStart, $dateEnd)
+    protected static function getTranslatedRange($format, $dateStart, $dateEnd)
     {
         $string = str_replace('From%', '%', $format);
         $string = $dateStart->getLocalized($string);
         $string = str_replace('To%', '%', $string);
         $string = $dateEnd->getLocalized($string);
+
         return $string;
     }
 
@@ -69,10 +70,11 @@ class Week extends Period
      */
     public function getPrettyString()
     {
-        $out = Piwik::translate('General_DateRangeFromTo',
-            array($this->getDateStart()->toString(),
-                  $this->getDateEnd()->toString())
-        );
+        $dateStart = $this->getDateStart();
+        $dateEnd   = $this->getDateEnd();
+
+        $out = Piwik::translate('General_DateRangeFromTo', array($dateStart->toString(), $dateEnd->toString()));
+
         return $out;
     }
 
@@ -84,6 +86,7 @@ class Week extends Period
         if ($this->subperiodsProcessed) {
             return;
         }
+
         parent::generate();
         $date = $this->date;
 
