@@ -17,6 +17,8 @@ use Piwik\Config as PiwikConfig;
 use Piwik\Config;
 use Piwik\DataTable\Filter\CalculateEvolutionFilter;
 use Piwik\Date;
+use Piwik\Exception\NoPrivilegesException;
+use Piwik\Exception\NoWebsiteFoundException;
 use Piwik\FrontController;
 use Piwik\Menu\MenuTop;
 use Piwik\Menu\MenuUser;
@@ -29,7 +31,6 @@ use Piwik\Piwik;
 use Piwik\Plugins\CoreAdminHome\CustomLogo;
 use Piwik\Plugins\CoreVisualizations\Visualizations\JqplotGraph\Evolution;
 use Piwik\Plugins\LanguagesManager\LanguagesManager;
-use Piwik\Plugins\UsersManager\UserPreferences;
 use Piwik\Registry;
 use Piwik\SettingsPiwik;
 use Piwik\Site;
@@ -40,18 +41,18 @@ use Piwik\ViewDataTable\Factory as ViewDataTableFactory;
 
 /**
  * Base class of all plugin Controllers.
- * 
+ *
  * Plugins that wish to add display HTML should create a Controller that either
  * extends from this class or from {@link ControllerAdmin}. Every public method in
  * the controller will be exposed as a controller method and can be invoked via
  * an HTTP request.
- * 
+ *
  * Learn more about Piwik's MVC system [here](/guides/mvc-in-piwik).
- * 
+ *
  * ### Examples
- * 
+ *
  * **Defining a controller**
- * 
+ *
  *     class Controller extends \Piwik\Plugin\Controller
  *     {
  *         public function index()
@@ -61,17 +62,17 @@ use Piwik\ViewDataTable\Factory as ViewDataTableFactory;
  *             return $view->render();
  *         }
  *     }
- * 
+ *
  * **Linking to a controller action**
  *
  *     <a href="?module=MyPlugin&action=index&idSite=1&period=day&date=2013-10-10">Link</a>
- * 
+ *
  */
 abstract class Controller
 {
     /**
      * The plugin name, eg. `'Referrers'`.
-     * 
+     *
      * @var string
      * @api
      */
@@ -95,7 +96,7 @@ abstract class Controller
 
     /**
      * The value of the **idSite** query parameter.
-     * 
+     *
      * @var int
      * @api
      */
@@ -103,7 +104,7 @@ abstract class Controller
 
     /**
      * The Site object created with {@link $idSite}.
-     * 
+     *
      * @var Site
      * @api
      */
@@ -111,7 +112,7 @@ abstract class Controller
 
     /**
      * Constructor.
-     * 
+     *
      * @api
      */
     public function __construct()
@@ -248,10 +249,60 @@ abstract class Controller
         return $view->render();
     }
 
+    /**
+     * Assigns the given variables to the template and renders it.
+     *
+     * Example:
+     *
+     *     public function myControllerAction () {
+     *        return $this->renderTemplate('index', array(
+     *            'answerToLife' => '42'
+     *        ));
+     *     }
+     *
+     * This will render the 'index.twig' file within the plugin templates folder and assign the view variable
+     * `answerToLife` to `42`.
+     *
+     * @param string $template   The name of the template file. If only a name is given it will automatically use
+     *                           the template within the plugin folder. For instance 'myTemplate' will result in
+     *                           '@$pluginName/myTemplate.twig'. Alternatively you can include the full path:
+     *                           '@anyOtherFolder/otherTemplate'. The trailing '.twig' is not needed.
+     * @param array $variables   For instance array('myViewVar' => 'myValue'). In template you can use {{ myViewVar }}
+     * @return string
+     * @since 2.5.0
+     * @api
+     */
+    protected function renderTemplate($template, array $variables = array())
+    {
+        if (false === strpos($template, '@') || false === strpos($template, '/')) {
+            $template = '@' . $this->pluginName . '/' . $template;
+        }
+
+        $view = new View($template);
+
+        // alternatively we could check whether the templates extends either admin.twig or dashboard.twig and based on
+        // that call the correct method. This will be needed once we unify Controller and ControllerAdmin see
+        // https://github.com/piwik/piwik/issues/6151
+        if ($this instanceof ControllerAdmin) {
+            $this->setBasicVariablesView($view);
+        } elseif (empty($this->site) || empty($this->idSite)) {
+            $this->setBasicVariablesView($view);
+        } else {
+            $this->setGeneralVariablesView($view);
+        }
+
+        foreach ($variables as $key => $value) {
+            $view->$key = $value;
+        }
+
+        return $view->render();
+    }
+
     /**
      * Convenience method that creates and renders a ViewDataTable for a API method.
      *
-     * @param string $apiAction The name of the API action (eg, `'getResolution'`).
+     * @param string|\Piwik\Plugin\Report $apiAction The name of the API action (eg, `'getResolution'`) or
+     *                                      an instance of an report.
      * @param bool $controllerAction The name of the Controller action name  that is rendering the report. Defaults
      *                               to the `$apiAction`.
      * @param bool $fetch If `true`, the rendered string is returned, if `false` it is `echo`'d.
@@ -262,6 +313,21 @@ abstract class Controller
      */
     protected function renderReport($apiAction, $controllerAction = false)
     {
+        if (empty($controllerAction) && is_string($apiAction)) {
+            $report = Report::factory($this->pluginName, $apiAction);
+
+            if (!empty($report)) {
+                $apiAction = $report;
+            }
+        }
+
+        if ($apiAction instanceof Report) {
+            $this->checkSitePermission();
+            $apiAction->checkIsEnabled();
+
+            return $apiAction->render();
+        }
+
         $pluginName = $this->pluginName;
 
         /** @var Proxy $apiProxy */
@@ -428,11 +494,11 @@ abstract class Controller
 
     /**
      * Returns a URL to a sparkline image for a report served by the current plugin.
-     * 
+     *
      * The result of this URL should be used with the [sparkline()](/api-reference/Piwik/View#twig) twig function.
-     * 
+     *
      * The current site ID and period will be used.
-     * 
+     *
      * @param string $action Method name of the controller that serves the report.
      * @param array $customParameters The array of query parameter name/value pairs that
      *                                should be set in result URL.
@@ -488,9 +554,9 @@ abstract class Controller
 
     /**
      * Assigns variables to {@link Piwik\View} instances that display an entire page.
-     * 
+     *
      * The following variables assigned:
-     * 
+     *
      * **date** - The value of the **date** query parameter.
      * **idSite** - The value of the **idSite** query parameter.
      * **rawDate** - The value of the **date** query parameter.
@@ -503,11 +569,11 @@ abstract class Controller
      * **config_action_url_category_delimiter** - The value of the `[General] action_url_category_delimiter`
      *                                            INI config option.
      * **topMenu** - The result of `MenuTop::getInstance()->getMenu()`.
-     * 
+     *
      * As well as the variables set by {@link setPeriodVariablesView()}.
-     * 
+     *
      * Will exit on error.
-     * 
+     *
      * @param View $view
      * @return void
      * @api
@@ -516,72 +582,64 @@ abstract class Controller
     {
         $view->date = $this->strDate;
 
-        try {
-            $view->idSite = $this->idSite;
-            if (empty($this->site) || empty($this->idSite)) {
-                throw new Exception("The requested website idSite is not found in the request, or is invalid.
-				Please check that you are logged in Piwik and have permission to access the specified website.");
-            }
-            $this->setPeriodVariablesView($view);
+        $view->idSite = $this->idSite;
+        $this->checkSitePermission();
+        $this->setPeriodVariablesView($view);
 
-            $rawDate = Common::getRequestVar('date');
-            $periodStr = Common::getRequestVar('period');
-            if ($periodStr != 'range') {
-                $date = Date::factory($this->strDate);
-                $period = Period\Factory::build($periodStr, $date);
-            } else {
-                $period = new Range($periodStr, $rawDate, $this->site->getTimezone());
-            }
-            $view->rawDate = $rawDate;
-            $view->prettyDate = self::getCalendarPrettyDate($period);
+        $rawDate = Common::getRequestVar('date');
+        $periodStr = Common::getRequestVar('period');
+        if ($periodStr != 'range') {
+            $date = Date::factory($this->strDate);
+            $period = Period\Factory::build($periodStr, $date);
+        } else {
+            $period = new Range($periodStr, $rawDate, $this->site->getTimezone());
+        }
+        $view->rawDate = $rawDate;
+        $view->prettyDate = self::getCalendarPrettyDate($period);
 
-            $view->siteName = $this->site->getName();
-            $view->siteMainUrl = $this->site->getMainUrl();
+        $view->siteName = $this->site->getName();
+        $view->siteMainUrl = $this->site->getMainUrl();
 
-            $datetimeMinDate = $this->site->getCreationDate()->getDatetime();
-            $minDate = Date::factory($datetimeMinDate, $this->site->getTimezone());
-            $this->setMinDateView($minDate, $view);
+        $datetimeMinDate = $this->site->getCreationDate()->getDatetime();
+        $minDate = Date::factory($datetimeMinDate, $this->site->getTimezone());
+        $this->setMinDateView($minDate, $view);
 
-            $maxDate = Date::factory('now', $this->site->getTimezone());
-            $this->setMaxDateView($maxDate, $view);
+        $maxDate = Date::factory('now', $this->site->getTimezone());
+        $this->setMaxDateView($maxDate, $view);
 
-            // Setting current period start & end dates, for pre-setting the calendar when "Date Range" is selected
-            $dateStart = $period->getDateStart();
-            if ($dateStart->isEarlier($minDate)) {
-                $dateStart = $minDate;
-            }
-            $dateEnd = $period->getDateEnd();
-            if ($dateEnd->isLater($maxDate)) {
-                $dateEnd = $maxDate;
-            }
+        // Setting current period start & end dates, for pre-setting the calendar when "Date Range" is selected
+        $dateStart = $period->getDateStart();
+        if ($dateStart->isEarlier($minDate)) {
+            $dateStart = $minDate;
+        }
+        $dateEnd = $period->getDateEnd();
+        if ($dateEnd->isLater($maxDate)) {
+            $dateEnd = $maxDate;
+        }
 
-            $view->startDate = $dateStart;
-            $view->endDate = $dateEnd;
+        $view->startDate = $dateStart;
+        $view->endDate = $dateEnd;
 
-            $language = LanguagesManager::getLanguageForSession();
-            $view->language = !empty($language) ? $language : LanguagesManager::getLanguageCodeForCurrentUser();
+        $language = LanguagesManager::getLanguageForSession();
+        $view->language = !empty($language) ? $language : LanguagesManager::getLanguageCodeForCurrentUser();
 
-            $this->setBasicVariablesView($view);
+        $this->setBasicVariablesView($view);
 
-            $view->topMenu  = MenuTop::getInstance()->getMenu();
-            $view->userMenu = MenuUser::getInstance()->getMenu();
+        $view->topMenu  = MenuTop::getInstance()->getMenu();
+        $view->userMenu = MenuUser::getInstance()->getMenu();
 
-            $notifications = $view->notifications;
-            if (empty($notifications)) {
-                $view->notifications = NotificationManager::getAllNotificationsToDisplay();
-                NotificationManager::cancelAllNonPersistent();
-            }
-
-        } catch (Exception $e) {
-            Piwik_ExitWithMessage($e->getMessage(), $e->getTraceAsString());
+        $notifications = $view->notifications;
+        if (empty($notifications)) {
+            $view->notifications = NotificationManager::getAllNotificationsToDisplay();
+            NotificationManager::cancelAllNonPersistent();
         }
     }
 
     /**
      * Assigns a set of generally useful variables to a {@link Piwik\View} instance.
-     * 
+     *
      * The following variables assigned:
-     * 
+     *
      * **enableMeasurePiwikForSiteId** - The value of the `[Debug] enable_measure_piwik_usage_in_idsite`
      *                                     INI config option.
      * **isSuperUser** - True if the current user is the Super User, false if otherwise.
@@ -594,7 +652,7 @@ abstract class Controller
      * **hasSVGLogo** - True if there is a SVG logo, false if otherwise.
      * **enableFrames** - The value of the `[General] enable_framed_pages` INI config option. If
      *                    true, {@link Piwik\View::setXFrameOptions()} is called on the view.
-     * 
+     *
      * Also calls {@link setHostValidationVariablesView()}.
      *
      * @param View $view
@@ -610,6 +668,10 @@ abstract class Controller
         $view->isUserIsAnonymous  = Piwik::isUserIsAnonymous();
         $view->hasSuperUserAccess = Piwik::hasUserSuperUserAccess();
 
+        if (!Piwik::isUserIsAnonymous()) {
+            $view->emailSuperUser = implode(',', Piwik::getAllSuperUserAccessEmailAddresses());
+        }
+
         $this->addCustomLogoInfo($view);
 
         $view->logoHeader = \Piwik\Plugins\API\API::getInstance()->getHeaderLogoUrl();
@@ -721,7 +783,7 @@ abstract class Controller
 
     /**
      * Sets general period variables on a view, including:
-     * 
+     *
      * - **displayUniqueVisitors** - Whether unique visitors should be displayed for the current
      *                               period.
      * - **period** - The value of the **period** query parameter.
@@ -754,7 +816,7 @@ abstract class Controller
 
     /**
      * Helper method used to redirect the current HTTP request to another module/action.
-     * 
+     *
      * This function will exit immediately after executing.
      *
      * @param string $moduleToRedirect The plugin to redirect to, eg. `"MultiSites"`.
@@ -768,59 +830,46 @@ abstract class Controller
     public function redirectToIndex($moduleToRedirect, $actionToRedirect, $websiteId = null, $defaultPeriod = null,
                                     $defaultDate = null, $parameters = array())
     {
-
-        $userPreferences = new UserPreferences();
-
-        if (empty($websiteId)) {
-            $websiteId = $userPreferences->getDefaultWebsiteId();
-        }
-        if (empty($defaultDate)) {
-            $defaultDate = $userPreferences->getDefaultDate();
-        }
-        if (empty($defaultPeriod)) {
-            $defaultPeriod = $userPreferences->getDefaultPeriod();
-        }
-        $parametersString = '';
-        if (!empty($parameters)) {
-            $parametersString = '&' . Url::getQueryStringFromParameters($parameters);
-        }
-
-        if ($websiteId) {
-            $url = "index.php?module=" . $moduleToRedirect
-                . "&action=" . $actionToRedirect
-                . "&idSite=" . $websiteId
-                . "&period=" . $defaultPeriod
-                . "&date=" . $defaultDate
-                . $parametersString;
-            Url::redirectToUrl($url);
-            exit;
+        try {
+            $this->doRedirectToUrl($moduleToRedirect, $actionToRedirect, $websiteId, $defaultPeriod, $defaultDate, $parameters);
+        } catch(Exception $e) {
+            // no website ID to default to, so could not redirect
         }
 
         if (Piwik::hasUserSuperUserAccess()) {
-            Piwik_ExitWithMessage("Error: no website was found in this Piwik installation.
-			<br />Check the table '" . Common::prefixTable('site') . "' in your database, it should contain your Piwik websites.", false, true);
+            $siteTableName = Common::prefixTable('site');
+            $message = "Error: no website was found in this Piwik installation.
+			<br />Check the table '$siteTableName' in your database, it should contain your Piwik websites.";
+
+            $ex = new NoWebsiteFoundException($message);
+            $ex->setIsHtmlMessage();
+
+            throw $ex;
         }
 
-        $currentLogin = Piwik::getCurrentUserLogin();
-        if (!empty($currentLogin)
-            && $currentLogin != 'anonymous'
-        ) {
+        if (!Piwik::isUserIsAnonymous()) {
+            $currentLogin = Piwik::getCurrentUserLogin();
             $emails = implode(',', Piwik::getAllSuperUserAccessEmailAddresses());
-            $errorMessage = sprintf(Piwik::translate('CoreHome_NoPrivilegesAskPiwikAdmin'), $currentLogin, "<br/><a href='mailto:" . $emails . "?subject=Access to Piwik for user $currentLogin'>", "</a>");
+            $errorMessage  = sprintf(Piwik::translate('CoreHome_NoPrivilegesAskPiwikAdmin'), $currentLogin, "<br/><a href='mailto:" . $emails . "?subject=Access to Piwik for user $currentLogin'>", "</a>");
             $errorMessage .= "<br /><br />&nbsp;&nbsp;&nbsp;<b><a href='index.php?module=" . Registry::get('auth')->getName() . "&amp;action=logout'>&rsaquo; " . Piwik::translate('General_Logout') . "</a></b><br />";
-            Piwik_ExitWithMessage($errorMessage, false, true);
+
+            $ex = new NoPrivilegesException($errorMessage);
+            $ex->setIsHtmlMessage();
+
+            throw $ex;
         }
 
         echo FrontController::getInstance()->dispatch(Piwik::getLoginPluginName(), false);
         exit;
     }
 
+
     /**
      * Checks that the token_auth in the URL matches the currently logged-in user's token_auth.
-     * 
+     *
      * This is a protection against CSRF and should be used in all controller
      * methods that modify Piwik or any user settings.
-     * 
+     *
      * **The token_auth should never appear in the browser's address bar.**
      *
      * @throws \Piwik\NoAccessException If the token doesn't match.
@@ -831,7 +880,7 @@ abstract class Controller
         $tokenRequest = Common::getRequestVar('token_auth', false);
         $tokenUser = Piwik::getCurrentUserTokenAuth();
 
-        if(empty($tokenRequest) && empty($tokenUser)) {
+        if (empty($tokenRequest) && empty($tokenUser)) {
             return; // UI tests
         }
 
@@ -926,4 +975,36 @@ abstract class Controller
 
         return $result;
     }
+
+    protected function checkSitePermission()
+    {
+        if (empty($this->site) || empty($this->idSite)) {
+            throw new Exception("The requested website idSite is not found in the request, or is invalid.
+				Please check that you are logged in Piwik and have permission to access the specified website.");
+        }
+    }
+
+    /**
+     * @param $moduleToRedirect
+     * @param $actionToRedirect
+     * @param $websiteId
+     * @param $defaultPeriod
+     * @param $defaultDate
+     * @param $parameters
+     * @throws Exception
+     */
+    private function doRedirectToUrl($moduleToRedirect, $actionToRedirect, $websiteId, $defaultPeriod, $defaultDate, $parameters)
+    {
+        $menu = new Menu();
+
+        $parameters = array_merge(
+            $menu->urlForDefaultUserParams($websiteId, $defaultPeriod, $defaultDate),
+            $parameters
+        );
+        $queryParams = !empty($parameters) ? '&' . Url::getQueryStringFromParameters($parameters) : '';
+        $url = "index.php?module=%s&action=%s";
+        $url = sprintf($url, $moduleToRedirect, $actionToRedirect);
+        $url = $url . $queryParams;
+        Url::redirectToUrl($url);
+    }
 }
