@@ -9,25 +9,35 @@
 
 namespace Piwik\Plugin;
 
+use Piwik\Cache;
+use Piwik\Columns\Dimension;
 use Piwik\Common;
 use Piwik\Config as PiwikConfig;
 use Piwik\Config;
+use Piwik\Db;
+use Piwik\Development;
 use Piwik\EventDispatcher;
 use Piwik\Filesystem;
+use Piwik\Log;
 use Piwik\Option;
+use Piwik\Piwik;
 use Piwik\Plugin;
 use Piwik\Singleton;
 use Piwik\Theme;
 use Piwik\Tracker;
 use Piwik\Translate;
 use Piwik\Updater;
+use Piwik\SettingsServer;
+use Piwik\Plugin\Dimension\ActionDimension;
+use Piwik\Plugin\Dimension\ConversionDimension;
+use Piwik\Plugin\Dimension\VisitDimension;
 
 require_once PIWIK_INCLUDE_PATH . '/core/EventDispatcher.php';
 
 /**
  * The singleton that manages plugin loading/unloading and installation/uninstallation.
  *
- * @method static \Piwik\Plugin\Manager getInstance()
+ * @method static Manager getInstance()
  */
 class Manager extends Singleton
 {
@@ -73,6 +83,8 @@ class Manager extends Singleton
         'ExampleUI',
         'ExampleVisualization',
         'ExamplePluginTemplate',
+        'ExampleTracker',
+        'ExampleReport'
     );
 
     // Themes bundled with core package, disabled by default
@@ -80,6 +92,8 @@ class Manager extends Singleton
         'ExampleTheme'
     );
 
+    private $trackerPluginsNotToLoad = array();
+
     /**
      * Loads plugin that are enabled
      */
@@ -95,7 +109,7 @@ class Manager extends Singleton
     public function loadCorePluginsDuringTracker()
     {
         $pluginsToLoad = Config::getInstance()->Plugins['Plugins'];
-        $pluginsToLoad = array_diff($pluginsToLoad, Tracker::getPluginsNotToLoad());
+        $pluginsToLoad = array_diff($pluginsToLoad, $this->getTrackerPluginsNotToLoad());
         $this->loadPlugins($pluginsToLoad);
     }
 
@@ -104,18 +118,65 @@ class Manager extends Singleton
      */
     public function loadTrackerPlugins()
     {
-        $this->unloadPlugins();
-        $pluginsTracker = PiwikConfig::getInstance()->Plugins_Tracker['Plugins_Tracker'];
+        $cacheId = 'PluginsTracker';
+        $cache = Cache::getEagerCache();
+
+        if ($cache->contains($cacheId)) {
+            $pluginsTracker = $cache->fetch($cacheId);
+        } else {
+
+            $this->unloadPlugins();
+            $this->loadActivatedPlugins();
+
+            $pluginsTracker = array();
+
+            foreach ($this->loadedPlugins as $pluginName => $plugin) {
+                if ($this->isTrackerPlugin($plugin)) {
+                    $pluginsTracker[] = $pluginName;
+                }
+            }
+
+            if (!empty($pluginsTracker)) {
+                $cache->save($cacheId, $pluginsTracker);
+            }
+        }
+
         if (empty($pluginsTracker)) {
+            $this->unloadPlugins();
             return array();
         }
 
-        $pluginsTracker = array_diff($pluginsTracker, Tracker::getPluginsNotToLoad());
+        $pluginsTracker = array_diff($pluginsTracker, $this->getTrackerPluginsNotToLoad());
         $this->doNotLoadAlwaysActivatedPlugins();
         $this->loadPlugins($pluginsTracker);
+
+        // we could simply unload all plugins first before loading plugins but this way it is much faster
+        // since we won't have to create each plugin again and we won't have to parse each plugin metadata file
+        // again etc
+        $this->makeSureOnlyActivatedPluginsAreLoaded();
+
         return $pluginsTracker;
     }
 
+    /**
+     * Do not load the specified plugins (used during testing, to disable Provider plugin)
+     * @param array $plugins
+     */
+    public function setTrackerPluginsNotToLoad($plugins)
+    {
+        $this->trackerPluginsNotToLoad = $plugins;
+    }
+
+    /**
+     * Get list of plugins to not load
+     *
+     * @return array
+     */
+    public function getTrackerPluginsNotToLoad()
+    {
+        return $this->trackerPluginsNotToLoad;
+    }
+
     public function getCorePluginsDisabledByDefault()
     {
         return array_merge( $this->corePluginsDisabledByDefault, $this->coreThemesDisabledByDefault);
@@ -131,7 +192,7 @@ class Manager extends Singleton
     public function isPluginOfficialAndNotBundledWithCore($pluginName)
     {
         static $gitModules;
-        if(empty($gitModules)) {
+        if (empty($gitModules)) {
             $gitModules = file_get_contents(PIWIK_INCLUDE_PATH . '/.gitmodules');
         }
         // All submodules are officially maintained plugins
@@ -142,27 +203,16 @@ class Manager extends Singleton
     /**
      * Update Plugins config
      *
-     * @param array $plugins Plugins
+     * @param array $pluginsToLoad Plugins
      */
     private function updatePluginsConfig($pluginsToLoad)
     {
+        $pluginsToLoad = $this->sortPluginsSameOrderAsGlobalConfig($pluginsToLoad);
         $section = PiwikConfig::getInstance()->Plugins;
         $section['Plugins'] = $pluginsToLoad;
         PiwikConfig::getInstance()->Plugins = $section;
     }
 
-    /**
-     * Update Plugins_Tracker config
-     *
-     * @param array $plugins Plugins
-     */
-    private function updatePluginsTrackerConfig($plugins)
-    {
-        $section = PiwikConfig::getInstance()->Plugins_Tracker;
-        $section['Plugins_Tracker'] = $plugins;
-        PiwikConfig::getInstance()->Plugins_Tracker = $section;
-    }
-
     /**
      * Update PluginsInstalled config
      *
@@ -214,7 +264,7 @@ class Manager extends Singleton
     public function isPluginActivated($name)
     {
         return in_array($name, $this->pluginsToLoad)
-        || $this->isPluginAlwaysActivated($name);
+        || ($this->doLoadAlwaysActivatedPlugins && $this->isPluginAlwaysActivated($name));
     }
 
     /**
@@ -268,6 +318,13 @@ class Manager extends Singleton
         $this->removePluginFromConfig($pluginName);
 
         $this->clearCache($pluginName);
+
+        /**
+         * Event triggered after a plugin has been deactivated.
+         *
+         * @param string $pluginName The plugin that has been deactivated.
+         */
+        Piwik::postEvent('PluginManager.pluginDeactivated', array($pluginName));
     }
 
     /**
@@ -287,7 +344,7 @@ class Manager extends Singleton
      */
     public function findComponents($componentName, $expectedSubclass)
     {
-        $plugins    = $this->getLoadedPlugins();
+        $plugins    = $this->getPluginsLoadedAndActivated();
         $components = array();
 
         foreach ($plugins as $plugin) {
@@ -301,6 +358,22 @@ class Manager extends Singleton
         return $components;
     }
 
+    public function findMultipleComponents($directoryWithinPlugin, $expectedSubclass)
+    {
+        $plugins = $this->getPluginsLoadedAndActivated();
+        $found   = array();
+
+        foreach ($plugins as $plugin) {
+            $components = $plugin->findMultipleComponents($directoryWithinPlugin, $expectedSubclass);
+
+            if (!empty($components)) {
+                $found = array_merge($found, $components);
+            }
+        }
+
+        return $found;
+    }
+
     /**
      * Uninstalls a Plugin (deletes plugin files from the disk)
      * Only deactivated plugins can be uninstalled
@@ -314,7 +387,9 @@ class Manager extends Singleton
         if ($this->isPluginLoaded($pluginName)) {
             throw new \Exception("To uninstall the plugin $pluginName, first disable it in Piwik > Settings > Plugins");
         }
-        $this->returnLoadedPluginsInfo();
+        $this->loadAllPluginsAndGetTheirInfo();
+
+        \Piwik\Settings\Manager::cleanupPluginSettings($pluginName);
 
         $this->executePluginDeactivate($pluginName);
         $this->executePluginUninstall($pluginName);
@@ -324,9 +399,7 @@ class Manager extends Singleton
         $this->unloadPluginFromMemory($pluginName);
 
         $this->removePluginFromConfig($pluginName);
-
-        Option::delete('version_' . $pluginName);
-        \Piwik\Settings\Manager::cleanupPluginSettings($pluginName);
+        $this->removeInstalledVersionFromOptionTable($pluginName);
         $this->clearCache($pluginName);
 
         self::deletePluginFromFilesystem($pluginName);
@@ -357,6 +430,7 @@ class Manager extends Singleton
      */
     public function installLoadedPlugins()
     {
+        Log::debug("Loaded plugins: " . implode(", ", array_keys($this->getLoadedPlugins())));
         $messages = array();
         foreach ($this->getLoadedPlugins() as $plugin) {
             try {
@@ -365,6 +439,7 @@ class Manager extends Singleton
                 $messages[] = $e->getMessage();
             }
         }
+
         return $messages;
     }
 
@@ -403,6 +478,12 @@ class Manager extends Singleton
 
         $this->clearCache($pluginName);
 
+        /**
+         * Event triggered after a plugin has been activated.
+         *
+         * @param string $pluginName The plugin that has been activated.
+         */
+        Piwik::postEvent('PluginManager.pluginActivated', array($pluginName));
     }
 
     protected function isPluginInFilesystem($pluginName)
@@ -415,7 +496,7 @@ class Manager extends Singleton
 
     /**
      * Returns the currently enabled theme.
-     * 
+     *
      * If no theme is enabled, the **Morpheus** plugin is returned (this is the base and default theme).
      *
      * @return Plugin
@@ -476,7 +557,7 @@ class Manager extends Singleton
      *
      * @return array An array that maps plugin names with arrays of plugin information. Plugin
      *               information consists of the following entries:
-     * 
+     *
      *               - **activated**: Whether the plugin is activated.
      *               - **alwaysActivated**: Whether the plugin should always be activated,
      *                                      or not.
@@ -487,7 +568,7 @@ class Manager extends Singleton
      *                           See {@link Piwik\Plugin::getInformation()}.
      * @api
      */
-    public function returnLoadedPluginsInfo()
+    public function loadAllPluginsAndGetTheirInfo()
     {
         $language = Translate::getLanguageToLoad();
 
@@ -500,7 +581,7 @@ class Manager extends Singleton
         $listPlugins = array_unique($listPlugins);
         foreach ($listPlugins as $pluginName) {
             // Hide plugins that are never going to be used
-            if($this->isPluginBogus($pluginName)) {
+            if ($this->isPluginBogus($pluginName)) {
                 continue;
             }
 
@@ -539,10 +620,10 @@ class Manager extends Singleton
             );
             $plugins[$pluginName] = $info;
         }
+
         return $plugins;
     }
 
-
     protected static function isManifestFileFound($path)
     {
         return file_exists($path . "/" . MetadataLoader::PLUGIN_JSON_FILENAME);
@@ -556,43 +637,37 @@ class Manager extends Singleton
      */
     public function isPluginBundledWithCore($name)
     {
-        // Reading the plugins from the global.ini.php config file
-        $pluginsBundledWithPiwik = PiwikConfig::getInstance()->getFromGlobalConfig('Plugins');
-        $pluginsBundledWithPiwik = $pluginsBundledWithPiwik['Plugins'];
-
-        return (!empty($pluginsBundledWithPiwik)
-                    && in_array($name, $pluginsBundledWithPiwik))
+        return $this->isPluginEnabledByDefault($name)
                 || in_array($name, $this->getCorePluginsDisabledByDefault())
                 || $name == self::DEFAULT_THEME;
     }
 
     protected function isPluginThirdPartyAndBogus($pluginName)
     {
-        if($this->isPluginBundledWithCore($pluginName)) {
+        if ($this->isPluginBundledWithCore($pluginName)) {
             return false;
         }
-        if($this->isPluginBogus($pluginName)) {
+        if ($this->isPluginBogus($pluginName)) {
             return true;
         }
 
         $path = $this->getPluginsDirectory() . $pluginName;
-        if(!$this->isManifestFileFound($path)) {
+        if (!$this->isManifestFileFound($path)) {
             return true;
         }
         return false;
     }
 
-
     /**
-     * Load the specified plugins.
+     * Load AND activates the specified plugins. It will also overwrite all previously loaded plugins, so it acts
+     * as a setter. 
      *
      * @param array $pluginsToLoad Array of plugins to load.
      */
     public function loadPlugins(array $pluginsToLoad)
     {
-        $pluginsToLoad = array_unique($pluginsToLoad);
-        $this->pluginsToLoad = $pluginsToLoad;
-        $this->reloadPlugins();
+        $this->pluginsToLoad = $this->makePluginsToLoad($pluginsToLoad);
+        $this->reloadActivatedPlugins();
     }
 
     /**
@@ -621,11 +696,45 @@ class Manager extends Singleton
         if (empty($language)) {
             $language = Translate::getLanguageToLoad();
         }
-        $plugins = $this->getLoadedPlugins();
 
-        foreach ($plugins as $plugin) {
-            $this->loadTranslation($plugin, $language);
+        $cacheKey = 'PluginTranslations';
+
+        if (!empty($language)) {
+            $cacheKey .= '-' . trim($language);
         }
+
+        if (!empty($this->loadedPlugins)) {
+            // makes sure to create a translation in case loaded plugins change (ie Tests vs Tracker vs UI etc)
+            $cacheKey .= '-' . md5(implode('', $this->getLoadedPluginsName()));
+        }
+
+        $cache = Cache::getLazyCache();
+        $translations = $cache->fetch($cacheKey);
+
+        if (!empty($translations) &&
+            is_array($translations) &&
+            !Development::isEnabled()) { // TODO remove this one here once we have environments in DI
+
+            Translate::mergeTranslationArray($translations);
+            return;
+        }
+
+        $translations = array();
+        $pluginNames  = self::getAllPluginsNames();
+
+        foreach ($pluginNames as $pluginName) {
+            if ($this->isPluginLoaded($pluginName) ||
+                $this->isPluginBundledWithCore($pluginName)) {
+
+                $this->loadTranslation($pluginName, $language);
+
+                if (isset($GLOBALS['Piwik_translations'][$pluginName])) {
+                    $translations[$pluginName] = $GLOBALS['Piwik_translations'][$pluginName];
+                }
+            }
+        }
+
+        $cache->save($cacheKey, $translations, 43200); // ttl=12hours
     }
 
     /**
@@ -698,7 +807,7 @@ class Manager extends Singleton
         $plugins = $this->getLoadedPlugins();
         $enabled = $this->getActivatedPlugins();
 
-        if(empty($enabled)) {
+        if (empty($enabled)) {
             return array();
         }
         $enabled = array_combine($enabled, $enabled);
@@ -740,14 +849,8 @@ class Manager extends Singleton
      * Load the plugins classes installed.
      * Register the observers for every plugin.
      */
-    private function reloadPlugins()
+    private function reloadActivatedPlugins()
     {
-        if ($this->doLoadAlwaysActivatedPlugins) {
-            $this->pluginsToLoad = array_merge($this->pluginsToLoad, $this->pluginToAlwaysActivate);
-        }
-
-        $this->pluginsToLoad = array_unique($this->pluginsToLoad);
-
         $pluginsToPostPendingEventsTo = array();
         foreach ($this->pluginsToLoad as $pluginName) {
             if (!$this->isPluginLoaded($pluginName)
@@ -784,7 +887,6 @@ class Manager extends Singleton
         return $ignored;
     }
 
-
     /**
      * Returns the name of all plugins found in this Piwik instance
      * (including those not enabled and themes)
@@ -802,9 +904,9 @@ class Manager extends Singleton
         return $pluginsToLoad;
     }
 
-
     /**
-     * Loads the plugin filename and instantiates the plugin with the given name, eg. UserCountry
+     * Loads the plugin filename and instantiates the plugin with the given name, eg. UserCountry.
+     * Contrary to loadPlugins() it does not activate the plugin, it only loads it.
      *
      * @param string $pluginName
      * @throws \Exception
@@ -837,7 +939,6 @@ class Manager extends Singleton
 
         $path = self::getPluginsDirectory() . $pluginFileName;
 
-
         if (!file_exists($path)) {
             // Create the smallest minimal Piwik Plugin
             // Eg. Used for Morpheus default theme which does not have a Morpheus.php file
@@ -919,8 +1020,9 @@ class Manager extends Singleton
      *
      * @param string $pluginName plugin name without prefix (eg. 'UserCountry')
      * @param Plugin $newPlugin
+     * @internal
      */
-    private function addLoadedPlugin($pluginName, Plugin $newPlugin)
+    public function addLoadedPlugin($pluginName, Plugin $newPlugin)
     {
         $this->loadedPlugins[$pluginName] = $newPlugin;
     }
@@ -936,7 +1038,7 @@ class Manager extends Singleton
     private function loadTranslation($plugin, $langCode)
     {
         // we are in Tracker mode if Loader is not (yet) loaded
-        if (!class_exists('Piwik\\Loader', false)) {
+        if (SettingsServer::isTrackerApiRequest()) {
             return false;
         }
 
@@ -964,7 +1066,10 @@ class Manager extends Singleton
         }
 
         // merge in specific language translations (to overwrite english defaults)
-        if (file_exists($defaultLangPath)) {
+        if (!empty($langCode) &&
+            $defaultEnglishLangPath != $defaultLangPath &&
+            file_exists($defaultLangPath)) {
+
             $translations = $this->getTranslationsFromFile($defaultLangPath);
             $translationsLoaded = true;
             if (isset($translations[$pluginName])) {
@@ -1033,18 +1138,6 @@ class Manager extends Singleton
             $saveConfig = true;
         }
 
-        if ($this->isTrackerPlugin($plugin)) {
-            $pluginsTracker = PiwikConfig::getInstance()->Plugins_Tracker['Plugins_Tracker'];
-            if (is_null($pluginsTracker)) {
-                $pluginsTracker = array();
-            }
-            if (!in_array($pluginName, $pluginsTracker)) {
-                $pluginsTracker[] = $pluginName;
-                $this->updatePluginsTrackerConfig($pluginsTracker);
-                $saveConfig = true;
-            }
-        }
-
         if ($saveConfig) {
             PiwikConfig::getInstance()->forceSave();
         }
@@ -1052,6 +1145,16 @@ class Manager extends Singleton
 
     public function isTrackerPlugin(Plugin $plugin)
     {
+        $dimensions = VisitDimension::getDimensions($plugin);
+        if (!empty($dimensions)) {
+            return true;
+        }
+
+        $dimensions = ActionDimension::getDimensions($plugin);
+        if (!empty($dimensions)) {
+            return true;
+        }
+
         $hooks = $plugin->getListHooksRegistered();
         $hookNames = array_keys($hooks);
         foreach ($hookNames as $name) {
@@ -1062,6 +1165,12 @@ class Manager extends Singleton
                 return true;
             }
         }
+
+        $dimensions = ConversionDimension::getDimensions($plugin);
+        if (!empty($dimensions)) {
+            return true;
+        }
+
         return false;
     }
 
@@ -1099,18 +1208,6 @@ class Manager extends Singleton
         $this->updatePluginsConfig($pluginsEnabled);
     }
 
-    private function removePluginFromTrackerConfig($pluginName)
-    {
-        $pluginsTracker = PiwikConfig::getInstance()->Plugins_Tracker['Plugins_Tracker'];
-        if (!is_null($pluginsTracker)) {
-            $key = array_search($pluginName, $pluginsTracker);
-            if ($key !== false) {
-                unset($pluginsTracker[$key]);
-                $this->updatePluginsTrackerConfig($pluginsTracker);
-            }
-        }
-    }
-
     /**
      * @param  string $pathToTranslationFile
      * @throws \Exception
@@ -1191,7 +1288,6 @@ class Manager extends Singleton
     private function removePluginFromConfig($pluginName)
     {
         $this->removePluginFromPluginsConfig($pluginName);
-        $this->removePluginFromTrackerConfig($pluginName);
         PiwikConfig::getInstance()->forceSave();
     }
 
@@ -1205,6 +1301,40 @@ class Manager extends Singleton
             $plugin->uninstall();
         } catch (\Exception $e) {
         }
+
+        if (empty($plugin)) {
+            return;
+        }
+
+        try {
+            foreach (VisitDimension::getDimensions($plugin) as $dimension) {
+                $this->uninstallDimension($dimension);
+            }
+        } catch (\Exception $e) {
+        }
+
+        try {
+            foreach (ActionDimension::getDimensions($plugin) as $dimension) {
+                $this->uninstallDimension($dimension);
+            }
+        } catch (\Exception $e) {
+        }
+
+        try {
+            foreach (ConversionDimension::getDimensions($plugin) as $dimension) {
+                $this->uninstallDimension($dimension);
+            }
+        } catch (\Exception $e) {
+        }
+    }
+
+    /**
+     * @param ConversionDimension|VisitDimension|ActionDimension $dimension
+     */
+    private function uninstallDimension(Dimension $dimension)
+    {
+        $dimension->uninstall();
+        Option::delete('version_' . $dimension->getVersion());
     }
 
     /**
@@ -1216,6 +1346,82 @@ class Manager extends Singleton
         $pluginsInstalled = $this->getInstalledPluginsName();
         return in_array($pluginName, $pluginsInstalled);
     }
+
+    private function removeInstalledVersionFromOptionTable($version)
+    {
+        Option::delete('version_' . $version);
+    }
+
+    private function makeSureOnlyActivatedPluginsAreLoaded()
+    {
+        foreach ($this->getLoadedPlugins() as $pluginName => $plugin) {
+            if (!in_array($pluginName, $this->pluginsToLoad)) {
+                $this->unloadPlugin($plugin);
+            }
+        }
+    }
+
+    /**
+     * Reading the plugins from the global.ini.php config file
+     *
+     * @return array
+     */
+    protected function getPluginsFromGlobalIniConfigFile()
+    {
+        $pluginsBundledWithPiwik = PiwikConfig::getInstance()->getFromGlobalConfig('Plugins');
+        $pluginsBundledWithPiwik = $pluginsBundledWithPiwik['Plugins'];
+        return $pluginsBundledWithPiwik;
+    }
+
+    /**
+     * @param $name
+     * @return bool
+     */
+    protected function isPluginEnabledByDefault($name)
+    {
+        $pluginsBundledWithPiwik = $this->getPluginsFromGlobalIniConfigFile();
+
+        if(empty($pluginsBundledWithPiwik)) {
+            return false;
+        }
+        return in_array($name, $pluginsBundledWithPiwik);
+    }
+
+    /**
+     * @param array $pluginsToLoad
+     * @return array
+     */
+    private function makePluginsToLoad(array $pluginsToLoad)
+    {
+        $pluginsToLoad = array_unique($pluginsToLoad);
+        if ($this->doLoadAlwaysActivatedPlugins) {
+            $pluginsToLoad = array_merge($pluginsToLoad, $this->pluginToAlwaysActivate);
+        }
+        $pluginsToLoad = array_unique($pluginsToLoad);
+        $pluginsToLoad = $this->sortPluginsSameOrderAsGlobalConfig($pluginsToLoad);
+        return $pluginsToLoad;
+    }
+
+    private function sortPluginsSameOrderAsGlobalConfig(array $plugins)
+    {
+        $global = $this->getPluginsFromGlobalIniConfigFile();
+        if(empty($global)) {
+            return $plugins;
+        }
+        $global = array_values($global);
+        $plugins = array_values($plugins);
+
+        $defaultPluginsLoadedFirst = array_intersect($global, $plugins);
+
+        $otherPluginsToLoadAfterDefaultPlugins = array_diff($plugins, $defaultPluginsLoadedFirst);
+
+        // sort by name to have a predictable order for those extra plugins
+        sort($otherPluginsToLoadAfterDefaultPlugins);
+
+        $sorted = array_merge($defaultPluginsLoadedFirst, $otherPluginsToLoadAfterDefaultPlugins);
+
+        return $sorted;
+    }
 }
 
 /**
