@@ -95,12 +95,13 @@ class MetadataLoader
             return array();
         }
 
-        $info = Common::json_decode($json, $assoc = true);
+        $info = json_decode($json, $assoc = true);
         if (!is_array($info)
             || empty($info)
         ) {
             throw new Exception("Invalid JSON file: $path");
         }
+
         return $info;
     }
 }
