@@ -9,11 +9,12 @@
 
 namespace Piwik\Plugin;
 
+use Piwik\API\DataTablePostProcessor;
 use Piwik\Common;
 use Piwik\DataTable;
 use Piwik\Date;
 use Piwik\Log;
-use Piwik\MetricsFormatter;
+use Piwik\Metrics\Formatter\Html as HtmlFormatter;
 use Piwik\NoAccessException;
 use Piwik\Option;
 use Piwik\Period;
@@ -24,11 +25,11 @@ use Piwik\ViewDataTable\Manager as ViewDataTableManager;
 
 /**
  * The base class for report visualizations that output HTML and use JavaScript.
- * 
+ *
  * Report visualizations that extend from this class will be displayed like all others in
  * the Piwik UI. The following extra UI controls will be displayed around the visualization
  * itself:
- * 
+ *
  * - report documentation,
  * - a footer message (if {@link Piwik\ViewDataTable\Config::$show_footer_message} is set),
  * - a list of links to related reports (if {@link Piwik\ViewDataTable\Config::$related_reports} is set),
@@ -37,33 +38,33 @@ use Piwik\ViewDataTable\Manager as ViewDataTableManager;
  * - a limit control that allows users to change the amount of rows displayed (if
  *   {@link Piwik\ViewDataTable\Config::$show_limit_control} is true),
  * - and more depending on the visualization.
- * 
+ *
  * ### Rendering Process
- * 
+ *
  * The following process is used to render reports:
- * 
+ *
  * - The report is loaded through Piwik's Reporting API.
  * - The display and request properties that require report data in order to determine a default
  *   value are defaulted. These properties are:
- * 
+ *
  *   - {@link Piwik\ViewDataTable\Config::$columns_to_display}
  *   - {@link Piwik\ViewDataTable\RequestConfig::$filter_sort_column}
  *   - {@link Piwik\ViewDataTable\RequestConfig::$filter_sort_order}
- * 
+ *
  * - Priority filters are applied to the report (see {@link Piwik\ViewDataTable\Config::$filters}).
  * - The filters that are applied to every report in the Reporting API (called **generic filters**)
  *   are applied. (see {@link Piwik\API\Request})
  * - The report's queued filters are applied.
  * - A {@link Piwik\View} instance is created and rendered.
- * 
+ *
  * ### Rendering Hooks
- * 
+ *
  * The Visualization class defines several overridable methods that are called at specific
  * points during the rendering process. Derived classes can override these methods change
  * the data that is displayed or set custom properties.
- * 
+ *
  * The overridable methods (called **rendering hooks**) are as follows:
- * 
+ *
  * - **beforeLoadDataTable**: Called at the start of the rendering process before any data
  *                            is loaded.
  * - **beforeGenericFiltersAreAppliedToLoadedDataTable**: Called after data is loaded and after priority
@@ -76,20 +77,20 @@ use Piwik\ViewDataTable\Manager as ViewDataTableManager;
  * - **beforeRender**: Called immediately before a {@link Piwik\View} is created and rendered.
  * - **isThereDataToDisplay**: Called after a {@link Piwik\View} is created to determine if the report has
  *                             data or not. If not, a message is displayed to the user.
- * 
+ *
  * ### The DataTable JavaScript class
- * 
+ *
  * In the UI, visualization behavior is provided by logic in the **DataTable** JavaScript class.
  * When creating new visualizations, the **DataTable** JavaScript class (or one of its existing
  * descendants) should be extended.
- * 
+ *
  * To learn more read the [Visualizing Report Data](/guides/visualizing-report-data#creating-new-visualizations)
  * guide.
  *
  * ### Examples
- * 
+ *
  * **Changing the data that is loaded**
- * 
+ *
  *     class MyVisualization extends Visualization
  *     {
  *         // load the previous period's data as well as the requested data. this will change
@@ -101,20 +102,20 @@ use Piwik\ViewDataTable\Manager as ViewDataTableManager;
  *
  *             $this->requestConfig->request_parameters_to_modify['date'] = $previousDate . ',' . $date;
  *         }
- * 
+ *
  *         // since we load the previous period's data too, we need to override the logic to
  *         // check if there is data or not.
  *         public function isThereDataToDisplay()
  *         {
  *             $tables = $this->dataTable->getDataTables()
  *             $requestedDataTable = end($tables);
- * 
+ *
  *             return $requestedDataTable->getRowsCount() != 0;
  *         }
  *     }
- * 
+ *
  * **Force properties to be set to certain values**
- * 
+ *
  *     class MyVisualization extends Visualization
  *     {
  *         // ensure that some properties are set to certain values before rendering.
@@ -133,9 +134,9 @@ class Visualization extends ViewDataTable
 {
     /**
      * The Twig template file to use when rendering, eg, `"@MyPlugin/_myVisualization.twig"`.
-     * 
+     *
      * Must be defined by classes that extend Visualization.
-     * 
+     *
      * @api
      */
     const TEMPLATE_FILE = '';
@@ -143,6 +144,12 @@ class Visualization extends ViewDataTable
     private $templateVars = array();
     private $reportLastUpdatedMessage = null;
     private $metadata = null;
+    protected $metricsFormatter = null;
+
+    /**
+     * @var Report
+     */
+    protected $report;
 
     final public function __construct($controllerAction, $apiMethodToRequestDataTable, $params = array())
     {
@@ -152,7 +159,11 @@ class Visualization extends ViewDataTable
             throw new \Exception('You have not defined a constant named TEMPLATE_FILE in your visualization class.');
         }
 
+        $this->metricsFormatter = new HtmlFormatter();
+
         parent::__construct($controllerAction, $apiMethodToRequestDataTable, $params);
+
+        $this->report = Report::factory($this->requestConfig->getApiModuleToRequest(), $this->requestConfig->getApiMethodToRequest());
     }
 
     protected function buildView()
@@ -160,24 +171,23 @@ class Visualization extends ViewDataTable
         $this->overrideSomeConfigPropertiesIfNeeded();
 
         try {
-
             $this->beforeLoadDataTable();
 
-            $this->loadDataTableFromAPI(array('disable_generic_filters' => 1));
+            $this->loadDataTableFromAPI(array('disable_generic_filters' => 1, 'format_metrics' => 0));
             $this->postDataTableLoadedFromAPI();
 
             $requestPropertiesAfterLoadDataTable = $this->requestConfig->getProperties();
 
             $this->applyFilters();
+            $this->addVisualizationInfoFromMetricMetadata();
             $this->afterAllFiltersAreApplied();
             $this->beforeRender();
 
             $this->logMessageIfRequestPropertiesHaveChanged($requestPropertiesAfterLoadDataTable);
-
         } catch (NoAccessException $e) {
             throw $e;
         } catch (\Exception $e) {
-            Log::warning("Failed to get data from API: " . $e->getMessage() . "\n" . $e->getTraceAsString());
+            Log::error("Failed to get data from API: " . $e->getMessage() . "\n" . $e->getTraceAsString());
 
             $message = $e->getMessage();
             if (\Piwik_ShouldPrintBackTraceWithMessage()) {
@@ -253,9 +263,9 @@ class Visualization extends ViewDataTable
 
     /**
      * Returns `true` if there is data to display, `false` if otherwise.
-     * 
+     *
      * Derived classes should override this method if they change the amount of data that is loaded.
-     * 
+     *
      * @api
      */
     protected function isThereDataToDisplay()
@@ -286,7 +296,7 @@ class Visualization extends ViewDataTable
         }
 
         if (empty($this->requestConfig->filter_sort_column)) {
-            $this->requestConfig->setDefaultSort($this->config->columns_to_display, $hasNbUniqVisitors);
+            $this->requestConfig->setDefaultSort($this->config->columns_to_display, $hasNbUniqVisitors, $columns);
         }
 
         // deal w/ table metadata
@@ -297,6 +307,34 @@ class Visualization extends ViewDataTable
                 $this->config->report_last_updated_message = $this->makePrettyArchivedOnText();
             }
         }
+
+        $pivotBy = Common::getRequestVar('pivotBy', false) ?: $this->requestConfig->pivotBy;
+        if (empty($pivotBy)
+            && $this->dataTable instanceof DataTable
+        ) {
+            $this->config->disablePivotBySubtableIfTableHasNoSubtables($this->dataTable);
+        }
+    }
+
+    private function addVisualizationInfoFromMetricMetadata()
+    {
+        $dataTable = $this->dataTable instanceof DataTable\Map ? $this->dataTable->getFirstRow() : $this->dataTable;
+
+        $metrics = Report::getMetricsForTable($dataTable, $this->report);
+
+        // TODO: instead of iterating & calling translate everywhere, maybe we can get all translated names in one place.
+        //       may be difficult, though, since translated metrics are specific to the report.
+        foreach ($metrics as $metric) {
+            $name = $metric->getName();
+
+            if (empty($this->config->translations[$name])) {
+                $this->config->translations[$name] = $metric->getTranslatedName();
+            }
+
+            if (empty($this->config->metrics_documentation[$name])) {
+                $this->config->metrics_documentation[$name] = $metric->getDocumentation();
+            }
+        }
     }
 
     private function applyFilters()
@@ -312,13 +350,17 @@ class Visualization extends ViewDataTable
 
         if (!in_array($this->requestConfig->filter_sort_column, $this->config->columns_to_display)) {
             $hasNbUniqVisitors = in_array('nb_uniq_visitors', $this->config->columns_to_display);
-            $this->requestConfig->setDefaultSort($this->config->columns_to_display, $hasNbUniqVisitors);
+            $this->requestConfig->setDefaultSort($this->config->columns_to_display, $hasNbUniqVisitors, $this->dataTable->getColumns());
         }
 
+        $postProcessor = $this->makeDataTablePostProcessor(); // must be created after requestConfig is final
+
         if (!$this->requestConfig->areGenericFiltersDisabled()) {
-            $this->applyGenericFilters();
+            $this->dataTable = $postProcessor->applyGenericFilters($this->dataTable);
         }
 
+        $postProcessor->applyComputeProcessedMetrics($this->dataTable);
+
         $this->afterGenericFiltersAreAppliedToLoadedDataTable();
 
         // queue other filters so they can be applied later if queued filters are disabled
@@ -331,6 +373,14 @@ class Visualization extends ViewDataTable
         if (!$this->requestConfig->areQueuedFiltersDisabled()) {
             $this->dataTable->applyQueuedFilters();
         }
+
+        if ($this->requestConfig->shouldFormatMetrics()) {
+            $formatter = $this->metricsFormatter;
+            $report = $this->report;
+            $this->dataTable->filter(function (DataTable $table) use ($formatter, $report) {
+                $formatter->formatMetrics($table, $report);
+            });
+        }
     }
 
     private function removeEmptyColumnsFromDisplay()
@@ -365,9 +415,8 @@ class Visualization extends ViewDataTable
         $today    = mktime(0, 0, 0);
 
         if ($date->getTimestamp() > $today) {
-
             $elapsedSeconds = time() - $date->getTimestamp();
-            $timeAgo        = MetricsFormatter::getPrettyTimeFromSeconds($elapsedSeconds);
+            $timeAgo        = $this->metricsFormatter->getPrettyTimeFromSeconds($elapsedSeconds);
 
             return Piwik::translate('CoreHome_ReportGeneratedXAgo', $timeAgo);
         }
@@ -491,6 +540,7 @@ class Visualization extends ViewDataTable
             'filter_excludelowpop',
             'filter_excludelowpop_value',
         );
+
         foreach ($deleteFromJavascriptVariables as $name) {
             if (isset($javascriptVariablesToSet[$name])) {
                 unset($javascriptVariablesToSet[$name]);
@@ -507,7 +557,7 @@ class Visualization extends ViewDataTable
 
     /**
      * Hook that is called before loading report data from the API.
-     * 
+     *
      * Use this method to change the request parameters that is sent to the API when requesting
      * data.
      *
@@ -519,7 +569,7 @@ class Visualization extends ViewDataTable
 
     /**
      * Hook that is executed before generic filters are applied.
-     * 
+     *
      * Use this method if you need access to the entire dataset (since generic filters will
      * limit and truncate reports).
      *
@@ -559,10 +609,7 @@ class Visualization extends ViewDataTable
         // eg $this->config->showFooterColumns = true;
     }
 
-    /**
-     * Second, generic filters (Sort, Limit, Replace Column Names, etc.)
-     */
-    private function applyGenericFilters()
+    private function makeDataTablePostProcessor()
     {
         $requestArray = $this->request->getRequestArray();
         $request      = \Piwik\API\Request::getRequestArrayFromString($requestArray);
@@ -572,8 +619,7 @@ class Visualization extends ViewDataTable
             $request['filter_sort_order']  = '';
         }
 
-        $genericFilter = new \Piwik\API\DataTableGenericFilter($request);
-        $genericFilter->filter($this->dataTable);
+        return new DataTablePostProcessor($this->requestConfig->getApiModuleToRequest(), $this->requestConfig->getApiMethodToRequest(), $request);
     }
 
     private function logMessageIfRequestPropertiesHaveChanged(array $requestPropertiesBefore)
