@@ -10,16 +10,18 @@ namespace Piwik;
 
 use Exception;
 use Piwik\API\Request;
+use Piwik\Container\StaticContainer;
 use Piwik\DataTable\Row;
 use Piwik\DataTable\Simple;
 use Piwik\DataTable;
 use Piwik\Plugins\ImageGraph\API;
+use Piwik\BaseFactory;
 
 /**
  * A Report Renderer produces user friendly renderings of any given Piwik report.
  * All new Renderers must be copied in ReportRenderer and added to the $availableReportRenderers.
  */
-abstract class ReportRenderer
+abstract class ReportRenderer extends BaseFactory
 {
     const DEFAULT_REPORT_FONT = 'dejavusans';
     const REPORT_TEXT_COLOR = "68,68,68";
@@ -33,38 +35,28 @@ abstract class ReportRenderer
     const PDF_FORMAT = 'pdf';
     const CSV_FORMAT = 'csv';
 
-    static private $availableReportRenderers = array(
+    private static $availableReportRenderers = array(
         self::PDF_FORMAT,
         self::HTML_FORMAT,
         self::CSV_FORMAT,
     );
 
-    /**
-     * Return the ReportRenderer associated to the renderer type $rendererType
-     *
-     * @throws exception If the renderer is unknown
-     * @param string $rendererType
-     * @return \Piwik\ReportRenderer
-     */
-    static public function factory($rendererType)
+    protected static function getClassNameFromClassId($rendererType)
     {
-        $name = ucfirst(strtolower($rendererType));
-        $className = 'Piwik\ReportRenderer\\' . $name;
+        return 'Piwik\ReportRenderer\\' . self::normalizeRendererType($rendererType);
+    }
 
-        try {
-            Loader::loadClass($className);
-            return new $className;
-        } catch (Exception $e) {
+    protected static function getInvalidClassIdExceptionMessage($rendererType)
+    {
+        return Piwik::translate(
+            'General_ExceptionInvalidReportRendererFormat',
+            array(self::normalizeRendererType($rendererType), implode(', ', self::$availableReportRenderers))
+        );
+    }
 
-            @header('Content-Type: text/html; charset=utf-8');
-
-            throw new Exception(
-                Piwik::translate(
-                    'General_ExceptionInvalidReportRendererFormat',
-                    array($name, implode(', ', self::$availableReportRenderers))
-                )
-            );
-        }
+    protected static function normalizeRendererType($rendererType)
+    {
+        return ucfirst(strtolower($rendererType));
     }
 
     /**
@@ -153,8 +145,7 @@ abstract class ReportRenderer
      */
     protected static function getOutputPath($filename)
     {
-        $outputFilename = PIWIK_USER_PATH . '/tmp/assets/' . $filename;
-        $outputFilename = SettingsPiwik::rewriteTmpPathWithInstanceId($outputFilename);
+        $outputFilename = StaticContainer::getContainer()->get('path.tmp') . '/assets/' . $filename;
 
         @chmod($outputFilename, 0600);
         @unlink($outputFilename);
@@ -166,15 +157,11 @@ abstract class ReportRenderer
         $filename = self::appendExtension($filename, $extension);
         $outputFilename = self::getOutputPath($filename);
 
-        $emailReport = @fopen($outputFilename, "w");
-
-        if (!$emailReport) {
-            throw new Exception ("The file : " . $outputFilename . " can not be opened in write mode.");
+        $bytesWritten = file_put_contents($outputFilename, $content);
+        if ($bytesWritten === false) {
+            throw new Exception ("ReportRenderer: Could not write to file '" . $outputFilename . "'.");
         }
 
-        fwrite($emailReport, $content);
-        fclose($emailReport);
-
         return $outputFilename;
     }
 
