@@ -27,7 +27,6 @@ class Mysqli extends Db
     protected $charset;
     protected $activeTransaction = false;
 
-
     /**
      * Builds the DB object
      *
@@ -74,7 +73,14 @@ class Mysqli extends Db
             $timer = $this->initProfiler();
         }
 
-        $this->connection = mysqli_connect($this->host, $this->username, $this->password, $this->dbname, $this->port, $this->socket);
+        $this->connection = mysqli_init();
+
+        // Make sure MySQL returns all matched rows on update queries including
+        // rows that actually didn't have to be updated because the values didn't
+        // change. This matches common behaviour among other database systems.
+        // See #6296 why this is important in tracker
+        $flags = MYSQLI_CLIENT_FOUND_ROWS;
+        mysqli_real_connect($this->connection, $this->host, $this->username, $this->password, $this->dbname, $this->port, $this->socket, $flags);
         if (!$this->connection || mysqli_connect_errno()) {
             throw new DbException("Connect failed: " . mysqli_connect_error());
         }
@@ -206,8 +212,8 @@ class Mysqli extends Db
             return $result;
         } catch (Exception $e) {
             throw new DbException("Error query: " . $e->getMessage() . "
-								In query: $query
-								Parameters: " . var_export($parameters, true));
+                                   In query: $query
+                                   Parameters: " . var_export($parameters, true));
         }
     }
 
@@ -279,57 +285,62 @@ class Mysqli extends Db
         return mysqli_affected_rows($this->connection);
     }
 
-	/**
-	 * Start Transaction
-	 * @return string TransactionID
-	 */
+    /**
+     * Start Transaction
+     * @return string TransactionID
+     */
+    public function beginTransaction()
+    {
+        if (!$this->activeTransaction === false) {
+            return;
+        }
 
-	public function beginTransaction()
-	{
-		if(!$this->activeTransaction === false ) {
-			return;
-		}
+        if ( $this->connection->autocommit(false)) {
+            $this->activeTransaction = uniqid();
+            return $this->activeTransaction;
+        }
+    }
 
-		if( $this->connection->autocommit(false) ) {
-			$this->activeTransaction = uniqid();
-			return $this->activeTransaction;
-		}
-	}
+    /**
+     * Commit Transaction
+     * @param $xid
+     * @throws DbException
+     * @internal param TransactionID $string from beginTransaction
+     */
+    public function commit($xid)
+    {
+        if ($this->activeTransaction != $xid || $this->activeTransaction === false) {
 
-	/**
-	 * Commit Transaction
-	 * @param string TransactionID from beginTransaction
-	 */
+            return;
+        }
 
-	public function commit($xid)
-	{
-		if($this->activeTransaction !=  $xid || $this->activeTransaction === false  ) { 
-		
-			return;
-		}
-		$this->activeTransaction = false;
+        $this->activeTransaction = false;
 
-		if(!$this->connection->commit() ) {
-			throw new DbException("Commit failed"); 
-		}
-		$this->connection->autocommit(true);
-	}
+        if (!$this->connection->commit() ) {
+            throw new DbException("Commit failed");
+        }
 
-	/**
-	 * Rollback Transaction
-	 * @param string TransactionID from beginTransaction
-	 */
+        $this->connection->autocommit(true);
+    }
 
-	public function rollBack($xid)
-	{
-		if($this->activeTransaction !=  $xid || $this->activeTransaction === false  ) { 
-			return;
-		}
-		$this->activeTransaction = false;
+    /**
+     * Rollback Transaction
+     * @param $xid
+     * @throws DbException
+     * @internal param TransactionID $string from beginTransaction
+     */
+    public function rollBack($xid)
+    {
+        if ($this->activeTransaction != $xid || $this->activeTransaction === false) {
+            return;
+        }
 
-		if(!$this->connection->rollback() ) {
-			throw new DbException("Rollback failed"); 
-		}
-		$this->connection->autocommit(true); 
-	}
+        $this->activeTransaction = false;
+
+        if (!$this->connection->rollback() ) {
+            throw new DbException("Rollback failed");
+        }
+
+        $this->connection->autocommit(true);
+    }
 }
