@@ -49,21 +49,25 @@ class PageUrl
 
         if (empty($parsedUrl['query'])) {
             if (empty($parsedUrl['fragment'])) {
+
                 return UrlHelper::getParseUrlReverse($parsedUrl);
             }
+
             // Exclude from the hash tag as well
             $queryParameters = UrlHelper::getArrayFromQueryString($parsedUrl['fragment']);
             $parsedUrl['fragment'] = UrlHelper::getQueryStringWithExcludedParameters($queryParameters, $parametersToExclude);
             $url = UrlHelper::getParseUrlReverse($parsedUrl);
+
             return $url;
         }
+
         $queryParameters = UrlHelper::getArrayFromQueryString($parsedUrl['query']);
         $parsedUrl['query'] = UrlHelper::getQueryStringWithExcludedParameters($queryParameters, $parametersToExclude);
         $url = UrlHelper::getParseUrlReverse($parsedUrl);
+
         return $url;
     }
 
-
     /**
      * Returns the array of parameters names that must be excluded from the Query String in all tracked URLs
      * @static
@@ -80,23 +84,20 @@ class PageUrl
         );
 
         $website = Cache::getCacheWebsiteAttributes($idSite);
-        $excludedParameters = isset($website['excluded_parameters'])
-            ? $website['excluded_parameters']
-            : array();
+        $excludedParameters = self::getExcludedParametersFromWebsite($website);
 
         if (!empty($excludedParameters)) {
             Common::printDebug('Excluding parameters "' . implode(',', $excludedParameters) . '" from URL');
         }
 
         $parametersToExclude = array_merge($excludedParameters,
-            self::$queryParametersToExclude,
-            $campaignTrackingParameters);
+                                           self::$queryParametersToExclude,
+                                           $campaignTrackingParameters);
 
         $parametersToExclude = array_map('strtolower', $parametersToExclude);
         return $parametersToExclude;
     }
 
-
     /**
      * Returns true if URL fragments should be removed for a specific site,
      * false if otherwise.
@@ -154,8 +155,9 @@ class PageUrl
         if (empty($parsedUrl)) {
             return $parsedUrl;
         }
+
         if (!empty($parsedUrl['host'])) {
-            $parsedUrl['host'] = mb_strtolower($parsedUrl['host'], 'UTF-8');
+            $parsedUrl['host'] = Common::mb_strtolower($parsedUrl['host'], 'UTF-8');
         }
 
         if (!empty($parsedUrl['fragment'])) {
@@ -176,19 +178,24 @@ class PageUrl
     public static function convertMatrixUrl($originalUrl)
     {
         $posFirstSemiColon = strpos($originalUrl, ";");
-        if ($posFirstSemiColon === false) {
+
+        if (false === $posFirstSemiColon) {
             return $originalUrl;
         }
+
         $posQuestionMark = strpos($originalUrl, "?");
-        $replace = ($posQuestionMark === false);
+        $replace = (false === $posQuestionMark);
+
         if ($posQuestionMark > $posFirstSemiColon) {
             $originalUrl = substr_replace($originalUrl, ";", $posQuestionMark, 1);
             $replace = true;
         }
+
         if ($replace) {
             $originalUrl = substr_replace($originalUrl, "?", strpos($originalUrl, ";"), 1);
             $originalUrl = str_replace(";", "&", $originalUrl);
         }
+
         return $originalUrl;
     }
 
@@ -212,10 +219,12 @@ class PageUrl
     {
         if (is_string($value)) {
             $decoded = urldecode($value);
-            if (@mb_check_encoding($decoded, $encoding)) {
+            if (function_exists('mb_check_encoding')
+                && @mb_check_encoding($decoded, $encoding)) {
                 $value = urlencode(mb_convert_encoding($decoded, 'UTF-8', $encoding));
             }
         }
+
         return $value;
     }
 
@@ -228,6 +237,7 @@ class PageUrl
                 $value = PageUrl::reencodeParameterValue($value, $encoding);
             }
         }
+
         return $queryParameters;
     }
 
@@ -247,14 +257,20 @@ class PageUrl
      */
     public static function reencodeParameters(&$queryParameters, $encoding = false)
     {
-        // if query params are encoded w/ non-utf8 characters (due to browser bug or whatever),
-        // encode to UTF-8.
-        if ($encoding !== false
-            && strtolower($encoding) != 'utf-8'
-            && function_exists('mb_check_encoding')
-        ) {
-            $queryParameters = PageUrl::reencodeParametersArray($queryParameters, $encoding);
+        if (function_exists('mb_check_encoding')) {
+            // if query params are encoded w/ non-utf8 characters (due to browser bug or whatever),
+            // encode to UTF-8.
+            if (strtolower($encoding) != 'utf-8'
+                && $encoding != false
+            ) {
+                Common::printDebug("Encoding page URL query parameters to $encoding.");
+
+                $queryParameters = PageUrl::reencodeParametersArray($queryParameters, $encoding);
+            }
+        } else {
+            Common::printDebug("Page charset supplied in tracking request, but mbstring extension is not available.");
         }
+
         return $queryParameters;
     }
 
@@ -263,6 +279,7 @@ class PageUrl
         $url = Common::unsanitizeInputValue($url);
         $url = PageUrl::cleanupString($url);
         $url = PageUrl::convertMatrixUrl($url);
+
         return $url;
     }
 
@@ -276,6 +293,7 @@ class PageUrl
     public static function reconstructNormalizedUrl($url, $prefixId)
     {
         $map = array_flip(self::$urlPrefixMap);
+
         if ($prefixId !== null && isset($map[$prefixId])) {
             $fullUrl = $map[$prefixId] . $url;
         } else {
@@ -285,7 +303,8 @@ class PageUrl
         // Clean up host & hash tags, for URLs
         $parsedUrl = @parse_url($fullUrl);
         $parsedUrl = PageUrl::cleanupHostAndHashTag($parsedUrl);
-        $url = UrlHelper::getParseUrlReverse($parsedUrl);
+        $url       = UrlHelper::getParseUrlReverse($parsedUrl);
+
         if (!empty($url)) {
             return $url;
         }
@@ -304,12 +323,14 @@ class PageUrl
     {
         foreach (self::$urlPrefixMap as $prefix => $id) {
             if (strtolower(substr($url, 0, strlen($prefix))) == $prefix) {
+
                 return array(
                     'url'      => substr($url, strlen($prefix)),
                     'prefixId' => $id
                 );
             }
         }
+
         return array('url' => $url, 'prefixId' => null);
     }
 
@@ -319,10 +340,30 @@ class PageUrl
 
         if (!UrlHelper::isLookLikeUrl($url)) {
             Common::printDebug("WARNING: URL looks invalid and is discarded");
-            $url = false;
-            return $url;
+
+            return false;
         }
+
         return $url;
     }
-}
 
+    private static function getExcludedParametersFromWebsite($website)
+    {
+        if (isset($website['excluded_parameters'])) {
+            return $website['excluded_parameters'];
+        }
+
+        return array();
+    }
+
+    public static function urldecodeValidUtf8($value)
+    {
+        $value = urldecode($value);
+        if (function_exists('mb_check_encoding')
+            && !@mb_check_encoding($value, 'utf-8')
+        ) {
+            return urlencode($value);
+        }
+        return $value;
+    }
+}
\ No newline at end of file
