@@ -8,7 +8,6 @@
  */
 namespace Piwik\Tracker;
 
-use DeviceDetector\Parser\Bot;
 use Piwik\Common;
 use Piwik\Config;
 use Piwik\DeviceDetectorFactory;
@@ -27,16 +26,16 @@ class VisitExcluded
      */
     public function __construct(Request $request, $ip = false, $userAgent = false)
     {
-        if ($ip === false) {
+        if (false === $ip) {
             $ip = $request->getIp();
         }
 
-        if ($userAgent === false) {
+        if (false === $userAgent) {
             $userAgent = $request->getUserAgent();
         }
 
-        $this->request = $request;
-        $this->idSite = $request->getIdSite();
+        $this->request   = $request;
+        $this->idSite    = $request->getIdSite();
         $this->userAgent = $userAgent;
         $this->ip = $ip;
     }
@@ -168,8 +167,17 @@ class VisitExcluded
     {
         return array(
             // Google
-            '66.249.0.0/16',
-            '64.233.172.0/24',
+            '216.239.32.0/19',
+            '64.233.160.0/19',
+            '66.249.80.0/20',
+            '72.14.192.0/18',
+            '209.85.128.0/17',
+            '66.102.0.0/20',
+            '74.125.0.0/16',
+            '64.18.0.0/20',
+            '207.126.144.0/20',
+            '173.194.0.0/16',
+
             // Live/Bing/MSN
             '64.4.0.0/18',
             '65.52.0.0/14',
@@ -181,6 +189,7 @@ class VisitExcluded
             '207.68.192.0/20',
             '131.253.26.0/20',
             '131.253.24.0/20',
+
             // Yahoo
             '72.30.198.0/20',
             '72.30.196.0/20',
@@ -200,6 +209,7 @@ class VisitExcluded
             Common::printDebug('Piwik ignore cookie was found, visit not tracked.');
             return true;
         }
+
         return false;
     }
 
@@ -211,12 +221,15 @@ class VisitExcluded
     protected function isVisitorIpExcluded()
     {
         $websiteAttributes = Cache::getCacheWebsiteAttributes($this->idSite);
+
         if (!empty($websiteAttributes['excluded_ips'])) {
-            if (IP::isIpInRange($this->ip, $websiteAttributes['excluded_ips'])) {
-                Common::printDebug('Visitor IP ' . IP::N2P($this->ip) . ' is excluded from being tracked');
+            $ip = \Piwik\Network\IP::fromBinaryIP($this->ip);
+            if ($ip->isInRanges($websiteAttributes['excluded_ips'])) {
+                Common::printDebug('Visitor IP ' . $ip->toString() . ' is excluded from being tracked');
                 return true;
             }
         }
+
         return false;
     }
 
@@ -232,6 +245,7 @@ class VisitExcluded
     protected function isUserAgentExcluded()
     {
         $websiteAttributes = Cache::getCacheWebsiteAttributes($this->idSite);
+
         if (!empty($websiteAttributes['excluded_user_agents'])) {
             foreach ($websiteAttributes['excluded_user_agents'] as $excludedUserAgent) {
                 // if the excluded user agent string part is in this visit's user agent, this visit should be excluded
@@ -240,6 +254,7 @@ class VisitExcluded
                 }
             }
         }
+
         return false;
     }
 
@@ -254,12 +269,15 @@ class VisitExcluded
         $spamHosts = explode(",", $spamHosts);
 
         $referrerUrl = $this->request->getParam('urlref');
+
         foreach($spamHosts as $spamHost) {
-            if( strpos($referrerUrl, $spamHost) !== false) {
+            $spamHost = trim($spamHost);
+            if ( strpos($referrerUrl, $spamHost) !== false) {
                 Common::printDebug('Referrer URL is a known spam: ' . $spamHost);
                 return true;
             }
         }
+
         return false;
     }
 }
