@@ -9,7 +9,6 @@
 
 namespace Piwik\Translate\Filter;
 
-
 /**
  */
 class ByParameterCount extends FilterAbstract
@@ -37,14 +36,15 @@ class ByParameterCount extends FilterAbstract
     {
         $cleanedTranslations = array();
 
-        foreach ($translations AS $pluginName => $pluginTranslations) {
+        foreach ($translations as $pluginName => $pluginTranslations) {
 
-            foreach ($pluginTranslations AS $key => $translation) {
+            foreach ($pluginTranslations as $key => $translation) {
 
                 if (isset($this->baseTranslations[$pluginName][$key])) {
                     $baseTranslation = $this->baseTranslations[$pluginName][$key];
                 } else {
-                    $baseTranslation = '';
+                    // english string was deleted, do not error
+                    continue;
                 }
 
                 // ensure that translated strings have the same number of %s as the english source strings
