@@ -9,7 +9,6 @@
 
 namespace Piwik\Translate\Validate;
 
-
 use Piwik\Common;
 
 /**
@@ -19,7 +18,6 @@ class CoreTranslations extends ValidateAbstract
     /**
      * Error States
      */
-    const ERRORSTATE_MINIMUMTRANSLATIONS = 'At least 250 translations required';
     const ERRORSTATE_LOCALEREQUIRED = 'Locale required';
     const ERRORSTATE_TRANSLATORINFOREQUIRED = 'Translator info required';
     const ERRORSTATE_TRANSLATOREMAILREQUIRED = 'Translator email required';
@@ -55,11 +53,6 @@ class CoreTranslations extends ValidateAbstract
     {
         $this->message = null;
 
-        if (250 > count($translations, COUNT_RECURSIVE)) {
-            $this->message = self::ERRORSTATE_MINIMUMTRANSLATIONS;
-            return false;
-        }
-
         if (empty($translations['General']['Locale'])) {
             $this->message = self::ERRORSTATE_LOCALEREQUIRED;
             return false;
