@@ -10,6 +10,7 @@
 namespace Piwik\Translate;
 
 use Exception;
+use Piwik\Container\StaticContainer;
 use Piwik\Filesystem;
 use Piwik\Piwik;
 use Piwik\Translate\Filter\FilterAbstract;
@@ -71,7 +72,7 @@ class Writer
     protected $filterMessages = array();
 
     const UNFILTERED = 'unfiltered';
-    const FILTERED = 'filtered';
+    const FILTERED   = 'filtered';
 
     protected $currentState = self::UNFILTERED;
 
@@ -152,12 +153,14 @@ class Writer
     public function getTranslations($lang)
     {
         $path = $this->getTranslationPathBaseDirectory('lang', $lang);
+
         if (!is_readable($path)) {
             return array();
         }
 
         $data = file_get_contents($path);
         $translations = json_decode($data, true);
+
         return $translations;
     }
 
@@ -198,16 +201,19 @@ class Writer
         if (!empty($this->pluginName)) {
 
             if ($base == 'tmp') {
-                return sprintf('%s/tmp/plugins/%s/lang/%s.json', PIWIK_INCLUDE_PATH, $this->pluginName, $lang);
+                return sprintf('%s/plugins/%s/lang/%s.json', StaticContainer::getContainer()->get('path.tmp'), $this->pluginName, $lang);
             } else {
                 return sprintf('%s/plugins/%s/lang/%s.json', PIWIK_INCLUDE_PATH, $this->pluginName, $lang);
             }
         }
 
+        if ($base == 'tmp') {
+            return sprintf('%s/%s.json', StaticContainer::getContainer()->get('path.tmp'), $lang);
+        }
+
         return sprintf('%s/%s/%s.json', PIWIK_INCLUDE_PATH, $base, $lang);
     }
 
-
     /**
      * Converts translations to a string that can be written to a file
      *
@@ -292,7 +298,7 @@ class Writer
 
         $this->validationMessage = null;
 
-        foreach ($this->validators AS $validator) {
+        foreach ($this->validators as $validator) {
             if (!$validator->isValid($this->translations)) {
                 $this->validationMessage = $validator->getMessage();
                 return false;
@@ -362,7 +368,7 @@ class Writer
 
         $cleanedTranslations = $this->translations;
 
-        foreach ($this->filters AS $filter) {
+        foreach ($this->filters as $filter) {
 
             $cleanedTranslations = $filter->filter($cleanedTranslations);
             $filteredData = $filter->getFilteredData();
