@@ -9,8 +9,9 @@
 namespace Piwik;
 
 use Exception;
+use Piwik\Container\StaticContainer;
 use Piwik\DataTable\Filter\SafeDecodeLabel;
-use Piwik\Period\Range;
+use Piwik\Metrics\Formatter;
 use Piwik\Translate;
 use Piwik\View\RenderTokenParser;
 use Piwik\Visualization\Sparkline;
@@ -35,6 +36,8 @@ class Twig
      */
     private $twig;
 
+    private $formatter;
+
     public function __construct()
     {
         $loader = $this->getDefaultThemeLoader();
@@ -44,9 +47,11 @@ class Twig
 		$manager = Plugin\Manager::getInstance();
 		$theme   = $manager->getThemeEnabled();
 		$loaders = array();
-		
+
+        $this->formatter = new Formatter();
+
 		//create loader for custom theme to overwrite twig templates
-		if($theme && $theme->getPluginName() != \Piwik\Plugin\Manager::DEFAULT_THEME) {
+		if ($theme && $theme->getPluginName() != \Piwik\Plugin\Manager::DEFAULT_THEME) {
 			$customLoader = $this->getCustomThemeLoader($theme);
 			if ($customLoader) {
 				//make it possible to overwrite plugin templates
@@ -56,12 +61,11 @@ class Twig
 		}
 
 		$loaders[] = $loader;
-        
+
         $chainLoader = new Twig_Loader_Chain($loaders);
 
         // Create new Twig Environment and set cache dir
-        $templatesCompiledPath = PIWIK_USER_PATH . '/tmp/templates_c';
-        $templatesCompiledPath = SettingsPiwik::rewriteTmpPathWithInstanceId($templatesCompiledPath);
+        $templatesCompiledPath = StaticContainer::getContainer()->get('path.tmp') . '/templates_c';
 
         $this->twig = new Twig_Environment($chainLoader,
             array(
@@ -187,7 +191,7 @@ class Twig
 	 * @return \Twig_Loader_Filesystem
 	 */
 	protected function getCustomThemeLoader(Plugin $theme){
-		if(!file_exists(sprintf("%s/plugins/%s/templates/", PIWIK_INCLUDE_PATH, $theme->getPluginName()))){
+		if (!file_exists(sprintf("%s/plugins/%s/templates/", PIWIK_INCLUDE_PATH, $theme->getPluginName()))){
 			return false;
 		}
 		$themeLoader = new Twig_Loader_Filesystem(array(
@@ -245,7 +249,7 @@ class Twig
     protected function addFilter_prettyDate()
     {
         $prettyDate = new Twig_SimpleFilter('prettyDate', function ($dateString, $period) {
-            return Range::factory($period, $dateString)->getLocalizedShortString();
+            return Period\Factory::build($period, $dateString)->getLocalizedShortString();
         });
         $this->twig->addFilter($prettyDate);
     }
@@ -273,21 +277,23 @@ class Twig
 
     protected function addFilter_money()
     {
-        $moneyFilter = new Twig_SimpleFilter('money', function ($amount) {
+        $formatter = $this->formatter;
+        $moneyFilter = new Twig_SimpleFilter('money', function ($amount) use ($formatter) {
             if (func_num_args() != 2) {
                 throw new Exception('the money modifier expects one parameter: the idSite.');
             }
             $idSite = func_get_args();
             $idSite = $idSite[1];
-            return MetricsFormatter::getPrettyMoney($amount, $idSite);
+            return $formatter->getPrettyMoney($amount, $idSite);
         });
         $this->twig->addFilter($moneyFilter);
     }
 
     protected function addFilter_sumTime()
     {
-        $sumtimeFilter = new Twig_SimpleFilter('sumtime', function ($numberOfSeconds) {
-            return MetricsFormatter::getPrettyTimeFromSeconds($numberOfSeconds);
+        $formatter = $this->formatter;
+        $sumtimeFilter = new Twig_SimpleFilter('sumtime', function ($numberOfSeconds) use ($formatter) {
+            return $formatter->getPrettyTimeFromSeconds($numberOfSeconds, true);
         });
         $this->twig->addFilter($sumtimeFilter);
     }
