@@ -10,7 +10,7 @@ namespace Piwik;
 
 /**
  * Contains less commonly needed URL helper methods.
- * 
+ *
  */
 class UrlHelper
 {
@@ -101,16 +101,16 @@ class UrlHelper
      */
     public static function isLookLikeUrl($url)
     {
-        return preg_match('~^(ftp|news|http|https)?://(.*)$~D', $url, $matches) !== 0
-        && strlen($matches[2]) > 0;
+        return preg_match('~^((ftp|news|http|https)?:)?//(.*)$~D', $url, $matches) !== 0
+        && strlen($matches[3]) > 0;
     }
 
     /**
      * Returns a URL created from the result of the [parse_url](http://php.net/manual/en/function.parse-url.php)
      * function.
-     * 
+     *
      * Copied from the PHP comments at [http://php.net/parse_url](http://php.net/parse_url).
-     * 
+     *
      * @param array $parsed Result of [parse_url](http://php.net/manual/en/function.parse-url.php).
      * @return false|string The URL or `false` if `$parsed` isn't an array.
      * @api
@@ -234,7 +234,6 @@ class UrlHelper
         return $result;
     }
 
-
     /**
      * Extracts a keyword from a raw not encoded URL.
      * Will only extract keyword if a known search engine has been detected.
@@ -394,20 +393,23 @@ class UrlHelper
                     $key = self::getParameterFromQueryString($query, $variableName);
                     $key = trim(urldecode($key));
 
-                    // Special case: Google & empty q parameter
+                    // Special cases: empty or no keywords
                     if (empty($key)
-                        && $variableName == 'q'
-
                         && (
                             // Google search with no keyword
                             ($searchEngineName == 'Google'
-                                && ( // First, they started putting an empty q= parameter
-                                    strpos($query, '&q=') !== false
-                                    || strpos($query, '?q=') !== false
-                                    // then they started sending the full host only (no path/query string)
-                                    || (empty($query) && (empty($referrerPath) || $referrerPath == '/') && empty($referrerParsed['fragment']))
-                                )
+                                && (empty($query) && (empty($referrerPath) || $referrerPath == '/') && empty($referrerParsed['fragment']))
                             )
+
+                            // Yahoo search with no keyword
+                            || ($searchEngineName == 'Yahoo!'
+                                && ($referrerParsed['host'] == 'r.search.yahoo.com')
+                            )
+
+                            // empty keyword parameter
+                            || strpos($query, sprintf('&%s=', $variableName)) !== false
+                            || strpos($query, sprintf('?%s=', $variableName)) !== false
+
                             // search engines with no keyword
                             || $searchEngineName == 'Google Images'
                             || $searchEngineName == 'DuckDuckGo')
