@@ -10,15 +10,16 @@ namespace Piwik\ViewDataTable;
 
 use Piwik\Common;
 use Piwik\Piwik;
+use Piwik\Plugin\Report;
 use Piwik\Plugins\CoreVisualizations\Visualizations\HtmlTable;
 
 /**
  * Provides a means of creating {@link Piwik\Plugin\ViewDataTable} instances by ID.
  *
  * ### Examples
- * 
+ *
  * **Creating a ViewDataTable for a report**
- * 
+ *
  *     // method in MyPlugin\Controller
  *     public function myReport()
  *     {
@@ -27,9 +28,9 @@ use Piwik\Plugins\CoreVisualizations\Visualizations\HtmlTable;
  *         $view->config->translations['myFancyMetric'] = "My Fancy Metric";
  *         return $view->render();
  *     }
- * 
+ *
  * **Displaying a report in another way**
- * 
+ *
  *     // method in MyPlugin\Controller
  *     // use the same data that's used in myReport() above, but transform it in some way before
  *     // displaying.
@@ -39,9 +40,9 @@ use Piwik\Plugins\CoreVisualizations\Visualizations\HtmlTable;
  *         $view->config->filters[] = array('MyMagicFilter', array('an arg', 'another arg'));
  *         return $view->render();
  *     }
- * 
+ *
  * **Force a report to be shown as a bar graph**
- * 
+ *
  *     // method in MyPlugin\Controller
  *     // force the myReport report to show as a bar graph if there is no viewDataTable query param,
  *     // even though it is configured to show as a table.
@@ -51,7 +52,7 @@ use Piwik\Plugins\CoreVisualizations\Visualizations\HtmlTable;
  *                                $forceDefault = true);
  *         return $view->render();
  *     }
- * 
+ *
  *
  * @api
  */
@@ -96,10 +97,6 @@ class Factory
 
         $defaultViewType = self::getDefaultViewTypeForReport($apiAction);
 
-        if (!$forceDefault && !empty($defaultViewType)) {
-            $defaultType = $defaultViewType;
-        }
-
         $isWidget = Common::getRequestVar('widget', '0', 'string');
 
         if (!empty($isWidget)) {
@@ -114,14 +111,26 @@ class Factory
             $savedViewDataTable = $params['viewDataTable'];
         }
 
-        $type = Common::getRequestVar('viewDataTable', $savedViewDataTable, 'string');
+        // order of default viewDataTables' priority is: function specified default, saved default, configured default for report
+        //   function specified default is preferred
+        // -> force default == true : defaultType ?: saved ?: defaultView
+        // -> force default == false : saved ?: defaultType ?: defaultView
+        if ($forceDefault) {
+            $defaultType = $defaultType ?: $savedViewDataTable ?: $defaultViewType;
+        } else {
+            $defaultType = $savedViewDataTable ?: $defaultType ?: $defaultViewType;
+        }
+
+        $type = Common::getRequestVar('viewDataTable', $defaultType, 'string');
 
         // Common::getRequestVar removes backslashes from the defaultValue in case magic quotes are enabled.
         // therefore do not pass this as a default value to getRequestVar()
         if ('' === $type) {
-            $type = $defaultType ? : HtmlTable::ID;
+            $type = $defaultType ?: HtmlTable::ID;
         }
 
+        $params['viewDataTable'] = $type;
+
         $visualizations = Manager::getAvailableViewDataTables();
 
         if (array_key_exists($type, $visualizations)) {
@@ -148,6 +157,13 @@ class Factory
      */
     private static function getDefaultViewTypeForReport($apiAction)
     {
+        list($module, $action) = explode('.', $apiAction);
+        $report = Report::factory($module, $action);
+
+        if (!empty($report) && $report->isEnabled()) {
+            return $report->getDefaultTypeViewDataTable();
+        }
+
         $defaultViewTypes = self::getDefaultTypeViewDataTable();
         return isset($defaultViewTypes[$apiAction]) ? $defaultViewTypes[$apiAction] : false;
     }
@@ -161,24 +177,7 @@ class Factory
         if (null === self::$defaultViewTypes) {
             self::$defaultViewTypes = array();
             /**
-             * Triggered when gathering the default view types for all available reports.
-             * 
-             * If you define your own report, you may want to subscribe to this event to
-             * make sure the correct default Visualization is used (for example, a pie graph,
-             * bar graph, or something else).
-             *
-             * If there is no default type associated with a report, the **table** visualization
-             * used.
-             * 
-             * **Example**
-             * 
-             *     public function getDefaultTypeViewDataTable(&$defaultViewTypes)
-             *     {
-             *         $defaultViewTypes['Referrers.getSocials']       = HtmlTable::ID;
-             *         $defaultViewTypes['Referrers.getUrlsForSocial'] = Pie::ID;
-             *     }
-             * 
-             * @param array &$defaultViewTypes The array mapping report IDs with visualization IDs.
+             * @ignore
              */
             Piwik::postEvent('ViewDataTable.getDefaultType', array(&self::$defaultViewTypes));
         }
