@@ -8,14 +8,16 @@
  */
 namespace Piwik;
 
-use Piwik\Plugin\Manager as PluginManager;
+use Piwik\Cache as PiwikCache;
+use Piwik\Plugin\Report;
+use Piwik\Plugin\Widgets;
 
 /**
  * Manages the global list of reports that can be displayed as dashboard widgets.
- * 
+ *
  * Reports are added as dashboard widgets through the {@hook WidgetsList.addWidgets}
  * event. Observers for this event should call the {@link add()} method to add reports.
- * 
+ *
  * @api
  * @method static \Piwik\WidgetsList getInstance()
  */
@@ -26,14 +28,21 @@ class WidgetsList extends Singleton
      *
      * @var array
      */
-    static protected $widgets = array();
+    protected static $widgets = array();
 
     /**
      * Indicates whether the hook was posted or not
      *
      * @var bool
      */
-    static protected $hookCalled = false;
+    protected static $hookCalled = false;
+
+    /**
+     * In get() we won't use a cached result in case this is true. Instead we will sort the widgets again and cache
+     * a new result. To make tests work...
+     * @var bool
+     */
+    private static $listCacheToBeInvalidated = false;
 
     /**
      * Returns all available widgets.
@@ -52,19 +61,33 @@ class WidgetsList extends Singleton
      *               )
      *               ```
      */
-    static public function get()
+    public static function get()
     {
+        $cache   = self::getCacheForCompleteList();
+        $cacheId = self::getCacheId();
+
+        if (!self::$listCacheToBeInvalidated && $cache->contains($cacheId)) {
+            return $cache->fetch($cacheId);
+        }
+
         self::addWidgets();
 
         uksort(self::$widgets, array('Piwik\WidgetsList', '_sortWidgetCategories'));
 
         $widgets = array();
         foreach (self::$widgets as $key => $v) {
-            if (isset($widgets[Piwik::translate($key)])) {
-                $v = array_merge($widgets[Piwik::translate($key)], $v);
+            $category = Piwik::translate($key);
+
+            if (isset($widgets[$category])) {
+                $v = array_merge($widgets[$category], $v);
             }
-            $widgets[Piwik::translate($key)] = $v;
+
+            $widgets[$category] = $v;
         }
+
+        $cache->save($cacheId, $widgets);
+        self::$listCacheToBeInvalidated = false;
+
         return $widgets;
     }
 
@@ -79,12 +102,25 @@ class WidgetsList extends Singleton
              */
             Piwik::postEvent('WidgetsList.addWidgets');
 
-            /** @var \Piwik\Plugin\Widgets[] $widgets */
-            $widgets     = PluginManager::getInstance()->findComponents('Widgets', 'Piwik\\Plugin\\Widgets');
             $widgetsList = self::getInstance();
 
-            foreach ($widgets as $widget) {
-                $widget->configure($widgetsList);
+            foreach (Report::getAllReports() as $report) {
+                if ($report->isEnabled()) {
+                    $report->configureWidget($widgetsList);
+                }
+            }
+
+            $widgetContainers = Widgets::getAllWidgets();
+            foreach ($widgetContainers as $widgetContainer) {
+                $widgets = $widgetContainer->getWidgets();
+
+                foreach ($widgets as $widget) {
+                    $widgetsList->add($widget['category'], $widget['name'], $widget['module'], $widget['method'], $widget['params']);
+                }
+            }
+
+            foreach ($widgetContainers as $widgetContainer) {
+                $widgetContainer->configureWidgetsList($widgetsList);
             }
         }
     }
@@ -125,19 +161,17 @@ class WidgetsList extends Singleton
     }
 
     /**
-     * Adds a report to the list of dashboard widgets.
+     * Returns the unique id of an widget with the given parameters
      *
-     * @param string $widgetCategory The widget category. This can be a translation token.
-     * @param string $widgetName The name of the widget. This can be a translation token.
-     * @param string $controllerName The report's controller name (same as the plugin name).
-     * @param string $controllerAction The report's controller action method name.
-     * @param array $customParameters Extra query parameters that should be sent while getting
-     *                                this report.
+     * @param $controllerName
+     * @param $controllerAction
+     * @param array $customParameters
+     * @return string
      */
-    static public function add($widgetCategory, $widgetName, $controllerName, $controllerAction, $customParameters = array())
+    public static function getWidgetUniqueId($controllerName, $controllerAction, $customParameters = array())
     {
-        $widgetName = Piwik::translate($widgetName);
         $widgetUniqueId = 'widget' . $controllerName . $controllerAction;
+
         foreach ($customParameters as $name => $value) {
             if (is_array($value)) {
                 // use 'Array' for backward compatibility;
@@ -147,10 +181,29 @@ class WidgetsList extends Singleton
             $widgetUniqueId .= $name . $value;
         }
 
+        return $widgetUniqueId;
+    }
+
+    /**
+     * Adds a report to the list of dashboard widgets.
+     *
+     * @param string $widgetCategory The widget category. This can be a translation token.
+     * @param string $widgetName The name of the widget. This can be a translation token.
+     * @param string $controllerName The report's controller name (same as the plugin name).
+     * @param string $controllerAction The report's controller action method name.
+     * @param array $customParameters Extra query parameters that should be sent while getting
+     *                                this report.
+     */
+    public static function add($widgetCategory, $widgetName, $controllerName, $controllerAction, $customParameters = array())
+    {
+        $widgetName     = Piwik::translate($widgetName);
+        $widgetUniqueId = self::getWidgetUniqueId($controllerName, $controllerAction, $customParameters);
+
         if (!array_key_exists($widgetCategory, self::$widgets)) {
             self::$widgets[$widgetCategory] = array();
         }
 
+        self::$listCacheToBeInvalidated = true;
         self::$widgets[$widgetCategory][] = array(
             'name'       => $widgetName,
             'uniqueId'   => $widgetUniqueId,
@@ -162,13 +215,13 @@ class WidgetsList extends Singleton
 
     /**
      * Removes one or more widgets from the widget list.
-     * 
+     *
      * @param string $widgetCategory The widget category. Can be a translation token.
-     * @param string|false $widgetName The name of the widget to remove. Cannot be a 
+     * @param string|false $widgetName The name of the widget to remove. Cannot be a
      *                                 translation token. If not supplied, the entire category
      *                                 will be removed.
      */
-    static public function remove($widgetCategory, $widgetName = false)
+    public static function remove($widgetCategory, $widgetName = false)
     {
         if (!isset(self::$widgets[$widgetCategory])) {
             return;
@@ -176,11 +229,13 @@ class WidgetsList extends Singleton
 
         if (empty($widgetName)) {
             unset(self::$widgets[$widgetCategory]);
+            self::$listCacheToBeInvalidated = true;
             return;
         }
         foreach (self::$widgets[$widgetCategory] as $id => $widget) {
             if ($widget['name'] == $widgetName || $widget['name'] == Piwik::translate($widgetName)) {
                 unset(self::$widgets[$widgetCategory][$id]);
+                self::$listCacheToBeInvalidated = true;
                 return;
             }
         }
@@ -193,10 +248,10 @@ class WidgetsList extends Singleton
      * @param string $controllerAction The controller action of the report.
      * @return bool
      */
-    static public function isDefined($controllerName, $controllerAction)
+    public static function isDefined($controllerName, $controllerAction)
     {
         $widgetsList = self::get();
-        foreach ($widgetsList as $widgetCategory => $widgets) {
+        foreach ($widgetsList as $widgets) {
             foreach ($widgets as $widget) {
                 if ($widget['parameters']['module'] == $controllerName
                     && $widget['parameters']['action'] == $controllerAction
@@ -217,5 +272,16 @@ class WidgetsList extends Singleton
     {
         self::$widgets    = array();
         self::$hookCalled = false;
+        self::getCacheForCompleteList()->delete(self::getCacheId());
+    }
+
+    private static function getCacheId()
+    {
+        return CacheId::pluginAware('WidgetsList');
+    }
+
+    private static function getCacheForCompleteList()
+    {
+        return PiwikCache::getTransientCache();
     }
 }
