@@ -11,6 +11,8 @@
 
 namespace Piwik;
 
+use Piwik\Container\StaticContainer;
+
 if (!defined('PIWIK_INCLUDE_PATH')) {
     define('PIWIK_INCLUDE_PATH', realpath(dirname(__FILE__) . "/../.."));
 }
@@ -32,16 +34,33 @@ if (!Common::isPhpCliMode()) {
 $testmode = in_array('--testmode', $_SERVER['argv']);
 if ($testmode) {
     require_once PIWIK_INCLUDE_PATH . "/tests/PHPUnit/TestingEnvironment.php";
-
     \Piwik_TestingEnvironment::addHooks();
 }
 
+
+function getPiwikDomain()
+{
+    foreach($_SERVER['argv'] as $param) {
+        $pattern = '--piwik-domain=';
+        if(false !== strpos($param, $pattern)) {
+            return substr($param, strlen($pattern));
+        }
+    }
+    return null;
+}
+
+$piwikDomain = getPiwikDomain();
+if($piwikDomain) {
+    Url::setHost($piwikDomain);
+}
+
 $token = Db::get()->fetchOne("SELECT token_auth
                               FROM " . Common::prefixTable("user") . "
                               WHERE superuser_access = 1
                               ORDER BY date_registered ASC");
 
-$filename = PIWIK_INCLUDE_PATH . '/tmp/cache/token.php';
+$filename = StaticContainer::getContainer()->get('path.tmp') . '/cache/token.php';
+
 $content  = "<?php exit; //\t" . $token;
 file_put_contents($filename, $content);
 echo $filename;
\ No newline at end of file
