@@ -47,13 +47,6 @@ except ImportError:
             print >> sys.stderr, 'simplejson (http://pypi.python.org/pypi/simplejson/) is required.'
             sys.exit(1)
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    try:
-        from ordereddict import OrderedDict
-    except ImportError:
-        pass
 
 
 ##
@@ -64,16 +57,14 @@ STATIC_EXTENSIONS = set((
     'gif jpg jpeg png bmp ico svg ttf eot woff class swf css js xml robots.txt'
 ).split())
 
-
 DOWNLOAD_EXTENSIONS = set((
-    '7z aac arc arj asf asx avi bin csv deb dmg doc exe flv gz gzip hqx '
+    '7z aac arc arj asf asx avi bin csv deb dmg doc docx exe flv gz gzip hqx '
     'jar mpg mp2 mp3 mp4 mpeg mov movie msi msp odb odf odg odp '
-    'ods odt ogg ogv pdf phps ppt qt qtm ra ram rar rpm sea sit tar tbz '
-    'bz2 tbz tgz torrent txt wav wma wmv wpd xls xml xsd z zip '
-    'azw3 epub mobi'
+    'ods odt ogg ogv pdf phps ppt pptx qt qtm ra ram rar rpm sea sit tar tbz '
+    'bz2 tbz tgz torrent txt wav wma wmv wpd xls xlsx xml xsd z zip '
+    'azw3 epub mobi apk'
 ).split())
 
-
 # A good source is: http://phpbb-bots.blogspot.com/
 EXCLUDED_USER_AGENTS = (
     'adsbot-google',
@@ -104,7 +95,6 @@ EXCLUDED_USER_AGENTS = (
     'yandex',
 )
 
-
 PIWIK_MAX_ATTEMPTS = 3
 PIWIK_DELAY_AFTER_FAILURE = 2
 
@@ -112,8 +102,6 @@ PIWIK_EXPECTED_IMAGE = base64.b64decode(
     'R0lGODlhAQABAIAAAAAAAAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=='
 )
 
-
-
 ##
 ## Formats.
 ##
@@ -134,13 +122,12 @@ class BaseFormat(object):
     def check_format_line(self, line):
         return False
 
-
 class JsonFormat(BaseFormat):
     def __init__(self, name):
         super(JsonFormat, self).__init__(name)
         self.json = None
         self.date_format = '%Y-%m-%dT%H:%M:%S'
-    
+
     def check_format_line(self, line):
         try:
             self.json = json.loads(line)
@@ -160,7 +147,7 @@ class JsonFormat(BaseFormat):
         # Some ugly patchs ...
         if key == 'generation_time_milli':
             self.json[key] =  int(self.json[key] * 1000)
-        # Patch date format ISO 8601 
+        # Patch date format ISO 8601
         elif key == 'date':
             tz = self.json[key][19:]
             self.json['timezone'] = tz.replace(':', '')
@@ -170,12 +157,10 @@ class JsonFormat(BaseFormat):
             return self.json[key]
         except KeyError:
             raise BaseFormatException()
-    
+
     def get_all(self,):
         return self.json
 
-
-
 class RegexFormat(BaseFormat):
 
     def __init__(self, name, regex, date_format=None):
@@ -201,54 +186,126 @@ class RegexFormat(BaseFormat):
 
     def get_all(self,):
         return self.matched.groupdict()
-            
 
+class W3cExtendedFormat(RegexFormat):
 
+    FIELDS_LINE_PREFIX = '#Fields: '
 
-class IisFormat(RegexFormat):
+    fields = {
+        'date': '(?P<date>^\d+[-\d+]+',
+        'time': '[\d+:]+)[.\d]*?', # TODO should not assume date & time will be together not sure how to fix ATM.
+        'cs-uri-stem': '(?P<path>/\S*)',
+        'cs-uri-query': '(?P<query_string>\S*)',
+        'c-ip': '"?(?P<ip>[\d*.]*)"?',
+        'cs(User-Agent)': '(?P<user_agent>".*?"|\S+)',
+        'cs(Referer)': '(?P<referrer>\S+)',
+        'sc-status': '(?P<status>\d+)',
+        'sc-bytes': '(?P<length>\S+)',
+        'cs-host': '(?P<host>\S+)',
+        'cs-username': '(?P<userid>\S+)',
+        'time-taken': '(?P<generation_time_secs>[.\d]+)'
+    }
 
     def __init__(self):
-        super(IisFormat, self).__init__('iis', None, '%Y-%m-%d %H:%M:%S')
+        super(W3cExtendedFormat, self).__init__('w3c_extended', None, '%Y-%m-%d %H:%M:%S')
 
     def check_format(self, file):
-        line = file.readline()
-        if not line.startswith('#Software: Microsoft Internet Information Services '):
+        # collect all header lines and the first line of the logfile
+        header_lines = []
+        while True:
+            line = file.readline()
+
+            if line.startswith('#'):
+                header_lines.append(line)
+            else:
+                break
+        first_line = line
+        fields_line = next((line for line in header_lines if line.startswith(W3cExtendedFormat.FIELDS_LINE_PREFIX)), None)
+
+        if not header_lines or not fields_line:
             file.seek(0)
             return
-        # Skip the next 2 lines.
-        for i in xrange(2):
-            file.readline()
-        # Parse the 4th line (regex)
+
+        # store the header lines for a later check for IIS
+        self.header_lines = header_lines
+
+        # Parse the 4th 'Fields: ' line to create the regex to use
         full_regex = []
-        line = file.readline()
-        fields = {
-            'date': '(?P<date>^\d+[-\d+]+',
-            'time': '[\d+:]+)',
-            'cs-uri-stem': '(?P<path>/\S*)',
-            'cs-uri-query': '(?P<query_string>\S*)',
-            'c-ip': '(?P<ip>[\d*.]*)',
-            'cs(User-Agent)': '(?P<user_agent>\S+)',
-            'cs(Referer)': '(?P<referrer>\S+)',
-            'sc-status': '(?P<status>\d+)',
-            'sc-bytes': '(?P<length>\S+)',
-            'cs-host': '(?P<host>\S+)',
-        }
+
+        expected_fields = type(self).fields.copy() # turn custom field mapping into field => regex mapping
+
+        # if the --w3c-time-taken-millisecs option is used, make sure the time-taken field is interpreted as milliseconds
+        if config.options.w3c_time_taken_in_millisecs:
+            expected_fields['time-taken'] = '(?P<generation_time_milli>[\d.]+)'
+
+        for mapped_field_name, field_name in config.options.custom_w3c_fields.iteritems():
+            expected_fields[mapped_field_name] = type(self).fields[field_name]
+            del expected_fields[field_name]
+
         # Skip the 'Fields: ' prefix.
-        line = line[9:]
-        for field in line.split():
+        fields_line = fields_line[9:]
+        for field in fields_line.split():
             try:
-                regex = fields[field]
+                regex = expected_fields[field]
             except KeyError:
                 regex = '\S+'
             full_regex.append(regex)
-        self.regex = re.compile(' '.join(full_regex))
+        full_regex = '\s+'.join(full_regex)
+        self.regex = re.compile(full_regex)
 
-        start_pos = file.tell()
-        nextline = file.readline()
-        file.seek(start_pos)
-        return self.check_format_line(nextline)
+        file.seek(0)
+        return self.check_format_line(first_line)
 
+    def check_for_iis_option(self):
+        if not config.options.w3c_time_taken_in_millisecs and self._is_time_taken_milli() and self._is_iis():
+            logging.info("WARNING: IIS log file being parsed without --w3c-time-taken-milli option. IIS"
+                         " stores millisecond values in the time-taken field. If your logfile does this, the aforementioned"
+                         " option must be used in order to get accurate generation times.")
 
+    def _is_iis(self):
+        return len([line for line in self.header_lines if 'internet information services' in line.lower() or 'iis' in line.lower()]) > 0
+
+    def _is_time_taken_milli(self):
+        return 'generation_time_milli' not in self.regex.pattern
+
+class IisFormat(W3cExtendedFormat):
+
+    fields = W3cExtendedFormat.fields.copy()
+    fields.update({
+        'time-taken': '(?P<generation_time_milli>[.\d]+)',
+        'sc-win32-status': '(?P<__win32_status>\S+)' # this group is useless for log importing, but capturing it
+                                                     # will ensure we always select IIS for the format instead of
+                                                     # W3C logs when detecting the format. This way there will be
+                                                     # less accidental importing of IIS logs w/o --w3c-time-taken-milli.
+    })
+
+    def __init__(self):
+        super(IisFormat, self).__init__()
+
+        self.name = 'iis'
+
+class AmazonCloudFrontFormat(W3cExtendedFormat):
+
+    fields = W3cExtendedFormat.fields.copy()
+    fields.update({
+        'x-event': '(?P<event_action>\S+)',
+        'x-sname': '(?P<event_name>\S+)',
+        'cs-uri-stem': '(?:rtmp:/)?(?P<path>/\S*)',
+        'c-user-agent': '(?P<user_agent>".*?"|\S+)'
+    })
+
+    def __init__(self):
+        super(AmazonCloudFrontFormat, self).__init__()
+
+        self.name = 'amazon_cloudfront'
+
+    def get(self, key):
+        if key == 'event_category' and 'event_category' not in self.matched.groupdict():
+            return 'cloudfront_rtmp'
+        elif key == 'status' and 'status' not in self.matched.groupdict():
+            return '200'
+        else:
+            return super(AmazonCloudFrontFormat, self).get(key)
 
 _HOST_PREFIX = '(?P<host>[\w\-\.]*)(?::\d+)? '
 _COMMON_LOG_FORMAT = (
@@ -272,20 +329,18 @@ FORMATS = {
     'common_vhost': RegexFormat('common_vhost', _HOST_PREFIX + _COMMON_LOG_FORMAT),
     'ncsa_extended': RegexFormat('ncsa_extended', _NCSA_EXTENDED_LOG_FORMAT),
     'common_complete': RegexFormat('common_complete', _HOST_PREFIX + _NCSA_EXTENDED_LOG_FORMAT),
+    'w3c_extended': W3cExtendedFormat(),
+    'amazon_cloudfront': AmazonCloudFrontFormat(),
     'iis': IisFormat(),
     's3': RegexFormat('s3', _S3_LOG_FORMAT),
     'icecast2': RegexFormat('icecast2', _ICECAST2_LOG_FORMAT),
     'nginx_json': JsonFormat('nginx_json'),
 }
 
-
-
-
 ##
 ## Code.
 ##
 
-
 class Configuration(object):
     """
     Stores all the configuration options by reading sys.argv and parsing,
@@ -384,11 +439,11 @@ class Configuration(object):
         )
         option_parser.add_option(
             '--exclude-path', dest='excluded_paths', action='append', default=[],
-            help="Paths to exclude. Can be specified multiple times"
+            help="Any URL path matching this exclude-path will not be imported in Piwik. Can be specified multiple times"
         )
         option_parser.add_option(
             '--exclude-path-from', dest='exclude_path_from',
-            help="Each line from this file is a path to exclude"
+            help="Each line from this file is a path to exclude (see: --exclude-path)"
         )
         option_parser.add_option(
             '--include-path', dest='included_paths', action='append', default=[],
@@ -407,7 +462,7 @@ class Configuration(object):
         option_parser.add_option(
             '--enable-static', dest='enable_static',
             action='store_true', default=False,
-            help="Track static files (images, css, js, etc.)"
+            help="Track static files (images, css, js, ico, ttf, etc.)"
         )
         option_parser.add_option(
             '--enable-bots', dest='enable_bots',
@@ -505,8 +560,38 @@ class Configuration(object):
             '--enable-testmode', dest='enable_testmode', default=False, action='store_true',
             help="If set, it will try to get the token_auth from the piwik_tests directory"
         )
+        option_parser.add_option(
+            '--download-extensions', dest='download_extensions', default=None,
+            help="By default Piwik tracks as Downloads the most popular file extensions. If you set this parameter (format: pdf,doc,...) then files with an extension found in the list will be imported as Downloads, other file extensions downloads will be skipped."
+        )
+        option_parser.add_option(
+            '--w3c-map-field', action='callback', callback=self._set_w3c_field_map, type='string',
+            help="Map a custom log entry field in your W3C log to a default one. Use this option to load custom log "
+                 "files that use the W3C extended log format such as those from the Advanced Logging W3C module. Used "
+                 "as, eg, --w3c-map-field my-date=date. Recognized default fields include: %s\n\n"
+                 "Formats that extend the W3C extended log format (like the cloudfront RTMP log format) may define more "
+                 "fields that can be mapped."
+                     % (', '.join(W3cExtendedFormat.fields.keys()))
+        )
+        option_parser.add_option(
+            '--w3c-time-taken-millisecs', action='store_true', default=False, dest='w3c_time_taken_in_millisecs',
+            help="If set, interprets the time-taken W3C log field as a number of milliseconds. This must be set for importing"
+                 " IIS logs."
+        )
         return option_parser
 
+    def _set_w3c_field_map(self, option, opt_str, value, parser):
+        parts = value.split('=')
+
+        if len(parts) != 2:
+            fatal_error("Invalid --w3c-map-field option: '%s'" % value)
+
+        custom_name, default_name = parts
+
+        if not hasattr(parser.values, 'custom_w3c_fields'):
+            parser.values.custom_w3c_fields = {}
+
+        parser.values.custom_w3c_fields[custom_name] = default_name
 
     def _parse_args(self, option_parser):
         """
@@ -558,6 +643,15 @@ class Configuration(object):
         else:
             self.format = None
 
+        if not hasattr(self.options, 'custom_w3c_fields'):
+            self.options.custom_w3c_fields = {}
+        elif self.format is not None:
+            # validate custom field mappings
+            for custom_name, default_name in self.options.custom_w3c_fields.iteritems():
+                if default_name not in type(format).fields:
+                    fatal_error("custom W3C field mapping error: don't know how to parse and use the '%' field" % default_name)
+                    return
+
         if not self.options.piwik_url:
             fatal_error('no URL given for Piwik')
 
@@ -575,11 +669,14 @@ class Configuration(object):
         if self.options.recorders < 1:
             self.options.recorders = 1
 
+        if self.options.download_extensions:
+            self.options.download_extensions = set(self.options.download_extensions.split(','))
+        else:
+            self.options.download_extensions = DOWNLOAD_EXTENSIONS
 
     def __init__(self):
         self._parse_args(self._create_parser())
 
-
     def _get_token_auth(self):
         """
         If the token auth is not specified in the options, get it from Piwik.
@@ -644,24 +741,24 @@ class Configuration(object):
                 except:
                     fatal_error("We couldn't detect PHP. You can run the importer using the --login and --password option to fix this issue")
 
-
             command = [phpBinary, updatetokenfile]
             if self.options.enable_testmode:
                 command.append('--testmode')
 
+            hostname = urlparse.urlparse( self.options.piwik_url ).hostname
+            command.append('--piwik-domain=' + hostname )
+
             command = subprocess.list2cmdline(command)
             process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
             [stdout, stderr] = process.communicate()
             if process.returncode != 0:
                 fatal_error("`" + command + "` failed with error: " + stderr + ".\nReponse code was: " + str(process.returncode) + ". You can alternatively run the importer using the --login and --password option")
 
-
             filename = stdout
             credentials = open(filename, 'r').readline()
             credentials = credentials.split('\t')
             return credentials[1]
 
-
     def get_resolver(self):
         if self.options.site_id:
             logging.debug('Resolver: static')
@@ -670,8 +767,6 @@ class Configuration(object):
             logging.debug('Resolver: dynamic')
             return DynamicResolver()
 
-
-
 class Statistics(object):
     """
     Store statistics about parsed logs and recorded entries.
@@ -699,7 +794,6 @@ class Statistics(object):
         def __str__(self):
             return str(int(self.value))
 
-
     def __init__(self):
         self.time_start = None
         self.time_stop = None
@@ -727,12 +821,13 @@ class Statistics(object):
         self.count_lines_skipped_http_redirects = self.Counter()
         # Downloads
         self.count_lines_downloads = self.Counter()
+        # Ignored downloads when --download-extensions is used
+        self.count_lines_skipped_downloads = self.Counter()
 
         # Misc
         self.dates_recorded = set()
         self.monitor_stop = False
 
-
     def set_time_start(self):
         self.time_start = time.time()
 
@@ -773,13 +868,14 @@ Logs import summary
     %(count_lines_recorded)d requests imported successfully
     %(count_lines_downloads)d requests were downloads
     %(total_lines_ignored)d requests ignored:
-        %(count_lines_invalid)d invalid log lines
-        %(count_lines_skipped_user_agent)d requests done by bots, search engines, ...
         %(count_lines_skipped_http_errors)d HTTP errors
         %(count_lines_skipped_http_redirects)d HTTP redirects
-        %(count_lines_static)d requests to static resources (css, js, ...)
+        %(count_lines_invalid)d invalid log lines
         %(count_lines_no_site)d requests did not match any known site
-        %(count_lines_hostname_skipped)d requests did not match any requested hostname
+        %(count_lines_hostname_skipped)d requests did not match any --hostname
+        %(count_lines_skipped_user_agent)d requests done by bots, search engines...
+        %(count_lines_static)d requests to static resources (css, js, images, ico, ttf...)
+        %(count_lines_skipped_downloads)d requests to file downloads did not match any --download-extensions
 
 Website import summary
 ----------------------
@@ -807,6 +903,7 @@ Performance summary
             self.count_lines_skipped_http_errors.value,
             self.count_lines_skipped_http_redirects.value,
             self.count_lines_static.value,
+            self.count_lines_skipped_downloads.value,
             self.count_lines_no_site.value,
             self.count_lines_hostname_skipped.value,
         ]),
@@ -815,6 +912,7 @@ Performance summary
     'count_lines_skipped_http_errors': self.count_lines_skipped_http_errors.value,
     'count_lines_skipped_http_redirects': self.count_lines_skipped_http_redirects.value,
     'count_lines_static': self.count_lines_static.value,
+    'count_lines_skipped_downloads': self.count_lines_skipped_downloads.value,
     'count_lines_no_site': self.count_lines_no_site.value,
     'count_lines_hostname_skipped': self.count_lines_hostname_skipped.value,
     'total_sites': len(self.piwik_sites),
@@ -848,7 +946,6 @@ Performance summary
         )),
 }
 
-
     ##
     ## The monitor is a thread that prints a short summary each second.
     ##
@@ -875,8 +972,6 @@ Performance summary
     def stop_monitor(self):
         self.monitor_stop = True
 
-
-
 class Piwik(object):
     """
     Make requests to Piwik.
@@ -917,7 +1012,7 @@ class Piwik(object):
         """
         args = {
             'module' : 'API',
-            'format' : 'json',
+            'format' : 'json2',
             'method' : method,
         }
         # token_auth, by default, is taken from config.
@@ -950,7 +1045,6 @@ class Piwik(object):
             truncate_after = 4000
             raise urllib2.URLError('Piwik returned an invalid response: ' + res[:truncate_after])
 
-
     @staticmethod
     def _call_wrapper(func, expected_response, on_failure, *args, **kwargs):
         """
@@ -994,7 +1088,6 @@ class Piwik(object):
     def call_api(cls, method, **kwargs):
         return cls._call_wrapper(cls._call_api, None, None, method, **kwargs)
 
-
 ##
 ## Resolvers.
 ##
@@ -1009,17 +1102,9 @@ class StaticResolver(object):
     def __init__(self, site_id):
         self.site_id = site_id
         # Go get the main URL
-        sites = piwik.call_api(
+        site = piwik.call_api(
             'SitesManager.getSiteFromId', idSite=self.site_id
         )
-        try:
-            site = sites[0]
-        except (IndexError, KeyError):
-            logging.debug('response for SitesManager.getSiteFromId: %s', str(sites))
-
-            fatal_error(
-                "cannot get the main URL of this site: invalid site ID: %s" % site_id
-            )
         if site.get('result') == 'error':
             fatal_error(
                 "cannot get the main URL of this site: %s" % site.get('message')
@@ -1033,7 +1118,6 @@ class StaticResolver(object):
     def check_format(self, format):
         pass
 
-
 class DynamicResolver(object):
     """
     Use Piwik API to determine the site ID.
@@ -1144,7 +1228,6 @@ class DynamicResolver(object):
         else:
             return self._resolve_by_host(hit)
 
-
     def check_format(self, format):
         if config.options.replay_tracking:
             pass
@@ -1154,9 +1237,6 @@ class DynamicResolver(object):
                 "specify the Piwik site ID with the --idsite argument"
             )
 
-
-
-
 class Recorder(object):
     """
     A Recorder fetches hits from the Queue and inserts them into Piwik using
@@ -1197,7 +1277,7 @@ class Recorder(object):
         # We have to do this so visits from the same IP will be added in the right order.
         hits_by_client = [[] for r in cls.recorders]
         for hit in all_hits:
-            hits_by_client[abs(hash(hit.ip)) % len(cls.recorders)].append(hit)
+            hits_by_client[hit.get_visitor_id_hash() % len(cls.recorders)].append(hit)
 
         for i, recorder in enumerate(cls.recorders):
             recorder.queue.put(hits_by_client[i])
@@ -1285,8 +1365,9 @@ class Recorder(object):
             'cdt': self.date_to_piwik(hit.date),
             'idsite': site_id,
             'dp': '0' if config.options.reverse_dns else '1',
-            'ua': hit.user_agent.encode('utf8'),
+            'ua': hit.user_agent.encode('utf8')
         }
+
         if config.options.replay_tracking:
             # prevent request to be force recorded when option replay-tracking
             args['rec'] = '0'
@@ -1315,7 +1396,15 @@ class Recorder(object):
 			)
 
         if hit.generation_time_milli > 0:
-            args['gt_ms'] = hit.generation_time_milli
+            args['gt_ms'] = int(hit.generation_time_milli)
+
+        if hit.event_category and hit.event_action:
+            args['e_c'] = hit.event_category
+            args['e_a'] = hit.event_action
+
+            if hit.event_name:
+                args['e_n'] = hit.event_name
+
         return args
 
     def _record_hits(self, hits):
@@ -1354,27 +1443,6 @@ class Recorder(object):
 
         return response['message']
 
-    @staticmethod
-    def invalidate_reports():
-        if config.options.dry_run or not stats.dates_recorded:
-            return
-
-        if config.options.invalidate_dates is not None:
-            dates = [date for date in config.options.invalidate_dates.split(',') if date]
-        else:
-            dates = [date.strftime('%Y-%m-%d') for date in stats.dates_recorded]
-        if dates:
-            print '\nPurging Piwik archives for dates: ' + ' '.join(dates)
-            result = piwik.call_api(
-                'CoreAdminHome.invalidateArchivedReports',
-                dates=','.join(dates),
-                idSites=','.join(str(site_id) for site_id in stats.piwik_sites),
-            )
-            print('\nTo re-process these reports with your newly imported data, execute the following command: \n'
-                  '$ /path/to/piwik/console core:archive --url=http://example/piwik/\n'
-                  '\nReference: http://piwik.org/docs/setup-auto-archiving/ ')
-
-
 class Hit(object):
     """
     It's a simple container.
@@ -1387,6 +1455,16 @@ class Hit(object):
         if config.options.force_lowercase_path:
             self.full_path = self.full_path.lower()
 
+    def get_visitor_id_hash(self):
+        visitor_id = self.ip
+
+        if config.options.replay_tracking:
+            for param_name_to_use in ['uid', 'cid', '_id', 'cip']:
+                if param_name_to_use in self.args:
+                    visitor_id = self.args[param_name_to_use]
+                    break
+
+        return abs(hash(visitor_id))
 
 class Parser(object):
     """
@@ -1399,7 +1477,6 @@ class Parser(object):
                               in inspect.getmembers(self, predicate=inspect.ismethod)
                               if name.startswith('check_')]
 
-
     ## All check_* methods are called for each hit and must return True if the
     ## hit can be imported, False otherwise.
 
@@ -1428,9 +1505,15 @@ class Parser(object):
         return True
 
     def check_download(self, hit):
-        if hit.extension in DOWNLOAD_EXTENSIONS:
+        if hit.extension in config.options.download_extensions:
             stats.count_lines_downloads.increment()
             hit.is_download = True
+            return True
+        # the file is not in the white-listed downloads
+        # if it's a know download file, we shall skip it
+        elif hit.extension in DOWNLOAD_EXTENSIONS:
+            stats.count_lines_skipped_downloads.increment()
+            return False
         return True
 
     def check_user_agent(self, hit):
@@ -1490,7 +1573,8 @@ class Parser(object):
                     match = candidate_format.check_format_line(lineOrFile)
                 else:
                     match = candidate_format.check_format(lineOrFile)
-            except:
+            except Exception, e:
+                logging.debug('Error in format checking: %s', str(e))
                 pass
 
             if match:
@@ -1503,12 +1587,17 @@ class Parser(object):
                     if format_groups < match_groups:
                         format = candidate_format
                         format_groups = match_groups
-                except AttributeError: 
+                except AttributeError:
                     format = candidate_format
 
             else:
                 logging.debug('Format %s does not match', name)
-        
+
+        # if the format is W3cExtendedFormat, check if the logs are from IIS and if so, issue a warning if the
+        # --w3c-time-taken-milli option isn't set
+        if isinstance(format, W3cExtendedFormat):
+            format.check_for_iis_option()
+
         return format
 
     @staticmethod
@@ -1520,7 +1609,7 @@ class Parser(object):
 
         format = False
 
-        # check the format using the file (for formats like the IIS one)
+        # check the format using the file (for formats like the W3cExtendedFormat one)
         format = Parser.check_format(file)
 
         # check the format using the first N lines (to avoid irregular ones)
@@ -1528,6 +1617,9 @@ class Parser(object):
         limit = 100000
         while not format and lineno < limit:
             line = file.readline()
+            if not line: # if at eof, don't keep looping
+                break
+
             lineno = lineno + 1
 
             logging.debug("Detecting format against line %i" % lineno)
@@ -1560,7 +1652,7 @@ class Parser(object):
             file = sys.stdin
         else:
             if not os.path.exists(filename):
-                print >> sys.stderr, 'File %s does not exist' % filename
+                print >> sys.stderr, "\n=====> Warning: File %s does not exist <=====" % filename
                 return
             else:
                 if filename.endswith('.bz2'):
@@ -1597,10 +1689,6 @@ class Parser(object):
         resolver.check_format(format)
 
         hits = []
-        try:
-            cache_dates = OrderedDict()
-        except NameError:
-            cache_dates = None
         for lineno, line in enumerate(file):
             try:
                 line = line.decode(config.options.encoding)
@@ -1626,7 +1714,6 @@ class Parser(object):
                 is_robot=False,
                 is_error=False,
                 is_redirect=False,
-                date=None,
                 args={},
             )
 
@@ -1636,10 +1723,17 @@ class Parser(object):
             except BaseFormatException:
                 hit.path, _, hit.query_string = hit.full_path.partition(config.options.query_string_delimiter)
 
+            # W3cExtendedFormat detaults to - when there is no query string, but we want empty string
+            if hit.query_string == '-':
+                hit.query_string = ''
+
             hit.extension = hit.path.rsplit('.')[-1].lower()
 
             try:
                 hit.referrer = format.get('referrer')
+
+                if hit.referrer.startswith('"'):
+                    hit.referrer = hit.referrer[1:-1]
             except BaseFormatException:
                 hit.referrer = ''
             if hit.referrer == '-':
@@ -1647,6 +1741,11 @@ class Parser(object):
 
             try:
                 hit.user_agent = format.get('user_agent')
+
+                # in case a format parser included enclosing quotes, remove them so they are not
+                # sent to Piwik
+                if hit.user_agent.startswith('"'):
+                    hit.user_agent = hit.user_agent[1:-1]
             except BaseFormatException:
                 hit.user_agent = ''
 
@@ -1654,26 +1753,55 @@ class Parser(object):
             try:
                 hit.length = int(format.get('length'))
             except (ValueError, BaseFormatException):
-                # Some lines or formats don't have a length (e.g. 304 redirects, IIS logs)
+                # Some lines or formats don't have a length (e.g. 304 redirects, W3C logs)
                 hit.length = 0
 
             try:
-                hit.generation_time_milli = int(format.get('generation_time_milli'))
+                hit.generation_time_milli = float(format.get('generation_time_milli'))
             except BaseFormatException:
                 try:
-                    hit.generation_time_milli = int(format.get('generation_time_micro')) / 1000
+                    hit.generation_time_milli = float(format.get('generation_time_micro')) / 1000
                 except BaseFormatException:
-                    hit.generation_time_milli = 0
+                    try:
+                        hit.generation_time_milli = float(format.get('generation_time_secs')) * 1000
+                    except BaseFormatException:
+                        hit.generation_time_milli = 0
 
             if config.options.log_hostname:
                 hit.host = config.options.log_hostname
             else:
                 try:
                     hit.host = format.get('host').lower().strip('.')
+
+                    if hit.host.startswith('"'):
+                        hit.host = hit.host[1:-1]
                 except BaseFormatException:
                     # Some formats have no host.
                     pass
 
+            # Add userid
+            try:
+                hit.userid = None
+
+                userid = format.get('userid')
+                if userid != '-':
+                    hit.args['uid'] = userid
+            except:
+                pass
+
+            # add event info
+            try:
+                hit.event_category = hit.event_action = hit.event_name = None
+
+                hit.event_category = format.get('event_category')
+                hit.event_action = format.get('event_action')
+
+                hit.event_name = format.get('event_name')
+                if hit.event_name == '-':
+                    hit.event_name = None
+            except:
+                pass
+
             # Check if the hit must be excluded.
             if not all((method(hit) for method in self.check_methods)):
                 continue
@@ -1681,38 +1809,24 @@ class Parser(object):
             # Parse date.
             # We parse it after calling check_methods as it's quite CPU hungry, and
             # we want to avoid that cost for excluded hits.
-            if cache_dates is not None:
-                # To mitigate CPU usage, parsed dates are cached.
-                try:
-                    timezone_key = format.get('timezone')
-                except BaseFormatException:
-                    timezone_key = ''
-                date_key = (format.get('date'), timezone_key)
-                hit.date = cache_dates.get(date_key)
-            if not hit.date:
-                date_string = format.get('date')
-                try:
-                    hit.date = datetime.datetime.strptime(date_string, format.date_format)
-                except ValueError:
-                    invalid_line(line, 'invalid date')
-                    continue
+            date_string = format.get('date')
+            try:
+                hit.date = datetime.datetime.strptime(date_string, format.date_format)
+            except ValueError:
+                invalid_line(line, 'invalid date')
+                continue
 
-                # Parse timezone and substract its value from the date
-                try:
-                    timezone = float(format.get('timezone'))
-                except BaseFormatException:
-                    timezone = 0
-                except ValueError:
-                    invalid_line(line, 'invalid timezone')
-                    continue
+            # Parse timezone and substract its value from the date
+            try:
+                timezone = float(format.get('timezone'))
+            except BaseFormatException:
+                timezone = 0
+            except ValueError:
+                invalid_line(line, 'invalid timezone')
+                continue
 
-                if timezone:
-                    hit.date -= datetime.timedelta(hours=timezone/100)
-
-                if cache_dates is not None:
-                    if len(cache_dates) > 3600:
-                        cache_dates.popitem(False)
-                    cache_dates[date_key] = hit.date
+            if timezone:
+                hit.date -= datetime.timedelta(hours=timezone/100)
 
             if config.options.replay_tracking:
                 # we need a query string and we only consider requests with piwik.php
@@ -1741,9 +1855,6 @@ class Parser(object):
         if len(hits) > 0:
             Recorder.add_hits(hits)
 
-
-
-
 def main():
     """
     Start the importing process.
@@ -1768,14 +1879,8 @@ def main():
     if config.options.show_progress:
         stats.stop_monitor()
 
-    try:
-        Recorder.invalidate_reports()
-    except Piwik.Error, e:
-        pass
     stats.print_summary()
 
-
-
 def fatal_error(error, filename=None, lineno=None):
     print >> sys.stderr, 'Fatal error: %s' % error
     if filename and lineno is not None:
@@ -1785,7 +1890,6 @@ def fatal_error(error, filename=None, lineno=None):
         )
     os._exit(1)
 
-
 if __name__ == '__main__':
     try:
         piwik = Piwik()
