@@ -10,26 +10,27 @@ namespace Piwik\Plugins\API;
 
 use Piwik\API\Proxy;
 use Piwik\API\Request;
+use Piwik\Columns\Dimension;
 use Piwik\Config;
 use Piwik\DataTable;
 use Piwik\DataTable\Filter\ColumnDelete;
 use Piwik\DataTable\Row;
 use Piwik\Date;
 use Piwik\IP;
-use Piwik\Menu\MenuTop;
 use Piwik\Metrics;
 use Piwik\Period;
 use Piwik\Period\Range;
 use Piwik\Piwik;
+use Piwik\Plugin\Dimension\VisitDimension;
 use Piwik\Plugins\CoreAdminHome\CustomLogo;
-use Piwik\Tracker\GoalManager;
+use Piwik\Segment\SegmentExpression;
 use Piwik\Translate;
 use Piwik\Version;
 
 require_once PIWIK_INCLUDE_PATH . '/core/Config.php';
 
 /**
- * This API is the <a href='http://piwik.org/docs/analytics-api/metadata/' target='_blank'>Metadata API</a>: it gives information about all other available APIs methods, as well as providing
+ * This API is the <a href='http://piwik.org/docs/analytics-api/metadata/' rel='noreferrer' target='_blank'>Metadata API</a>: it gives information about all other available APIs methods, as well as providing
  * human readable and more complete outputs than normal API methods.
  *
  * Some of the information that is returned by the Metadata API:
@@ -42,7 +43,7 @@ require_once PIWIK_INCLUDE_PATH . '/core/Config.php';
  * <li>the method "getSuggestedValuesForSegment" returns top suggested values for a particular segment. It uses the Live.getLastVisitsDetails API to fetch the most recently used values, and will return the most often used values first.</li>
  * </ul>
  * The Metadata API is for example used by the Piwik Mobile App to automatically display all Piwik reports, with translated report & columns names and nicely formatted values.
- * More information on the <a href='http://piwik.org/docs/analytics-api/metadata/' target='_blank'>Metadata API documentation page</a>
+ * More information on the <a href='http://piwik.org/docs/analytics-api/metadata/' rel='noreferrer' target='_blank'>Metadata API documentation page</a>
  *
  * @method static \Piwik\Plugins\API\API getInstance()
  */
@@ -86,7 +87,7 @@ class API extends \Piwik\Plugin\API
      * translated here.
      * @return array
      */
-    static public function getDefaultMetricTranslations()
+    public static function getDefaultMetricTranslations()
     {
         return Metrics::getDefaultMetricTranslations();
     }
@@ -95,13 +96,19 @@ class API extends \Piwik\Plugin\API
     {
         $segments = array();
 
+        foreach (Dimension::getAllDimensions() as $dimension) {
+            foreach ($dimension->getSegments() as $segment) {
+                $segments[] = $segment->toArray();
+            }
+        }
+
         /**
          * Triggered when gathering all available segment dimensions.
-         * 
+         *
          * This event can be used to make new segment dimensions available.
-         * 
+         *
          * **Example**
-         * 
+         *
          *     public function getSegmentsMetadata(&$segments, $idSites)
          *     {
          *         $segments[] = array(
@@ -115,11 +122,11 @@ class API extends \Piwik\Plugin\API
          *             'permission'     => $isAuthenticatedWithViewAccess,
          *         );
          *     }
-         * 
+         *
          * @param array &$dimensions The list of available segment dimensions. Append to this list to add
          *                           new segments. Each element in this list must contain the
          *                           following information:
-         *                         
+         *
          *                           - **type**: Either `'metric'` or `'dimension'`. `'metric'` means
          *                                       the value is a numeric and `'dimension'` means it is
          *                                       a string. Also, `'metric'` values will be displayed
@@ -145,6 +152,21 @@ class API extends \Piwik\Plugin\API
 
         $isAuthenticatedWithViewAccess = Piwik::isUserHasViewAccess($idSites) && !Piwik::isUserIsAnonymous();
 
+        $segments[] = array(
+            'type'           => 'dimension',
+            'category'       => Piwik::translate('General_Visit'),
+            'name'           => 'General_UserId',
+            'segment'        => 'userId',
+            'acceptedValues' => 'any non empty unique string identifying the user (such as an email address or a username).',
+            'sqlSegment'     => 'log_visit.idvisitor',
+            'sqlFilterValue' => array('Piwik\Common', 'convertUserIdToVisitorIdBin'),
+            'sqlFilter'      => array($this, 'checkSegmentMatchTypeIsValidForUser'),
+            'permission'     => $isAuthenticatedWithViewAccess,
+
+            // TODO specify that this segment is not compatible with some operators
+//            'unsupportedOperators' = array(MATCH_CONTAINS, MATCH_DOES_NOT_CONTAIN),
+        );
+
         $segments[] = array(
             'type'           => 'dimension',
             'category'       => Piwik::translate('General_Visit'),
@@ -155,6 +177,7 @@ class API extends \Piwik\Plugin\API
             'sqlFilterValue' => array('Piwik\Common', 'convertVisitorIdToBin'),
             'permission'     => $isAuthenticatedWithViewAccess,
         );
+
         $segments[] = array(
             'type'           => 'dimension',
             'category'       => Piwik::translate('General_Visit'),
@@ -175,88 +198,6 @@ class API extends \Piwik\Plugin\API
             'sqlFilterValue' => array('Piwik\IP', 'P2N'),
             'permission'     => $isAuthenticatedWithViewAccess,
         );
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_NbActions',
-            'segment'    => 'actions',
-            'sqlSegment' => 'log_visit.visit_total_actions',
-        );
-        $segments[] = array(
-            'type'           => 'metric',
-            'category'       => Piwik::translate('General_Visit'),
-            'name'           => 'General_NbSearches',
-            'segment'        => 'searches',
-            'sqlSegment'     => 'log_visit.visit_total_searches',
-            'acceptedValues' => 'To select all visits who used internal Site Search, use: &segment=searches>0',
-        );
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_ColumnVisitDuration',
-            'segment'    => 'visitDuration',
-            'sqlSegment' => 'log_visit.visit_total_time',
-        );
-        $segments[] = array(
-            'type'           => 'dimension',
-            'category'       => Piwik::translate('General_Visit'),
-            'name'           => Piwik::translate('General_VisitType'),
-            'segment'        => 'visitorType',
-            'acceptedValues' => 'new, returning, returningCustomer' . ". " . Piwik::translate('General_VisitTypeExample', '"&segment=visitorType==returning,visitorType==returningCustomer"'),
-            'sqlSegment'     => 'log_visit.visitor_returning',
-            'sqlFilterValue' => function ($type) {
-                    return $type == "new" ? 0 : ($type == "returning" ? 1 : 2);
-                }
-        );
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_DaysSinceLastVisit',
-            'segment'    => 'daysSinceLastVisit',
-            'sqlSegment' => 'log_visit.visitor_days_since_last',
-        );
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_DaysSinceFirstVisit',
-            'segment'    => 'daysSinceFirstVisit',
-            'sqlSegment' => 'log_visit.visitor_days_since_first',
-        );
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_NumberOfVisits',
-            'segment'    => 'visitCount',
-            'sqlSegment' => 'log_visit.visitor_count_visits',
-        );
-
-        $segments[] = array(
-            'type'           => 'dimension',
-            'category'       => Piwik::translate('General_Visit'),
-            'name'           => 'General_VisitConvertedGoal',
-            'segment'        => 'visitConverted',
-            'acceptedValues' => '0, 1',
-            'sqlSegment'     => 'log_visit.visit_goal_converted',
-        );
-
-        $segments[] = array(
-            'type'           => 'dimension',
-            'category'       => Piwik::translate('General_Visit'),
-            'name'           => Piwik::translate('General_EcommerceVisitStatusDesc'),
-            'segment'        => 'visitEcommerceStatus',
-            'acceptedValues' => implode(", ", self::$visitEcommerceStatus)
-                . '. ' . Piwik::translate('General_EcommerceVisitStatusEg', '"&segment=visitEcommerceStatus==ordered,visitEcommerceStatus==orderedThenAbandonedCart"'),
-            'sqlSegment'     => 'log_visit.visit_goal_buyer',
-            'sqlFilterValue' => __NAMESPACE__ . '\API::getVisitEcommerceStatus',
-        );
-
-        $segments[] = array(
-            'type'       => 'metric',
-            'category'   => Piwik::translate('General_Visit'),
-            'name'       => 'General_DaysSinceLastEcommerceOrder',
-            'segment'    => 'daysSinceLastEcommerceOrder',
-            'sqlSegment' => 'log_visit.visitor_days_since_order',
-        );
 
         foreach ($segments as &$segment) {
             $segment['name'] = Piwik::translate($segment['name']);
@@ -273,47 +214,20 @@ class API extends \Piwik\Plugin\API
         return $segments;
     }
 
-    static protected $visitEcommerceStatus = array(
-        GoalManager::TYPE_BUYER_NONE                  => 'none',
-        GoalManager::TYPE_BUYER_ORDERED               => 'ordered',
-        GoalManager::TYPE_BUYER_OPEN_CART             => 'abandonedCart',
-        GoalManager::TYPE_BUYER_ORDERED_AND_OPEN_CART => 'orderedThenAbandonedCart',
-    );
-
-    /**
-     * @ignore
-     */
-    static public function getVisitEcommerceStatusFromId($id)
-    {
-        if (!isset(self::$visitEcommerceStatus[$id])) {
-            throw new \Exception("Unexpected ECommerce status value ");
-        }
-        return self::$visitEcommerceStatus[$id];
-    }
-
-    /**
-     * @ignore
-     */
-    static public function getVisitEcommerceStatus($status)
-    {
-        $id = array_search($status, self::$visitEcommerceStatus);
-        if ($id === false) {
-            throw new \Exception("Invalid 'visitEcommerceStatus' segment value $status");
-        }
-        return $id;
-    }
-
     private function sortSegments($row1, $row2)
     {
+        $customVarCategory = Piwik::translate('CustomVariables_CustomVariables');
+
         $columns = array('type', 'category', 'name', 'segment');
         foreach ($columns as $column) {
             // Keep segments ordered alphabetically inside categories..
             $type = -1;
             if ($column == 'name') $type = 1;
+
             $compare = $type * strcmp($row1[$column], $row2[$column]);
 
             // hack so that custom variables "page" are grouped together in the doc
-            if ($row1['category'] == Piwik::translate('CustomVariables_CustomVariables')
+            if ($row1['category'] == $customVarCategory
                 && $row1['category'] == $row2['category']
             ) {
                 $compare = strcmp($row1['segment'], $row2['segment']);
@@ -326,6 +240,31 @@ class API extends \Piwik\Plugin\API
         return $compare;
     }
 
+    /**
+     * Throw an exception if the User ID segment is used with an un-supported match type,
+     *
+     * @ignore
+     * @param $value
+     * @param $sqlSegment
+     * @param $matchType
+     * @param $name
+     * @return $value
+     * @throws \Exception
+     */
+    public function checkSegmentMatchTypeIsValidForUser($value, $sqlSegment, $matchType, $name)
+    {
+        $acceptedMatches = array(
+            SegmentExpression::MATCH_EQUAL,
+            SegmentExpression::MATCH_IS_NOT_NULL_NOR_EMPTY,
+            SegmentExpression::MATCH_IS_NULL_OR_EMPTY,
+            SegmentExpression::MATCH_NOT_EQUAL,
+        );
+        if (in_array($matchType, $acceptedMatches)) {
+            return $value;
+        }
+        $message = "Invalid Segment match type: try using 'userId' segment with one of the following match types: %s.";
+        throw new \Exception(sprintf($message, implode(", ", $acceptedMatches)));
+    }
 
     /**
      * Returns the url to application logo (~280x110px)
@@ -364,7 +303,6 @@ class API extends \Piwik\Plugin\API
         return $logo->getSVGLogoUrl($pathOnly);
     }
 
-
     /**
      * Returns whether there is an SVG Logo available.
      * @ignore
@@ -376,7 +314,6 @@ class API extends \Piwik\Plugin\API
         return $logo->hasSVGLogo();
     }
 
-
     /**
      * Loads reports metadata, then return the requested one,
      * matching optional API parameters.
@@ -444,7 +381,8 @@ class API extends \Piwik\Plugin\API
                 && !empty($reportMeta['metrics'])
             ) {
                 $plugin = $reportMeta['module'];
-                foreach ($reportMeta['metrics'] as $column => $columnTranslation) {
+                $allMetrics = array_merge($reportMeta['metrics'], @$reportMeta['processedMetrics'] ?: array());
+                foreach ($allMetrics as $column => $columnTranslation) {
                     // a metric from this report has been requested
                     if (isset($columnsMap[$column])
                         // or by default, return all metrics
@@ -464,24 +402,10 @@ class API extends \Piwik\Plugin\API
             $className = Request::getClassNameAPI($plugin);
             $params['columns'] = implode(',', $columns);
             $dataTable = Proxy::getInstance()->call($className, 'get', $params);
-            
-            // make sure the table has all columns
-            $array = ($dataTable instanceof DataTable\Map ? $dataTable->getDataTables() : array($dataTable));
-            foreach ($array as $table) {
-                // we don't support idSites=all&date=DATE1,DATE2
-                if ($table instanceof DataTable) {
-                    $firstRow = $table->getFirstRow();
-                    if (!$firstRow) {
-                        $firstRow = new Row;
-                        $table->addRow($firstRow);
-                    }
-                    foreach ($columns as $column) {
-                        if ($firstRow->getColumn($column) === false) {
-                            $firstRow->setColumn($column, 0);
-                        }
-                    }
-                }
-            }
+
+            $dataTable->filter(function (DataTable $table) {
+                $table->clearQueuedFilters();
+            });
 
             // merge reports
             if ($mergedDataTable === false) {
@@ -490,6 +414,13 @@ class API extends \Piwik\Plugin\API
                 $this->mergeDataTables($mergedDataTable, $dataTable);
             }
         }
+
+        if (!empty($columnsMap)
+            && !empty($mergedDataTable)
+        ) {
+            $mergedDataTable->queueFilter('ColumnDelete', array(false, array_keys($columnsMap)));
+        }
+
         return $mergedDataTable;
     }
 
@@ -503,18 +434,28 @@ class API extends \Piwik\Plugin\API
         if ($table1 instanceof DataTable\Map && $table2 instanceof DataTable\Map) {
             $subTables2 = $table2->getDataTables();
             foreach ($table1->getDataTables() as $index => $subTable1) {
+                if (!array_key_exists($index, $subTables2)) {
+                    // occurs when archiving starts on dayN and continues into dayN+1, see https://github.com/piwik/piwik/issues/5168#issuecomment-50959925
+                    continue;
+                }
                 $subTable2 = $subTables2[$index];
                 $this->mergeDataTables($subTable1, $subTable2);
             }
             return;
         }
 
-        $firstRow1 = $table1->getFirstRow();
         $firstRow2 = $table2->getFirstRow();
-        if ($firstRow2 instanceof Row) {
-            foreach ($firstRow2->getColumns() as $metric => $value) {
-                $firstRow1->setColumn($metric, $value);
-            }
+        if (!($firstRow2 instanceof Row)) {
+            return;
+        }
+
+        $firstRow1 = $table1->getFirstRow();
+        if (empty($firstRow1)) {
+            $firstRow1 = $table1->addRow(new Row());
+        }
+
+        foreach ($firstRow2->getColumns() as $metric => $value) {
+            $firstRow1->setColumn($metric, $value);
         }
     }
 
@@ -574,6 +515,18 @@ class API extends \Piwik\Plugin\API
         return $result;
     }
 
+    /**
+     * Return true if plugin is activated, false otherwise
+     *
+     * @param string $pluginName
+     * @return bool
+     */
+    public function isPluginActivated($pluginName)
+    {
+        Piwik::checkUserHasSomeViewAccess();
+        return \Piwik\Plugin\Manager::getInstance()->isPluginActivated($pluginName);
+    }
+
     /**
      * Given a segment, will return a list of the most used values for this particular segment.
      * @param $segmentName
@@ -583,7 +536,7 @@ class API extends \Piwik\Plugin\API
      */
     public function getSuggestedValuesForSegment($segmentName, $idSite)
     {
-        if(empty(Config::getInstance()->General['enable_segment_suggested_values'])) {
+        if (empty(Config::getInstance()->General['enable_segment_suggested_values'])) {
             return array();
         }
         Piwik::checkUserHasViewAccess($idSite);
@@ -602,8 +555,13 @@ class API extends \Piwik\Plugin\API
             throw new \Exception("Requested segment not found.");
         }
 
+        // if segment has suggested values callback then return result from it instead
+        if (isset($segmentFound['suggestedValuesCallback'])) {
+            return call_user_func($segmentFound['suggestedValuesCallback'], $idSite, $maxSuggestionsToReturn);
+        }
+
         // if period=range is disabled, do not proceed
-        if(!Period\Factory::isPeriodEnabledForAPI('range')) {
+        if (!Period\Factory::isPeriodEnabledForAPI('range')) {
             return array();
         }
 
@@ -662,7 +620,8 @@ class API extends \Piwik\Plugin\API
                                          'entryPageTitle', 'entryPageUrl', 'exitPageTitle', 'exitPageUrl');
         $isCustomVariablePage = stripos($segmentName, 'customVariablePage') !== false;
         $isEventSegment = stripos($segmentName, 'event') !== false;
-        $doesSegmentNeedActionsInfo = in_array($segmentName, $segmentsNeedActionsInfo) || $isCustomVariablePage || $isEventSegment;
+        $isContentSegment = stripos($segmentName, 'content') !== false;
+        $doesSegmentNeedActionsInfo = in_array($segmentName, $segmentsNeedActionsInfo) || $isCustomVariablePage || $isEventSegment || $isContentSegment;
         return $doesSegmentNeedActionsInfo;
     }
 
